@@ -39,6 +39,8 @@ import org.apache.helix.model.InstanceConfig;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
 import org.apache.helix.zookeeper.datamodel.ZNRecord;
 import org.apache.pinot.broker.broker.helix.ClusterChangeHandler;
+import org.apache.pinot.broker.routing.adaptiveserverselector.AdaptiveServerSelector;
+import org.apache.pinot.broker.routing.adaptiveserverselector.AdaptiveServerSelectorFactory;
 import org.apache.pinot.broker.routing.instanceselector.InstanceSelector;
 import org.apache.pinot.broker.routing.instanceselector.InstanceSelectorFactory;
 import org.apache.pinot.broker.routing.segmentpreselector.SegmentPreSelector;
@@ -47,7 +49,6 @@ import org.apache.pinot.broker.routing.segmentpruner.SegmentPruner;
 import org.apache.pinot.broker.routing.segmentpruner.SegmentPrunerFactory;
 import org.apache.pinot.broker.routing.segmentselector.SegmentSelector;
 import org.apache.pinot.broker.routing.segmentselector.SegmentSelectorFactory;
-import org.apache.pinot.broker.routing.timeboundary.TimeBoundaryInfo;
 import org.apache.pinot.broker.routing.timeboundary.TimeBoundaryManager;
 import org.apache.pinot.common.metadata.ZKMetadataProvider;
 import org.apache.pinot.common.metrics.BrokerMeter;
@@ -56,9 +57,12 @@ import org.apache.pinot.common.request.BrokerRequest;
 import org.apache.pinot.common.utils.HashUtil;
 import org.apache.pinot.core.routing.RoutingManager;
 import org.apache.pinot.core.routing.RoutingTable;
+import org.apache.pinot.core.routing.TimeBoundaryInfo;
 import org.apache.pinot.core.transport.ServerInstance;
+import org.apache.pinot.core.transport.server.routing.stats.ServerRoutingStatsManager;
 import org.apache.pinot.spi.config.table.QueryConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.CommonConstants.Helix;
 import org.apache.pinot.spi.utils.CommonConstants.Helix.StateModel.SegmentStateModel;
 import org.apache.pinot.spi.utils.InstanceTypeUtils;
@@ -75,9 +79,9 @@ import org.slf4j.LoggerFactory;
  * <ul>
  *   <li>{@link #buildRouting(String)}: Builds/rebuilds the routing for a table</li>
  *   <li>{@link #removeRouting(String)}: Removes the routing for a table</li>
- *   <li>{@link #refreshSegment(String, String): Refreshes the metadata for a segment}</li>
+ *   <li>{@link #refreshSegment(String, String)}: Refreshes the metadata for a segment</li>
  *   <li>{@link #routingExists(String)}: Returns whether the routing exists for a table</li>
- *   <li>{@link #getRoutingTable(BrokerRequest)}: Returns the routing table for a query</li>
+ *   <li>{@link #getRoutingTable(BrokerRequest, long)}: Returns the routing table for a query</li>
  *   <li>{@link #getTimeBoundaryInfo(String)}: Returns the time boundary info for a table</li>
  *   <li>{@link #getQueryTimeoutMs(String)}: Returns the table-level query timeout in milliseconds for a table</li>
  * </ul>
@@ -93,6 +97,8 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
   private final Map<String, ServerInstance> _enabledServerInstanceMap = new ConcurrentHashMap<>();
   // NOTE: _excludedServers doesn't need to be concurrent because it is only accessed within the synchronized block
   private final Set<String> _excludedServers = new HashSet<>();
+  private final ServerRoutingStatsManager _serverRoutingStatsManager;
+  private final PinotConfiguration _pinotConfig;
 
   private BaseDataAccessor<ZNRecord> _zkDataAccessor;
   private String _externalViewPathPrefix;
@@ -102,8 +108,11 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
 
   private Set<String> _routableServers;
 
-  public BrokerRoutingManager(BrokerMetrics brokerMetrics) {
+  public BrokerRoutingManager(BrokerMetrics brokerMetrics, ServerRoutingStatsManager serverRoutingStatsManager,
+      PinotConfiguration pinotConfig) {
     _brokerMetrics = brokerMetrics;
+    _serverRoutingStatsManager = serverRoutingStatsManager;
+    _pinotConfig = pinotConfig;
   }
 
   @Override
@@ -425,7 +434,11 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
     for (SegmentPruner segmentPruner : segmentPruners) {
       segmentPruner.init(idealState, externalView, preSelectedOnlineSegments);
     }
-    InstanceSelector instanceSelector = InstanceSelectorFactory.getInstanceSelector(tableConfig, _brokerMetrics);
+    AdaptiveServerSelector adaptiveServerSelector =
+        AdaptiveServerSelectorFactory.getAdaptiveServerSelector(_serverRoutingStatsManager, _pinotConfig);
+    InstanceSelector instanceSelector =
+        InstanceSelectorFactory.getInstanceSelector(tableConfig, _propertyStore, _brokerMetrics,
+            adaptiveServerSelector);
     instanceSelector.init(_routableServers, idealState, externalView, preSelectedOnlineSegments);
 
     // Add time boundary manager if both offline and real-time part exist for a hybrid table
@@ -436,7 +449,7 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
       String realtimeTableName = TableNameBuilder.REALTIME.tableNameWithType(rawTableName);
       if (_routingEntryMap.containsKey(realtimeTableName)) {
         LOGGER.info("Adding time boundary manager for table: {}", tableNameWithType);
-        timeBoundaryManager = new TimeBoundaryManager(tableConfig, _propertyStore);
+        timeBoundaryManager = new TimeBoundaryManager(tableConfig, _propertyStore, _brokerMetrics);
         timeBoundaryManager.init(idealState, externalView, preSelectedOnlineSegments);
       }
     } else {
@@ -465,7 +478,7 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
         Set<String> offlineTablePreSelectedOnlineSegments =
             offlineTableSegmentPreSelector.preSelect(offlineTableOnlineSegments);
         TimeBoundaryManager offlineTableTimeBoundaryManager =
-            new TimeBoundaryManager(offlineTableConfig, _propertyStore);
+            new TimeBoundaryManager(offlineTableConfig, _propertyStore, _brokerMetrics);
         offlineTableTimeBoundaryManager.init(offlineTableIdealState, offlineTableExternalView,
             offlineTablePreSelectedOnlineSegments);
         offlineTableRoutingEntry.setTimeBoundaryManager(offlineTableTimeBoundaryManager);
@@ -555,13 +568,13 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
    */
   @Nullable
   @Override
-  public RoutingTable getRoutingTable(BrokerRequest brokerRequest) {
+  public RoutingTable getRoutingTable(BrokerRequest brokerRequest, long requestId) {
     String tableNameWithType = brokerRequest.getQuerySource().getTableName();
     RoutingEntry routingEntry = _routingEntryMap.get(tableNameWithType);
     if (routingEntry == null) {
       return null;
     }
-    InstanceSelector.SelectionResult selectionResult = routingEntry.calculateRouting(brokerRequest);
+    InstanceSelector.SelectionResult selectionResult = routingEntry.calculateRouting(brokerRequest, requestId);
     Map<String, String> segmentToInstanceMap = selectionResult.getSegmentToInstanceMap();
     Map<ServerInstance, List<String>> serverInstanceToSegmentsMap = new HashMap<>();
     for (Map.Entry<String, String> entry : segmentToInstanceMap.entrySet()) {
@@ -596,6 +609,7 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
    * <p>NOTE: Time boundary info is only available for the offline part of the hybrid table.
    */
   @Nullable
+  @Override
   public TimeBoundaryInfo getTimeBoundaryInfo(String offlineTableName) {
     RoutingEntry routingEntry = _routingEntryMap.get(offlineTableName);
     if (routingEntry == null) {
@@ -704,7 +718,7 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
       }
     }
 
-    InstanceSelector.SelectionResult calculateRouting(BrokerRequest brokerRequest) {
+    InstanceSelector.SelectionResult calculateRouting(BrokerRequest brokerRequest, long requestId) {
       Set<String> selectedSegments = _segmentSelector.select(brokerRequest);
       int numTotalSelectedSegments = selectedSegments.size();
       if (!selectedSegments.isEmpty()) {
@@ -715,7 +729,7 @@ public class BrokerRoutingManager implements RoutingManager, ClusterChangeHandle
       int numPrunedSegments = numTotalSelectedSegments - selectedSegments.size();
       if (!selectedSegments.isEmpty()) {
         InstanceSelector.SelectionResult selectionResult = _instanceSelector.select(brokerRequest,
-            new ArrayList<>(selectedSegments));
+            new ArrayList<>(selectedSegments), requestId);
         selectionResult.setNumPrunedSegments(numPrunedSegments);
         return selectionResult;
       } else {
