@@ -18,20 +18,21 @@
  */
 package org.apache.pinot.connector.spark.connector
 
-import java.util.{List => JList, Map => JMap}
-
 import org.apache.helix.model.InstanceConfig
+import org.apache.pinot.common.datatable.DataTable
 import org.apache.pinot.common.metrics.BrokerMetrics
 import org.apache.pinot.common.request.BrokerRequest
-import org.apache.pinot.common.utils.DataTable
 import org.apache.pinot.connector.spark.datasource.PinotDataSourceReadOptions
 import org.apache.pinot.connector.spark.exceptions.PinotException
 import org.apache.pinot.connector.spark.utils.Logging
+import org.apache.pinot.core.transport.server.routing.stats.ServerRoutingStatsManager
 import org.apache.pinot.core.transport.{AsyncQueryResponse, QueryRouter, ServerInstance}
 import org.apache.pinot.spi.config.table.TableType
+import org.apache.pinot.spi.env.PinotConfiguration
 import org.apache.pinot.spi.metrics.PinotMetricUtils
 import org.apache.pinot.sql.parsers.CalciteSqlCompiler
 
+import java.util.{List => JList, Map => JMap}
 import scala.collection.JavaConverters._
 
 /**
@@ -46,7 +47,9 @@ private[pinot] class PinotServerDataFetcher(
   private val brokerId = "apache_spark"
   private val metricsRegistry = PinotMetricUtils.getPinotMetricsRegistry
   private val brokerMetrics = new BrokerMetrics(metricsRegistry)
-  private val queryRouter = new QueryRouter(brokerId, brokerMetrics)
+  private val pinotConfig = new PinotConfiguration()
+  private val serverRoutingStatsManager = new ServerRoutingStatsManager(pinotConfig)
+  private val queryRouter = new QueryRouter(brokerId, brokerMetrics, serverRoutingStatsManager)
   // TODO add support for TLS-secured server
 
   def fetchData(): List[DataTable] = {
