@@ -18,8 +18,8 @@
  */
 package org.apache.pinot.connector.spark.connector
 
+import org.apache.pinot.common.datatable.DataTable
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType
-import org.apache.pinot.common.utils.DataTable
 import org.apache.pinot.connector.spark.exceptions.PinotException
 import org.apache.pinot.spi.data.{FieldSpec, Schema}
 import org.apache.spark.sql.catalyst.InternalRow
