@@ -20,10 +20,8 @@ package org.apache.pinot.controller.helix.core.assignment.segment;
 
 import com.google.common.base.Preconditions;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 import java.util.TreeMap;
 import javax.annotation.Nullable;
 import org.apache.commons.collections.CollectionUtils;
@@ -31,8 +29,8 @@ import org.apache.commons.configuration.Configuration;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.common.assignment.InstancePartitions;
 import org.apache.pinot.common.tier.Tier;
-import org.apache.pinot.common.utils.SegmentUtils;
-import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.controller.helix.core.assignment.segment.strategy.SegmentAssignmentStrategy;
+import org.apache.pinot.controller.helix.core.assignment.segment.strategy.SegmentAssignmentStrategyFactory;
 import org.apache.pinot.spi.config.table.assignment.InstancePartitionsType;
 import org.apache.pinot.spi.utils.CommonConstants.Helix.StateModel.SegmentStateModel;
 import org.apache.pinot.spi.utils.RebalanceConfigConstants;
@@ -75,11 +73,6 @@ import org.apache.pinot.spi.utils.RebalanceConfigConstants;
  */
 public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
 
-  @Override
-  protected int getReplication(TableConfig tableConfig) {
-    return tableConfig.getValidationConfig().getReplicasPerPartitionNumber();
-  }
-
   @Override
   public List<String> assignSegment(String segmentName, Map<String, Map<String, String>> currentAssignment,
       Map<InstancePartitionsType, InstancePartitions> instancePartitionsMap) {
@@ -90,9 +83,20 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
     InstancePartitions instancePartitions = typeToInstancePartitions.getValue();
     _logger.info("Assigning segment: {} with instance partitions: {} for table: {}", segmentName, instancePartitions,
         _tableNameWithType);
-    List<String> instancesAssigned =
-        instancePartitionsType == InstancePartitionsType.CONSUMING ? assignConsumingSegment(segmentName,
-            instancePartitions) : assignSegment(segmentName, currentAssignment, instancePartitions);
+
+    // TODO: remove this check after we also refactor consuming segments assignment strategy
+    // See https://github.com/apache/pinot/issues/9047
+    List<String> instancesAssigned;
+    if (instancePartitionsType == InstancePartitionsType.COMPLETED) {
+      // Gets Segment assignment strategy for instance partitions
+      SegmentAssignmentStrategy segmentAssignmentStrategy = SegmentAssignmentStrategyFactory
+          .getSegmentAssignmentStrategy(_helixManager, _tableConfig, instancePartitionsType.toString(),
+              instancePartitions);
+      instancesAssigned = segmentAssignmentStrategy
+          .assignSegment(segmentName, currentAssignment, instancePartitions, InstancePartitionsType.COMPLETED);
+    } else {
+      instancesAssigned = assignConsumingSegment(segmentName, instancePartitions);
+    }
     _logger.info("Assigned segment: {} to instances: {} for table: {}", segmentName, instancesAssigned,
         _tableNameWithType);
     return instancesAssigned;
@@ -102,7 +106,8 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
    * Helper method to assign instances for CONSUMING segment based on the segment partition id and instance partitions.
    */
   private List<String> assignConsumingSegment(String segmentName, InstancePartitions instancePartitions) {
-    int segmentPartitionId = getSegmentPartitionId(segmentName);
+    int segmentPartitionId = SegmentAssignmentUtils
+        .getRealtimeSegmentPartitionId(segmentName, _tableNameWithType, _helixManager, _partitionColumn);
     int numReplicaGroups = instancePartitions.getNumReplicaGroups();
     int numPartitions = instancePartitions.getNumPartitions();
 
@@ -125,8 +130,14 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
       return instancesAssigned;
     } else {
       // Replica-group based assignment
-
-      checkReplication(instancePartitions);
+      // TODO: Refactor check replication this for segment assignment strategy in follow up PR
+      // See https://github.com/apache/pinot/issues/9047
+      if (numReplicaGroups != _replication) {
+        _logger.warn(
+            "Number of replica-groups in instance partitions {}: {} does not match replication in table config: {} for "
+                + "table: {}, using: {}", instancePartitions.getInstancePartitionsName(), numReplicaGroups,
+            _replication, _tableNameWithType, numReplicaGroups);
+      }
       List<String> instancesAssigned = new ArrayList<>(numReplicaGroups);
 
       if (numPartitions == 1) {
@@ -155,36 +166,25 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
     }
   }
 
-  @Override
-  protected int getSegmentPartitionId(String segmentName) {
-    Integer segmentPartitionId =
-        SegmentUtils.getRealtimeSegmentPartitionId(segmentName, _tableNameWithType, _helixManager, _partitionColumn);
-    if (segmentPartitionId == null) {
-      // This case is for the uploaded segments for which there's no partition information.
-      // A random, but consistent, partition id is calculated based on the hash code of the segment name.
-      // Note that '% 10K' is used to prevent having partition ids with large value which will be problematic later in
-      // instance assignment formula.
-      segmentPartitionId = Math.abs(segmentName.hashCode() % 10_000);
-    }
-    return segmentPartitionId;
-  }
-
   @Override
   public Map<String, Map<String, String>> rebalanceTable(Map<String, Map<String, String>> currentAssignment,
       Map<InstancePartitionsType, InstancePartitions> instancePartitionsMap, @Nullable List<Tier> sortedTiers,
       @Nullable Map<String, InstancePartitions> tierInstancePartitionsMap, Configuration config) {
     InstancePartitions completedInstancePartitions = instancePartitionsMap.get(InstancePartitionsType.COMPLETED);
     InstancePartitions consumingInstancePartitions = instancePartitionsMap.get(InstancePartitionsType.CONSUMING);
-    Preconditions.checkState(consumingInstancePartitions != null,
-        "Failed to find CONSUMING instance partitions for table: %s", _tableNameWithType);
-    boolean includeConsuming = config.getBoolean(RebalanceConfigConstants.INCLUDE_CONSUMING,
-        RebalanceConfigConstants.DEFAULT_INCLUDE_CONSUMING);
+    Preconditions
+        .checkState(consumingInstancePartitions != null, "Failed to find CONSUMING instance partitions for table: %s",
+            _tableNameWithType);
+    boolean includeConsuming = config
+        .getBoolean(RebalanceConfigConstants.INCLUDE_CONSUMING, RebalanceConfigConstants.DEFAULT_INCLUDE_CONSUMING);
     boolean bootstrap =
         config.getBoolean(RebalanceConfigConstants.BOOTSTRAP, RebalanceConfigConstants.DEFAULT_BOOTSTRAP);
 
     // Rebalance tiers first
     Pair<List<Map<String, Map<String, String>>>, Map<String, Map<String, String>>> pair =
-        rebalanceTiers(currentAssignment, sortedTiers, tierInstancePartitionsMap, bootstrap);
+        rebalanceTiers(currentAssignment, sortedTiers, tierInstancePartitionsMap, bootstrap,
+            InstancePartitionsType.COMPLETED);
+
     List<Map<String, Map<String, String>>> newTierAssignments = pair.getLeft();
     Map<String, Map<String, String>> nonTierAssignment = pair.getRight();
 
@@ -202,10 +202,13 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
     if (completedInstancePartitions != null) {
       // When COMPLETED instance partitions are provided, reassign COMPLETED segments in a balanced way (relocate
       // COMPLETED segments to offload them from CONSUMING instances to COMPLETED instances)
+      SegmentAssignmentStrategy segmentAssignmentStrategy =
+          SegmentAssignmentStrategyFactory.getSegmentAssignmentStrategy(_helixManager, _tableConfig,
+              InstancePartitionsType.COMPLETED.toString(), completedInstancePartitions);
       _logger.info("Reassigning COMPLETED segments with COMPLETED instance partitions for table: {}",
           _tableNameWithType);
       newAssignment = reassignSegments(InstancePartitionsType.COMPLETED.toString(), completedSegmentAssignment,
-          completedInstancePartitions, bootstrap);
+          completedInstancePartitions, bootstrap, segmentAssignmentStrategy, InstancePartitionsType.COMPLETED);
     } else {
       // When COMPLETED instance partitions are not provided, reassign COMPLETED segments the same way as CONSUMING
       // segments with CONSUMING instance partitions (ensure COMPLETED segments are served by the correct instances when
@@ -227,8 +230,8 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
     Map<String, Map<String, String>> consumingSegmentAssignment =
         completedConsumingOfflineSegmentAssignment.getConsumingSegmentAssignment();
     if (includeConsuming) {
-      _logger.info("Reassigning CONSUMING segments with CONSUMING instance partitions for table: {}",
-          _tableNameWithType);
+      _logger
+          .info("Reassigning CONSUMING segments with CONSUMING instance partitions for table: {}", _tableNameWithType);
 
       for (String segmentName : consumingSegmentAssignment.keySet()) {
         List<String> instancesAssigned = assignConsumingSegment(segmentName, consumingInstancePartitions);
@@ -253,15 +256,4 @@ public class RealtimeSegmentAssignment extends BaseSegmentAssignment {
         SegmentAssignmentUtils.getNumSegmentsToBeMovedPerInstance(currentAssignment, newAssignment));
     return newAssignment;
   }
-
-  @Override
-  protected Map<Integer, List<String>> getInstancePartitionIdToSegmentsMap(Set<String> segments,
-      int numInstancePartitions) {
-    Map<Integer, List<String>> instancePartitionIdToSegmentsMap = new HashMap<>();
-    for (String segmentName : segments) {
-      int instancePartitionId = getSegmentPartitionId(segmentName) % numInstancePartitions;
-      instancePartitionIdToSegmentsMap.computeIfAbsent(instancePartitionId, k -> new ArrayList<>()).add(segmentName);
-    }
-    return instancePartitionIdToSegmentsMap;
-  }
 }
