@@ -101,7 +101,7 @@ const defaultTableObj = {
       "stream.kafka.decoder.class.name": "org.apache.pinot.plugin.stream.kafka.KafkaJSONMessageDecoder",
       "realtime.segment.flush.threshold.rows": "0",
       "realtime.segment.flush.threshold.time": "24h",
-      "realtime.segment.flush.segment.size": "100M"
+      "realtime.segment.flush.threshold.segment.size": "100M"
     }
   },
   "metadata": {},
