@@ -34,15 +34,15 @@ import org.apache.pinot.spi.utils.InstanceTypeUtils;
 import org.apache.pinot.spi.utils.JsonUtils;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
-import org.testng.Assert;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.*;
 
-public class TableViewsTest {
-  private static final ControllerTest TEST_INSTANCE = ControllerTest.getInstance();
+
+public class TableViewsTest extends ControllerTest {
   private static final String OFFLINE_TABLE_NAME = "offlineTable";
   private static final String OFFLINE_SEGMENT_NAME = "offlineSegment";
   private static final String HYBRID_TABLE_NAME = "viewsTable";
@@ -50,29 +50,28 @@ public class TableViewsTest {
   @BeforeClass
   public void setUp()
       throws Exception {
-    TEST_INSTANCE.setupSharedStateAndValidate();
+    DEFAULT_INSTANCE.setupSharedStateAndValidate();
 
     // Create the offline table and add one segment
     TableConfig tableConfig =
         new TableConfigBuilder(TableType.OFFLINE).setTableName(OFFLINE_TABLE_NAME).setNumReplicas(2).build();
-    Assert.assertEquals(TEST_INSTANCE.getHelixManager().getInstanceType(), InstanceType.CONTROLLER);
-    TEST_INSTANCE.getHelixResourceManager().addTable(tableConfig);
-    TEST_INSTANCE.getHelixResourceManager()
+    assertEquals(DEFAULT_INSTANCE.getHelixManager().getInstanceType(), InstanceType.CONTROLLER);
+    DEFAULT_INSTANCE.getHelixResourceManager().addTable(tableConfig);
+    DEFAULT_INSTANCE.getHelixResourceManager()
         .addNewSegment(TableNameBuilder.OFFLINE.tableNameWithType(OFFLINE_TABLE_NAME),
             SegmentMetadataMockUtils.mockSegmentMetadata(OFFLINE_TABLE_NAME, OFFLINE_SEGMENT_NAME), "downloadUrl");
 
     // Create the hybrid table
     tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(HYBRID_TABLE_NAME)
-        .setNumReplicas(TEST_INSTANCE.MIN_NUM_REPLICAS).build();
-    TEST_INSTANCE.getHelixResourceManager().addTable(tableConfig);
+        .setNumReplicas(DEFAULT_MIN_NUM_REPLICAS).build();
+    DEFAULT_INSTANCE.getHelixResourceManager().addTable(tableConfig);
 
     // add schema for realtime table
-    TEST_INSTANCE.addDummySchema(HYBRID_TABLE_NAME);
+    DEFAULT_INSTANCE.addDummySchema(HYBRID_TABLE_NAME);
     StreamConfig streamConfig = FakeStreamConfigUtils.getDefaultHighLevelStreamConfigs();
     tableConfig = new TableConfigBuilder(TableType.REALTIME).setTableName(HYBRID_TABLE_NAME)
-        .setNumReplicas(TEST_INSTANCE.MIN_NUM_REPLICAS).setStreamConfigs(streamConfig.getStreamConfigsMap())
-        .build();
-    TEST_INSTANCE.getHelixResourceManager().addTable(tableConfig);
+        .setNumReplicas(DEFAULT_MIN_NUM_REPLICAS).setStreamConfigs(streamConfig.getStreamConfigsMap()).build();
+    DEFAULT_INSTANCE.getHelixResourceManager().addTable(tableConfig);
 
     // Wait for external view get updated
     long endTime = System.currentTimeMillis() + 10_000L;
@@ -92,12 +91,12 @@ public class TableViewsTest {
       if (tableView._offline == null) {
         continue;
       }
-      if ((tableView._realtime == null) || (tableView._realtime.size() != TEST_INSTANCE.NUM_SERVER_INSTANCES)) {
+      if ((tableView._realtime == null) || (tableView._realtime.size() != DEFAULT_NUM_SERVER_INSTANCES)) {
         continue;
       }
       return;
     }
-    Assert.fail("Failed to get external view updated");
+    fail("Failed to get external view updated");
   }
 
   @DataProvider(name = "viewProvider")
@@ -108,34 +107,34 @@ public class TableViewsTest {
   @Test(dataProvider = "viewProvider")
   public void testTableNotFound(String view)
       throws Exception {
-    String url = TEST_INSTANCE.getControllerRequestURLBuilder().forTableView("unknownTable", view, null);
+    String url = DEFAULT_INSTANCE.getControllerRequestURLBuilder().forTableView("unknownTable", view, null);
     HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
-    Assert.assertEquals(connection.getResponseCode(), 404);
+    assertEquals(connection.getResponseCode(), 404);
   }
 
   @Test(dataProvider = "viewProvider")
   public void testBadRequest(String view)
       throws Exception {
     String url =
-        TEST_INSTANCE.getControllerRequestURLBuilder().forTableView(OFFLINE_TABLE_NAME, view, "no_such_type");
+        DEFAULT_INSTANCE.getControllerRequestURLBuilder().forTableView(OFFLINE_TABLE_NAME, view, "no_such_type");
     HttpURLConnection connection = (HttpURLConnection) new URL(url).openConnection();
-    Assert.assertEquals(connection.getResponseCode(), 400);
+    assertEquals(connection.getResponseCode(), 400);
   }
 
   @Test(dataProvider = "viewProvider")
   public void testOfflineTableState(String view)
       throws Exception {
     TableViews.TableView tableView = getTableView(OFFLINE_TABLE_NAME, view, null);
-    Assert.assertNotNull(tableView._offline);
-    Assert.assertEquals(tableView._offline.size(), 1);
-    Assert.assertNull(tableView._realtime);
+    assertNotNull(tableView._offline);
+    assertEquals(tableView._offline.size(), 1);
+    assertNull(tableView._realtime);
 
     Map<String, String> serverMap = tableView._offline.get(OFFLINE_SEGMENT_NAME);
-    Assert.assertNotNull(serverMap);
-    Assert.assertEquals(serverMap.size(), 2);
+    assertNotNull(serverMap);
+    assertEquals(serverMap.size(), 2);
     for (Map.Entry<String, String> serverMapEntry : serverMap.entrySet()) {
-      Assert.assertTrue(InstanceTypeUtils.isServer(serverMapEntry.getKey()));
-      Assert.assertEquals(serverMapEntry.getValue(), "ONLINE");
+      assertTrue(InstanceTypeUtils.isServer(serverMapEntry.getKey()));
+      assertEquals(serverMapEntry.getValue(), "ONLINE");
     }
   }
 
@@ -143,41 +142,41 @@ public class TableViewsTest {
   public void testHybridTableState(String state)
       throws Exception {
     TableViews.TableView tableView = getTableView(HYBRID_TABLE_NAME, state, "realtime");
-    Assert.assertNull(tableView._offline);
-    Assert.assertNotNull(tableView._realtime);
-    Assert.assertEquals(tableView._realtime.size(), TEST_INSTANCE.NUM_SERVER_INSTANCES);
+    assertNull(tableView._offline);
+    assertNotNull(tableView._realtime);
+    assertEquals(tableView._realtime.size(), DEFAULT_NUM_SERVER_INSTANCES);
 
     tableView = getTableView(HYBRID_TABLE_NAME, state, "offline");
-    Assert.assertNotNull(tableView._offline);
-    Assert.assertEquals(tableView._offline.size(), 0);
-    Assert.assertNull(tableView._realtime);
+    assertNotNull(tableView._offline);
+    assertEquals(tableView._offline.size(), 0);
+    assertNull(tableView._realtime);
 
     tableView = getTableView(HYBRID_TABLE_NAME, state, null);
-    Assert.assertNotNull(tableView._offline);
-    Assert.assertEquals(tableView._offline.size(), 0);
-    Assert.assertNotNull(tableView._realtime);
-    Assert.assertEquals(tableView._realtime.size(), TEST_INSTANCE.NUM_SERVER_INSTANCES);
+    assertNotNull(tableView._offline);
+    assertEquals(tableView._offline.size(), 0);
+    assertNotNull(tableView._realtime);
+    assertEquals(tableView._realtime.size(), DEFAULT_NUM_SERVER_INSTANCES);
 
     tableView = getTableView(TableNameBuilder.OFFLINE.tableNameWithType(HYBRID_TABLE_NAME), state, null);
-    Assert.assertNotNull(tableView._offline);
-    Assert.assertEquals(tableView._offline.size(), 0);
-    Assert.assertNull(tableView._realtime);
+    assertNotNull(tableView._offline);
+    assertEquals(tableView._offline.size(), 0);
+    assertNull(tableView._realtime);
 
     tableView = getTableView(TableNameBuilder.REALTIME.tableNameWithType(HYBRID_TABLE_NAME), state, null);
-    Assert.assertNull(tableView._offline);
-    Assert.assertNotNull(tableView._realtime);
-    Assert.assertEquals(tableView._realtime.size(), TEST_INSTANCE.NUM_SERVER_INSTANCES);
+    assertNull(tableView._offline);
+    assertNotNull(tableView._realtime);
+    assertEquals(tableView._realtime.size(), DEFAULT_NUM_SERVER_INSTANCES);
   }
 
   private TableViews.TableView getTableView(String tableName, String view, String tableType)
       throws Exception {
-    return JsonUtils.stringToObject(ControllerTest.sendGetRequest(
-            TEST_INSTANCE.getControllerRequestURLBuilder().forTableView(tableName, view, tableType)),
+    return JsonUtils.stringToObject(
+        sendGetRequest(DEFAULT_INSTANCE.getControllerRequestURLBuilder().forTableView(tableName, view, tableType)),
         TableViews.TableView.class);
   }
 
   @AfterClass
   public void tearDown() {
-    TEST_INSTANCE.cleanup();
+    DEFAULT_INSTANCE.cleanup();
   }
 }
