@@ -30,14 +30,15 @@ import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.apache.pinot.util.TestUtils;
-import org.testng.Assert;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.fail;
 
-public class ControllerInstanceToggleTest {
-  private static final ControllerTest TEST_INSTANCE = ControllerTest.getInstance();
+
+public class ControllerInstanceToggleTest extends ControllerTest {
   private static final String RAW_TABLE_NAME = "toggleTable";
   private static final long TIMEOUT_MS = 10_000L;
   private static final String OFFLINE_TABLE_NAME = TableNameBuilder.OFFLINE.tableNameWithType(RAW_TABLE_NAME);
@@ -47,7 +48,7 @@ public class ControllerInstanceToggleTest {
   @BeforeClass
   public void setUp()
       throws Exception {
-    TEST_INSTANCE.setupSharedStateAndValidate();
+    DEFAULT_INSTANCE.setupSharedStateAndValidate();
   }
 
   @Test
@@ -55,84 +56,71 @@ public class ControllerInstanceToggleTest {
       throws Exception {
     // Create an offline table
     TableConfig tableConfig =
-        new TableConfigBuilder(TableType.OFFLINE).setTableName(RAW_TABLE_NAME).setNumReplicas(
-            TEST_INSTANCE.MIN_NUM_REPLICAS).build();
-    ControllerTest.sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forTableCreate(),
-            tableConfig.toJsonString());
-    Assert.assertEquals(
-        TEST_INSTANCE
-            .getHelixAdmin().getResourceIdealState(TEST_INSTANCE.getHelixClusterName(),
-            CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
-            .getPartitionSet()
-            .size(), 1);
-    Assert.assertEquals(
-        TEST_INSTANCE
-            .getHelixAdmin().getResourceIdealState(TEST_INSTANCE.getHelixClusterName(),
-            CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
-            .getInstanceSet(OFFLINE_TABLE_NAME)
-            .size(), TEST_INSTANCE.NUM_BROKER_INSTANCES);
+        new TableConfigBuilder(TableType.OFFLINE).setTableName(RAW_TABLE_NAME).setNumReplicas(DEFAULT_MIN_NUM_REPLICAS)
+            .build();
+    sendPostRequest(DEFAULT_INSTANCE.getControllerRequestURLBuilder().forTableCreate(), tableConfig.toJsonString());
+    assertEquals(DEFAULT_INSTANCE.getHelixAdmin()
+        .getResourceIdealState(DEFAULT_INSTANCE.getHelixClusterName(), CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
+        .getPartitionSet().size(), 1);
+    assertEquals(DEFAULT_INSTANCE.getHelixAdmin()
+        .getResourceIdealState(DEFAULT_INSTANCE.getHelixClusterName(), CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
+        .getInstanceSet(OFFLINE_TABLE_NAME).size(), DEFAULT_NUM_BROKER_INSTANCES);
 
     // Add segments
-    for (int i = 0; i < TEST_INSTANCE.NUM_SERVER_INSTANCES; i++) {
-      TEST_INSTANCE.getHelixResourceManager().addNewSegment(OFFLINE_TABLE_NAME,
-          SegmentMetadataMockUtils.mockSegmentMetadata(RAW_TABLE_NAME), "downloadUrl");
-      Assert.assertEquals(
-          TEST_INSTANCE
-              .getHelixAdmin().getResourceIdealState(TEST_INSTANCE.getHelixClusterName(), OFFLINE_TABLE_NAME)
-              .getNumPartitions(), i + 1);
+    for (int i = 0; i < DEFAULT_NUM_SERVER_INSTANCES; i++) {
+      DEFAULT_INSTANCE.getHelixResourceManager()
+          .addNewSegment(OFFLINE_TABLE_NAME, SegmentMetadataMockUtils.mockSegmentMetadata(RAW_TABLE_NAME),
+              "downloadUrl");
+      assertEquals(DEFAULT_INSTANCE.getHelixAdmin()
+          .getResourceIdealState(DEFAULT_INSTANCE.getHelixClusterName(), OFFLINE_TABLE_NAME).getNumPartitions(), i + 1);
     }
 
     // Disable server instances
-    int numEnabledInstances = TEST_INSTANCE.NUM_SERVER_INSTANCES;
-    for (String instanceName : TEST_INSTANCE
-        .getHelixAdmin().getInstancesInClusterWithTag(TEST_INSTANCE.getHelixClusterName(), SERVER_TAG_NAME)) {
+    int numEnabledInstances = DEFAULT_NUM_SERVER_INSTANCES;
+    for (String instanceName : DEFAULT_INSTANCE.getHelixAdmin()
+        .getInstancesInClusterWithTag(DEFAULT_INSTANCE.getHelixClusterName(), SERVER_TAG_NAME)) {
       toggleInstanceState(instanceName, "disable");
       numEnabledInstances--;
       checkNumOnlineInstancesFromExternalView(OFFLINE_TABLE_NAME, numEnabledInstances);
     }
 
     // Enable server instances
-    for (String instanceName : TEST_INSTANCE
-        .getHelixAdmin().getInstancesInClusterWithTag(TEST_INSTANCE.getHelixClusterName(), SERVER_TAG_NAME)) {
+    for (String instanceName : DEFAULT_INSTANCE.getHelixAdmin()
+        .getInstancesInClusterWithTag(DEFAULT_INSTANCE.getHelixClusterName(), SERVER_TAG_NAME)) {
       toggleInstanceState(instanceName, "ENABLE");
       numEnabledInstances++;
       checkNumOnlineInstancesFromExternalView(OFFLINE_TABLE_NAME, numEnabledInstances);
     }
 
     // Disable broker instances
-    for (String instanceName : TEST_INSTANCE
-        .getHelixAdmin().getInstancesInClusterWithTag(TEST_INSTANCE.getHelixClusterName(), BROKER_TAG_NAME)) {
+    numEnabledInstances = DEFAULT_NUM_BROKER_INSTANCES;
+    for (String instanceName : DEFAULT_INSTANCE.getHelixAdmin()
+        .getInstancesInClusterWithTag(DEFAULT_INSTANCE.getHelixClusterName(), BROKER_TAG_NAME)) {
       toggleInstanceState(instanceName, "Disable");
       numEnabledInstances--;
       checkNumOnlineInstancesFromExternalView(CommonConstants.Helix.BROKER_RESOURCE_INSTANCE, numEnabledInstances);
     }
 
     // Enable broker instances
-    for (String instanceName : TEST_INSTANCE
-        .getHelixAdmin().getInstancesInClusterWithTag(TEST_INSTANCE.getHelixClusterName(), BROKER_TAG_NAME)) {
+    for (String instanceName : DEFAULT_INSTANCE.getHelixAdmin()
+        .getInstancesInClusterWithTag(DEFAULT_INSTANCE.getHelixClusterName(), BROKER_TAG_NAME)) {
       toggleInstanceState(instanceName, "Enable");
       numEnabledInstances++;
       checkNumOnlineInstancesFromExternalView(CommonConstants.Helix.BROKER_RESOURCE_INSTANCE, numEnabledInstances);
     }
 
     // Delete table
-    ControllerTest
-        .sendDeleteRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forTableDelete(RAW_TABLE_NAME));
-    Assert.assertEquals(
-        TEST_INSTANCE
-            .getHelixAdmin().getResourceIdealState(TEST_INSTANCE.getHelixClusterName(),
-            CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
-            .getPartitionSet()
-            .size(), 0);
+    sendDeleteRequest(DEFAULT_INSTANCE.getControllerRequestURLBuilder().forTableDelete(RAW_TABLE_NAME));
+    assertEquals(DEFAULT_INSTANCE.getHelixAdmin()
+        .getResourceIdealState(DEFAULT_INSTANCE.getHelixClusterName(), CommonConstants.Helix.BROKER_RESOURCE_INSTANCE)
+        .getPartitionSet().size(), 0);
   }
 
   private void toggleInstanceState(String instanceName, String state) {
     // It may take time for an instance to toggle the state.
     TestUtils.waitForCondition(aVoid -> {
       try {
-        ControllerTest
-            .sendPostRequest(TEST_INSTANCE.getControllerRequestURLBuilder().forInstanceState(instanceName),
-                state);
+        sendPostRequest(DEFAULT_INSTANCE.getControllerRequestURLBuilder().forInstanceState(instanceName), state);
       } catch (IOException ioe) {
         // receive non-200 status code
         return false;
@@ -145,19 +133,19 @@ public class ControllerInstanceToggleTest {
       throws InterruptedException {
     long endTime = System.currentTimeMillis() + TIMEOUT_MS;
     while (System.currentTimeMillis() < endTime) {
-      ExternalView resourceExternalView = TEST_INSTANCE
-          .getHelixAdmin().getResourceExternalView(TEST_INSTANCE.getHelixClusterName(), resourceName);
+      ExternalView resourceExternalView = DEFAULT_INSTANCE.getHelixAdmin()
+          .getResourceExternalView(DEFAULT_INSTANCE.getHelixClusterName(), resourceName);
       Set<String> instanceSet = HelixHelper.getOnlineInstanceFromExternalView(resourceExternalView);
       if (instanceSet.size() == expectedNumOnlineInstances) {
         return;
       }
       Thread.sleep(100L);
     }
-    Assert.fail("Failed to reach " + expectedNumOnlineInstances + " online instances for resource: " + resourceName);
+    fail("Failed to reach " + expectedNumOnlineInstances + " online instances for resource: " + resourceName);
   }
 
   @AfterClass
   public void tearDown() {
-    TEST_INSTANCE.cleanup();
+    DEFAULT_INSTANCE.cleanup();
   }
 }
