@@ -25,9 +25,16 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.nio.ByteBuffer;
+import java.sql.Timestamp;
 import java.util.List;
 import javax.annotation.Nullable;
+import org.apache.pinot.common.datablock.ColumnarDataBlock;
+import org.apache.pinot.common.datablock.DataBlock;
+import org.apache.pinot.common.datablock.DataBlockUtils;
+import org.apache.pinot.common.datablock.RowDataBlock;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.utils.DataSchema;
+import org.apache.pinot.common.utils.RoaringBitmapUtils;
 import org.apache.pinot.core.common.ObjectSerDeUtils;
 import org.apache.pinot.spi.utils.ArrayCopyUtils;
 import org.apache.pinot.spi.utils.BigDecimalUtils;
@@ -37,8 +44,8 @@ import org.roaringbitmap.RoaringBitmap;
 
 public class DataBlockBuilder {
   private final DataSchema _dataSchema;
-  private final BaseDataBlock.Type _blockType;
-  private final DataSchema.ColumnDataType[] _columnDataType;
+  private final DataBlock.Type _blockType;
+  private final DataSchema.ColumnDataType[] _columnDataTypes;
 
   private int[] _columnOffsets;
   private int _rowSizeInBytes;
@@ -55,12 +62,12 @@ public class DataBlockBuilder {
   private final DataOutputStream _variableSizeDataOutputStream =
       new DataOutputStream(_variableSizeDataByteArrayOutputStream);
 
-  private DataBlockBuilder(DataSchema dataSchema, BaseDataBlock.Type blockType) {
+  private DataBlockBuilder(DataSchema dataSchema, DataBlock.Type blockType) {
     _dataSchema = dataSchema;
-    _columnDataType = dataSchema.getStoredColumnDataTypes();
+    _columnDataTypes = dataSchema.getColumnDataTypes();
     _blockType = blockType;
     _numColumns = dataSchema.size();
-    if (_blockType == BaseDataBlock.Type.COLUMNAR) {
+    if (_blockType == DataBlock.Type.COLUMNAR) {
       _cumulativeColumnOffsetSizeInBytes = new int[_numColumns];
       _columnSizeInBytes = new int[_numColumns];
       DataBlockUtils.computeColumnSizeInBytes(_dataSchema, _columnSizeInBytes);
@@ -69,7 +76,7 @@ public class DataBlockBuilder {
         _cumulativeColumnOffsetSizeInBytes[i] = cumulativeColumnOffset;
         cumulativeColumnOffset += _columnSizeInBytes[i] * _numRows;
       }
-    } else if (_blockType == BaseDataBlock.Type.ROW) {
+    } else if (_blockType == DataBlock.Type.ROW) {
       _columnOffsets = new int[_numColumns];
       _rowSizeInBytes = DataBlockUtils.computeColumnOffsets(dataSchema, _columnOffsets);
     }
@@ -81,22 +88,37 @@ public class DataBlockBuilder {
     if (nullRowIds == null || nullRowIds.isEmpty()) {
       _fixedSizeDataOutputStream.writeInt(0);
     } else {
-      byte[] bitmapBytes = ObjectSerDeUtils.ROARING_BITMAP_SER_DE.serialize(nullRowIds);
+      byte[] bitmapBytes = RoaringBitmapUtils.serialize(nullRowIds);
       _fixedSizeDataOutputStream.writeInt(bitmapBytes.length);
       _variableSizeDataByteArrayOutputStream.write(bitmapBytes);
     }
   }
 
-  public static RowDataBlock buildFromRows(List<Object[]> rows, @Nullable RoaringBitmap[] colNullBitmaps,
-      DataSchema dataSchema)
+  public static RowDataBlock buildFromRows(List<Object[]> rows, DataSchema dataSchema)
       throws IOException {
-    DataBlockBuilder rowBuilder = new DataBlockBuilder(dataSchema, BaseDataBlock.Type.ROW);
+    DataBlockBuilder rowBuilder = new DataBlockBuilder(dataSchema, DataBlock.Type.ROW);
+    // TODO: consolidate these null utils into data table utils.
+    // Selection / Agg / Distinct all have similar code.
+    int numColumns = rowBuilder._numColumns;
+    RoaringBitmap[] nullBitmaps = new RoaringBitmap[numColumns];
+    DataSchema.ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
+    DataSchema.ColumnDataType[] storedColumnDataTypes = dataSchema.getStoredColumnDataTypes();
+    Object[] nullPlaceholders = new Object[numColumns];
+    for (int colId = 0; colId < numColumns; colId++) {
+      nullBitmaps[colId] = new RoaringBitmap();
+      nullPlaceholders[colId] = columnDataTypes[colId].convert(storedColumnDataTypes[colId].getNullPlaceholder());
+    }
     rowBuilder._numRows = rows.size();
-    for (Object[] row : rows) {
+    for (int rowId = 0; rowId < rows.size(); rowId++) {
+      Object[] row = rows.get(rowId);
       ByteBuffer byteBuffer = ByteBuffer.allocate(rowBuilder._rowSizeInBytes);
-      for (int i = 0; i < rowBuilder._numColumns; i++) {
-        Object value = row[i];
-        switch (rowBuilder._columnDataType[i]) {
+      for (int colId = 0; colId < rowBuilder._numColumns; colId++) {
+        Object value = row[colId];
+        if (value == null) {
+          nullBitmaps[colId].add(rowId);
+          value = nullPlaceholders[colId];
+        }
+        switch (rowBuilder._columnDataTypes[colId]) {
           // Single-value column
           case INT:
             byteBuffer.putInt(((Number) value).intValue());
@@ -113,21 +135,29 @@ public class DataBlockBuilder {
           case BIG_DECIMAL:
             setColumn(rowBuilder, byteBuffer, (BigDecimal) value);
             break;
+          case BOOLEAN:
+            byteBuffer.putInt(((Boolean) value) ? 1 : 0);
+            break;
+          case TIMESTAMP:
+            byteBuffer.putLong(((Timestamp) value).getTime());
+            break;
           case STRING:
             setColumn(rowBuilder, byteBuffer, (String) value);
             break;
           case BYTES:
-            setColumn(rowBuilder, byteBuffer, (ByteArray) value);
+            if (value instanceof byte[]) {
+              setColumn(rowBuilder, byteBuffer, new ByteArray((byte[]) value));
+            } else {
+              setColumn(rowBuilder, byteBuffer, (ByteArray) value);
+            }
             break;
           case OBJECT:
             setColumn(rowBuilder, byteBuffer, value);
             break;
           // Multi-value column
-          case BOOLEAN_ARRAY:
           case INT_ARRAY:
             setColumn(rowBuilder, byteBuffer, (int[]) value);
             break;
-          case TIMESTAMP_ARRAY:
           case LONG_ARRAY:
             // LONG_ARRAY type covers INT_ARRAY and LONG_ARRAY
             if (value instanceof int[]) {
@@ -168,86 +198,180 @@ public class DataBlockBuilder {
             }
             break;
           case BYTES_ARRAY:
+            setColumn(rowBuilder, byteBuffer, (byte[][]) value);
+            break;
           case STRING_ARRAY:
             setColumn(rowBuilder, byteBuffer, (String[]) value);
             break;
+          case BOOLEAN_ARRAY:
+            boolean[] booleans = (boolean[]) value;
+            int length = booleans.length;
+            int[] ints = new int[length];
+            ArrayCopyUtils.copy(booleans, ints, length);
+            setColumn(rowBuilder, byteBuffer, ints);
+            break;
+          case TIMESTAMP_ARRAY:
+            Timestamp[] timestamps = (Timestamp[]) value;
+            length = timestamps.length;
+            long[] longs = new long[length];
+            ArrayCopyUtils.copy(timestamps, longs, length);
+            setColumn(rowBuilder, byteBuffer, longs);
+            break;
           default:
             throw new IllegalStateException(
-                String.format("Unsupported data type: %s for column: %s", rowBuilder._columnDataType[i],
-                    rowBuilder._dataSchema.getColumnName(i)));
+                String.format("Unsupported data type: %s for column: %s", rowBuilder._columnDataTypes[colId],
+                    rowBuilder._dataSchema.getColumnName(colId)));
         }
       }
       rowBuilder._fixedSizeDataByteArrayOutputStream.write(byteBuffer.array(), 0, byteBuffer.position());
     }
     // Write null bitmaps after writing data.
-    if (colNullBitmaps != null) {
-      for (RoaringBitmap nullBitmap : colNullBitmaps) {
-        rowBuilder.setNullRowIds(nullBitmap);
-      }
+    for (RoaringBitmap nullBitmap : nullBitmaps) {
+      rowBuilder.setNullRowIds(nullBitmap);
     }
     return buildRowBlock(rowBuilder);
   }
 
-  public static ColumnarDataBlock buildFromColumns(List<Object[]> columns, @Nullable RoaringBitmap[] colNullBitmaps,
-      DataSchema dataSchema)
+  public static ColumnarDataBlock buildFromColumns(List<Object[]> columns, DataSchema dataSchema)
       throws IOException {
-    DataBlockBuilder columnarBuilder = new DataBlockBuilder(dataSchema, BaseDataBlock.Type.COLUMNAR);
-    for (int i = 0; i < columns.size(); i++) {
-      Object[] column = columns.get(i);
+    DataBlockBuilder columnarBuilder = new DataBlockBuilder(dataSchema, DataBlock.Type.COLUMNAR);
+
+    // TODO: consolidate these null utils into data table utils.
+    // Selection / Agg / Distinct all have similar code.
+    int numColumns = columnarBuilder._numColumns;
+    RoaringBitmap[] nullBitmaps = new RoaringBitmap[numColumns];
+    DataSchema.ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
+    DataSchema.ColumnDataType[] storedColumnDataTypes = dataSchema.getStoredColumnDataTypes();
+    Object[] nullPlaceholders = new Object[numColumns];
+    for (int colId = 0; colId < numColumns; colId++) {
+      nullBitmaps[colId] = new RoaringBitmap();
+      nullPlaceholders[colId] = columnDataTypes[colId].convert(storedColumnDataTypes[colId].getNullPlaceholder());
+    }
+    for (int colId = 0; colId < columns.size(); colId++) {
+      Object[] column = columns.get(colId);
       columnarBuilder._numRows = column.length;
-      ByteBuffer byteBuffer = ByteBuffer.allocate(columnarBuilder._numRows * columnarBuilder._columnSizeInBytes[i]);
-      switch (columnarBuilder._columnDataType[i]) {
+      ByteBuffer byteBuffer = ByteBuffer.allocate(columnarBuilder._numRows * columnarBuilder._columnSizeInBytes[colId]);
+      Object value;
+      switch (columnarBuilder._columnDataTypes[colId]) {
         // Single-value column
         case INT:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             byteBuffer.putInt(((Number) value).intValue());
           }
           break;
         case LONG:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             byteBuffer.putLong(((Number) value).longValue());
           }
           break;
         case FLOAT:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             byteBuffer.putFloat(((Number) value).floatValue());
           }
           break;
         case DOUBLE:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             byteBuffer.putDouble(((Number) value).doubleValue());
           }
           break;
         case BIG_DECIMAL:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (BigDecimal) value);
           }
           break;
+        case BOOLEAN:
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
+            byteBuffer.putInt(((Boolean) value) ? 1 : 0);
+          }
+          break;
+        case TIMESTAMP:
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
+            byteBuffer.putLong(((Timestamp) value).getTime());
+          }
+          break;
         case STRING:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (String) value);
           }
           break;
         case BYTES:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (ByteArray) value);
           }
           break;
         case OBJECT:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, value);
           }
           break;
         // Multi-value column
-        case BOOLEAN_ARRAY:
         case INT_ARRAY:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (int[]) value);
           }
           break;
-        case TIMESTAMP_ARRAY:
         case LONG_ARRAY:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             if (value instanceof int[]) {
               // LONG_ARRAY type covers INT_ARRAY and LONG_ARRAY
               int[] ints = (int[]) value;
@@ -261,12 +385,22 @@ public class DataBlockBuilder {
           }
           break;
         case FLOAT_ARRAY:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (float[]) value);
           }
           break;
         case DOUBLE_ARRAY:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             // DOUBLE_ARRAY type covers INT_ARRAY, LONG_ARRAY, FLOAT_ARRAY and DOUBLE_ARRAY
             if (value instanceof int[]) {
               int[] ints = (int[]) value;
@@ -291,24 +425,53 @@ public class DataBlockBuilder {
             }
           }
           break;
+        case BOOLEAN_ARRAY:
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
+            int length = ((boolean[]) value).length;
+            int[] ints = new int[length];
+            ArrayCopyUtils.copy((boolean[]) value, ints, length);
+            setColumn(columnarBuilder, byteBuffer, ints);
+          }
+          break;
+        case TIMESTAMP_ARRAY:
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
+            int length = ((Timestamp[]) value).length;
+            long[] longs = new long[length];
+            ArrayCopyUtils.copy((Timestamp[]) value, longs, length);
+            setColumn(columnarBuilder, byteBuffer, longs);
+          }
+          break;
         case BYTES_ARRAY:
         case STRING_ARRAY:
-          for (Object value : column) {
+          for (int rowId = 0; rowId < columnarBuilder._numRows; rowId++) {
+            value = column[rowId];
+            if (value == null) {
+              nullBitmaps[colId].add(rowId);
+              value = nullPlaceholders[colId];
+            }
             setColumn(columnarBuilder, byteBuffer, (String[]) value);
           }
           break;
         default:
           throw new IllegalStateException(
-              String.format("Unsupported data type: %s for column: %s", columnarBuilder._columnDataType[i],
-                  columnarBuilder._dataSchema.getColumnName(i)));
+              String.format("Unsupported data type: %s for column: %s", columnarBuilder._columnDataTypes[colId],
+                  columnarBuilder._dataSchema.getColumnName(colId)));
       }
       columnarBuilder._fixedSizeDataByteArrayOutputStream.write(byteBuffer.array(), 0, byteBuffer.position());
     }
     // Write null bitmaps after writing data.
-    if (colNullBitmaps != null) {
-      for (RoaringBitmap nullBitmap : colNullBitmaps) {
-        columnarBuilder.setNullRowIds(nullBitmap);
-      }
+    for (RoaringBitmap nullBitmap : nullBitmaps) {
+      columnarBuilder.setNullRowIds(nullBitmap);
     }
     return buildColumnarBlock(columnarBuilder);
   }
@@ -355,14 +518,15 @@ public class DataBlockBuilder {
     builder._variableSizeDataByteArrayOutputStream.write(bytes);
   }
 
-  private static void setColumn(DataBlockBuilder builder, ByteBuffer byteBuffer, Object value)
+  // TODO: Move ser/de into AggregationFunction interface
+  private static void setColumn(DataBlockBuilder builder, ByteBuffer byteBuffer, @Nullable Object value)
       throws IOException {
     byteBuffer.putInt(builder._variableSizeDataByteArrayOutputStream.size());
-    int objectTypeValue = ObjectSerDeUtils.ObjectType.getObjectType(value).getValue();
-    if (objectTypeValue == ObjectSerDeUtils.ObjectType.Null.getValue()) {
+    if (value == null) {
       byteBuffer.putInt(0);
-      builder._variableSizeDataOutputStream.writeInt(objectTypeValue);
+      builder._variableSizeDataOutputStream.writeInt(DataTable.CustomObject.NULL_TYPE_VALUE);
     } else {
+      int objectTypeValue = ObjectSerDeUtils.ObjectType.getObjectType(value).getValue();
       byte[] bytes = ObjectSerDeUtils.serialize(value, objectTypeValue);
       byteBuffer.putInt(bytes.length);
       builder._variableSizeDataOutputStream.writeInt(objectTypeValue);
