@@ -20,9 +20,12 @@ package org.apache.pinot.core.data.manager;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
+import com.google.common.cache.Cache;
+import com.google.common.cache.CacheBuilder;
 import com.google.common.cache.LoadingCache;
 import java.io.File;
 import java.io.IOException;
+import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -30,6 +33,7 @@ import java.util.Map;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.Semaphore;
+import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 import javax.annotation.Nullable;
 import javax.annotation.concurrent.ThreadSafe;
@@ -41,15 +45,16 @@ import org.apache.helix.HelixManager;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
 import org.apache.helix.zookeeper.datamodel.ZNRecord;
 import org.apache.pinot.common.auth.AuthProviderUtils;
-import org.apache.pinot.common.metadata.ZKMetadataProvider;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.metrics.ServerGauge;
 import org.apache.pinot.common.metrics.ServerMeter;
 import org.apache.pinot.common.metrics.ServerMetrics;
 import org.apache.pinot.common.restlet.resources.SegmentErrorInfo;
 import org.apache.pinot.common.utils.TarGzCompressionUtils;
+import org.apache.pinot.common.utils.config.TierConfigUtils;
 import org.apache.pinot.common.utils.fetcher.SegmentFetcherFactory;
 import org.apache.pinot.core.data.manager.offline.ImmutableSegmentDataManager;
+import org.apache.pinot.core.util.PeerServerSegmentFinder;
 import org.apache.pinot.segment.local.data.manager.SegmentDataManager;
 import org.apache.pinot.segment.local.data.manager.TableDataManager;
 import org.apache.pinot.segment.local.data.manager.TableDataManagerConfig;
@@ -59,7 +64,6 @@ import org.apache.pinot.segment.local.segment.index.loader.IndexLoadingConfig;
 import org.apache.pinot.segment.local.segment.index.loader.LoaderUtils;
 import org.apache.pinot.segment.spi.ImmutableSegment;
 import org.apache.pinot.segment.spi.SegmentMetadata;
-import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoader;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderContext;
@@ -100,6 +104,8 @@ public abstract class BaseTableDataManager implements TableDataManager {
   // Fixed size LRU cache with TableName - SegmentName pair as key, and segment related
   // errors as the value.
   protected LoadingCache<Pair<String, String>, SegmentErrorInfo> _errorCache;
+  // Cache used for identifying segments which could not be acquired since they were recently deleted.
+  protected Cache<String, String> _recentlyDeletedSegments;
 
   @Override
   public void init(TableDataManagerConfig tableDataManagerConfig, String instanceId,
@@ -114,35 +120,35 @@ public abstract class BaseTableDataManager implements TableDataManager {
     _serverMetrics = serverMetrics;
     _helixManager = helixManager;
 
-    _authProvider = AuthProviderUtils.extractAuthProvider(toPinotConfiguration(_tableDataManagerConfig.getAuthConfig()),
-        null);
+    _authProvider =
+        AuthProviderUtils.extractAuthProvider(toPinotConfiguration(_tableDataManagerConfig.getAuthConfig()), null);
 
     _tableNameWithType = tableDataManagerConfig.getTableName();
     _tableDataDir = tableDataManagerConfig.getDataDir();
     _indexDir = new File(_tableDataDir);
     if (!_indexDir.exists()) {
-      Preconditions.checkState(_indexDir.mkdirs(),
-          "Unable to create index directory at %s. "
-              + "Please check for available space and write-permissions for this directory.",
-          _indexDir);
+      Preconditions.checkState(_indexDir.mkdirs(), "Unable to create index directory at %s. "
+          + "Please check for available space and write-permissions for this directory.", _indexDir);
     }
     _resourceTmpDir = new File(_indexDir, "tmp");
     // This is meant to cleanup temp resources from TableDataManager. But other code using this same
     // directory will have those deleted as well.
     FileUtils.deleteQuietly(_resourceTmpDir);
     if (!_resourceTmpDir.exists()) {
-      Preconditions.checkState(_resourceTmpDir.mkdirs(),
-          "Unable to create temp resources directory at %s. "
-              + "Please check for available space and write-permissions for this directory.",
-          _resourceTmpDir);
+      Preconditions.checkState(_resourceTmpDir.mkdirs(), "Unable to create temp resources directory at %s. "
+          + "Please check for available space and write-permissions for this directory.", _resourceTmpDir);
     }
     _errorCache = errorCache;
+    _recentlyDeletedSegments =
+        CacheBuilder.newBuilder().maximumSize(tableDataManagerConfig.getTableDeletedSegmentsCacheSize())
+            .expireAfterWrite(tableDataManagerConfig.getTableDeletedSegmentsCacheTtlMinutes(), TimeUnit.MINUTES)
+            .build();
     _streamSegmentDownloadUntarRateLimitBytesPerSec =
         tableDataManagerParams.getStreamSegmentDownloadUntarRateLimitBytesPerSec();
     _isStreamSegmentDownloadUntar = tableDataManagerParams.isStreamSegmentDownloadUntar();
     if (_isStreamSegmentDownloadUntar) {
       LOGGER.info("Using streamed download-untar for segment download! "
-              + "The rate limit interval for streamed download-untar is {} ms",
+              + "The rate limit interval for streamed download-untar is {} bytes/s",
           _streamSegmentDownloadUntarRateLimitBytesPerSec);
     }
     int maxParallelSegmentDownloads = tableDataManagerParams.getMaxParallelSegmentDownloads();
@@ -200,7 +206,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
     _serverMetrics.addValueToTableGauge(_tableNameWithType, ServerGauge.SEGMENT_COUNT, 1L);
 
     ImmutableSegmentDataManager newSegmentManager = new ImmutableSegmentDataManager(immutableSegment);
-    SegmentDataManager oldSegmentManager = _segmentDataManagerMap.put(segmentName, newSegmentManager);
+    SegmentDataManager oldSegmentManager = registerSegment(segmentName, newSegmentManager);
     if (oldSegmentManager == null) {
       _logger.info("Added new immutable segment: {} to table: {}", segmentName, _tableNameWithType);
     } else {
@@ -212,12 +218,12 @@ public abstract class BaseTableDataManager implements TableDataManager {
   @Override
   public void addSegment(File indexDir, IndexLoadingConfig indexLoadingConfig)
       throws Exception {
-    Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, _tableNameWithType);
-    addSegment(ImmutableSegmentLoader.load(indexDir, indexLoadingConfig, schema));
+    indexLoadingConfig.setTableDataDir(_tableDataDir);
+    addSegment(ImmutableSegmentLoader.load(indexDir, indexLoadingConfig, indexLoadingConfig.getSchema()));
   }
 
   @Override
-  public void addSegment(String segmentName, TableConfig tableConfig, IndexLoadingConfig indexLoadingConfig)
+  public void addSegment(String segmentName, IndexLoadingConfig indexLoadingConfig, SegmentZKMetadata zkMetadata)
       throws Exception {
     throw new UnsupportedOperationException();
   }
@@ -231,7 +237,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
   @Override
   public void removeSegment(String segmentName) {
     _logger.info("Removing segment: {} from table: {}", segmentName, _tableNameWithType);
-    SegmentDataManager segmentDataManager = _segmentDataManagerMap.remove(segmentName);
+    SegmentDataManager segmentDataManager = unregisterSegment(segmentName);
     if (segmentDataManager != null) {
       releaseSegment(segmentDataManager);
       _logger.info("Removed segment: {} from table: {}", segmentName, _tableNameWithType);
@@ -240,6 +246,15 @@ public abstract class BaseTableDataManager implements TableDataManager {
     }
   }
 
+  /**
+   * Returns true if the given segment has been deleted recently. The time range is determined by
+   * {@link org.apache.pinot.spi.config.instance.InstanceDataManagerConfig#getDeletedSegmentsCacheTtlMinutes()}.
+   */
+  @Override
+  public boolean isSegmentDeletedRecently(String segmentName) {
+    return _recentlyDeletedSegments.getIfPresent(segmentName) != null;
+  }
+
   @Override
   public List<SegmentDataManager> acquireAllSegments() {
     List<SegmentDataManager> segmentDataManagers = new ArrayList<>();
@@ -309,6 +324,11 @@ public abstract class BaseTableDataManager implements TableDataManager {
     return _indexDir;
   }
 
+  @Override
+  public TableDataManagerConfig getTableDataManagerConfig() {
+    return _tableDataManagerConfig;
+  }
+
   @Override
   public void addSegmentError(String segmentName, SegmentErrorInfo segmentErrorInfo) {
     _errorCache.put(Pair.of(_tableNameWithType, segmentName), segmentErrorInfo);
@@ -329,7 +349,11 @@ public abstract class BaseTableDataManager implements TableDataManager {
   public void reloadSegment(String segmentName, IndexLoadingConfig indexLoadingConfig, SegmentZKMetadata zkMetadata,
       SegmentMetadata localMetadata, @Nullable Schema schema, boolean forceDownload)
       throws Exception {
-    File indexDir = getSegmentDataDir(segmentName);
+    String segmentTier = getSegmentCurrentTier(segmentName);
+    indexLoadingConfig.setSegmentTier(segmentTier);
+    indexLoadingConfig.setTableDataDir(_tableDataDir);
+    indexLoadingConfig.setInstanceTierConfigs(_tableDataManagerConfig.getInstanceTierConfigs());
+    File indexDir = getSegmentDataDir(segmentName, segmentTier, indexLoadingConfig.getTableConfig());
     try {
       // Create backup directory to handle failure of segment reloading.
       createBackup(indexDir);
@@ -347,18 +371,22 @@ public abstract class BaseTableDataManager implements TableDataManager {
         }
         indexDir = downloadSegment(segmentName, zkMetadata);
       } else {
-        LOGGER.info("Reload existing segment: {} of table: {}", segmentName, _tableNameWithType);
+        LOGGER.info("Reload existing segment: {} of table: {} on tier: {}", segmentName, _tableNameWithType,
+            TierConfigUtils.normalizeTierName(segmentTier));
         // The indexDir is empty after calling createBackup, as it's renamed to a backup directory.
         // The SegmentDirectory should initialize accordingly. Like for SegmentLocalFSDirectory, it
         // doesn't load anything from an empty indexDir, but gets the info to complete the copyTo.
         try (SegmentDirectory segmentDirectory = initSegmentDirectory(segmentName, String.valueOf(zkMetadata.getCrc()),
-            indexLoadingConfig, schema)) {
+            indexLoadingConfig)) {
           segmentDirectory.copyTo(indexDir);
         }
       }
 
       // Load from indexDir and replace the old segment in memory. What's inside indexDir
       // may come from SegmentDirectory.copyTo() or the segment downloaded from deep store.
+      indexLoadingConfig.setSegmentTier(zkMetadata.getTier());
+      LOGGER.info("Load segment with data from indexDir: {} to tier: {}", indexDir,
+          TierConfigUtils.normalizeTierName(zkMetadata.getTier()));
       ImmutableSegment segment = ImmutableSegmentLoader.load(indexDir, indexLoadingConfig, schema);
       addSegment(segment);
 
@@ -390,6 +418,10 @@ public abstract class BaseTableDataManager implements TableDataManager {
     // still on the server but the metadata object has not been initialized yet. In this case,
     // we should check if the segment exists on server and try to load it. If the segment does
     // not exist or fails to get loaded, we download segment from deep store to load it again.
+    String segmentTier = zkMetadata.getTier();
+    indexLoadingConfig.setSegmentTier(segmentTier);
+    indexLoadingConfig.setTableDataDir(_tableDataDir);
+    indexLoadingConfig.setInstanceTierConfigs(_tableDataManagerConfig.getInstanceTierConfigs());
     if (localMetadata == null && tryLoadExistingSegment(segmentName, indexLoadingConfig, zkMetadata)) {
       return;
     }
@@ -406,9 +438,36 @@ public abstract class BaseTableDataManager implements TableDataManager {
           localMetadata.getCrc(), zkMetadata.getCrc());
     }
     File indexDir = downloadSegment(segmentName, zkMetadata);
-    addSegment(indexDir, indexLoadingConfig);
-    LOGGER.info("Downloaded and loaded segment: {} of table: {} with crc: {}", segmentName, _tableNameWithType,
-        zkMetadata.getCrc());
+    ImmutableSegment segment =
+        ImmutableSegmentLoader.load(indexDir, indexLoadingConfig, indexLoadingConfig.getSchema(), true);
+    addSegment(segment);
+    LOGGER.info("Downloaded and loaded segment: {} of table: {} with crc: {} on tier: {}", segmentName,
+        _tableNameWithType, zkMetadata.getCrc(), TierConfigUtils.normalizeTierName(segmentTier));
+  }
+
+  /**
+   * _segmentDataManagerMap is used for fetching segments that need to be queried. If a new segment is created,
+   * calling this method ensures that all queries in the future can use the new segment. This method may replace an
+   * existing segment with the same name.
+   */
+  @Nullable
+  protected SegmentDataManager registerSegment(String segmentName, SegmentDataManager segmentDataManager) {
+    SegmentDataManager oldSegmentDataManager = _segmentDataManagerMap.put(segmentName, segmentDataManager);
+    _recentlyDeletedSegments.invalidate(segmentName);
+    return oldSegmentDataManager;
+  }
+
+  /**
+   * De-registering a segment ensures that no query uses the given segment until a segment with that name is
+   * re-registered. There may be scenarios where the broker thinks that a segment is available even though it has
+   * been de-registered in the servers (either due to manual deletion or retention). In such cases, acquireSegments
+   * will mark those segments as missingSegments. The caller can use {@link #isSegmentDeletedRecently(String)} to
+   * identify this scenario.
+   */
+  @Nullable
+  protected SegmentDataManager unregisterSegment(String segmentName) {
+    _recentlyDeletedSegments.put(segmentName, segmentName);
+    return _segmentDataManagerMap.remove(segmentName);
   }
 
   protected boolean allowDownload(String segmentName, SegmentZKMetadata zkMetadata) {
@@ -424,7 +483,6 @@ public abstract class BaseTableDataManager implements TableDataManager {
   private File downloadSegmentFromDeepStore(String segmentName, SegmentZKMetadata zkMetadata)
       throws Exception {
     File tempRootDir = getTmpSegmentDataDir("tmp-" + segmentName + "-" + UUID.randomUUID());
-    FileUtils.forceMkdir(tempRootDir);
     if (_isStreamSegmentDownloadUntar && zkMetadata.getCrypterName() == null) {
       try {
         File untaredSegDir = downloadAndStreamUntarWithRateLimit(segmentName, zkMetadata, tempRootDir,
@@ -462,6 +520,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
       throws Exception {
     File tarFile = new File(tempRootDir, segmentName + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION);
     String uri = zkMetadata.getDownloadUrl();
+    boolean downloadSuccess = false;
     try {
       if (_segmentDownloadSemaphore != null) {
         long startTime = System.currentTimeMillis();
@@ -474,19 +533,54 @@ public abstract class BaseTableDataManager implements TableDataManager {
       SegmentFetcherFactory.fetchAndDecryptSegmentToLocal(uri, tarFile, zkMetadata.getCrypterName());
       LOGGER.info("Downloaded tarred segment: {} for table: {} from: {} to: {}, file length: {}", segmentName,
           _tableNameWithType, uri, tarFile, tarFile.length());
+      downloadSuccess = true;
       return tarFile;
     } catch (AttemptsExceededException e) {
       LOGGER.error("Attempts exceeded when downloading segment: {} for table: {} from: {} to: {}", segmentName,
           _tableNameWithType, uri, tarFile);
-      _serverMetrics.addMeteredTableValue(_tableNameWithType, ServerMeter.SEGMENT_DOWNLOAD_FAILURES, 1L);
-      throw e;
+      _serverMetrics.addMeteredTableValue(_tableNameWithType, ServerMeter.SEGMENT_DOWNLOAD_FROM_REMOTE_FAILURES, 1L);
+      if (_tableDataManagerConfig.getTablePeerDownloadScheme() == null) {
+        throw e;
+      }
+      downloadFromPeersWithoutStreaming(segmentName, zkMetadata, tarFile);
+      downloadSuccess = true;
+      return tarFile;
     } finally {
+      if (!downloadSuccess) {
+        _serverMetrics.addMeteredTableValue(_tableNameWithType, ServerMeter.SEGMENT_DOWNLOAD_FAILURES, 1L);
+      }
       if (_segmentDownloadSemaphore != null) {
         _segmentDownloadSemaphore.release();
       }
     }
   }
 
+  // not thread safe. Caller should invoke it with safe concurrency control.
+  protected void downloadFromPeersWithoutStreaming(String segmentName, SegmentZKMetadata zkMetadata,
+      File destTarFile) throws Exception {
+    Preconditions.checkArgument(_tableDataManagerConfig.getTablePeerDownloadScheme() != null,
+            "Download peers require non null peer download scheme");
+    List<URI> peerSegmentURIs = PeerServerSegmentFinder.getPeerServerURIs(segmentName,
+        _tableDataManagerConfig.getTablePeerDownloadScheme(), _helixManager, _tableNameWithType);
+    if (peerSegmentURIs.isEmpty()) {
+      String msg = String.format("segment %s doesn't have any peers", segmentName);
+      LOGGER.warn(msg);
+      // HelixStateTransitionHandler would catch the runtime exception and mark the segment state as Error
+      throw new RuntimeException(msg);
+    }
+    try {
+      // Next download the segment from a randomly chosen server using configured scheme.
+      SegmentFetcherFactory.fetchAndDecryptSegmentToLocal(peerSegmentURIs, destTarFile, zkMetadata.getCrypterName());
+      LOGGER.info("Fetched segment {} from peers: {} to: {} of size: {}", segmentName, peerSegmentURIs, destTarFile,
+              destTarFile.length());
+    } catch (AttemptsExceededException e) {
+      LOGGER.error("Attempts exceeded when downloading segment: {} for table: {} from peers {} to: {}", segmentName,
+              _tableNameWithType, peerSegmentURIs, destTarFile);
+      _serverMetrics.addMeteredTableValue(_tableNameWithType, ServerMeter.SEGMENT_DOWNLOAD_FROM_PEERS_FAILURES, 1L);
+      throw e;
+    }
+  }
+
   private File downloadAndStreamUntarWithRateLimit(String segmentName, SegmentZKMetadata zkMetadata, File tempRootDir,
       long maxStreamRateInByte)
       throws Exception {
@@ -498,8 +592,8 @@ public abstract class BaseTableDataManager implements TableDataManager {
       LOGGER.info("Acquired segment download semaphore for: {} (lock-time={}ms, queue-length={}).", segmentName,
           System.currentTimeMillis() - startTime, _segmentDownloadSemaphore.getQueueLength());
     }
-    LOGGER.info("Trying to download segment {} using streamed download-untar with maxStreamRateInByte {}",
-        segmentName, maxStreamRateInByte);
+    LOGGER.info("Trying to download segment {} using streamed download-untar with maxStreamRateInByte {}", segmentName,
+        maxStreamRateInByte);
     String uri = zkMetadata.getDownloadUrl();
     try {
       File ret = SegmentFetcherFactory.fetchAndStreamUntarToLocal(uri, tempRootDir, maxStreamRateInByte);
@@ -547,8 +641,40 @@ public abstract class BaseTableDataManager implements TableDataManager {
   }
 
   @VisibleForTesting
-  protected File getTmpSegmentDataDir(String segmentName) {
-    return new File(_resourceTmpDir, segmentName);
+  File getSegmentDataDir(String segmentName, @Nullable String segmentTier, TableConfig tableConfig) {
+    if (segmentTier == null) {
+      return getSegmentDataDir(segmentName);
+    }
+    try {
+      String tierDataDir =
+          TierConfigUtils.getDataDirForTier(tableConfig, segmentTier, _tableDataManagerConfig.getInstanceTierConfigs());
+      File tierTableDataDir = new File(tierDataDir, _tableNameWithType);
+      return new File(tierTableDataDir, segmentName);
+    } catch (Exception e) {
+      LOGGER.warn("Failed to get dataDir for segment: {} of table: {} on tier: {} due to error: {}", segmentName,
+          _tableNameWithType, segmentTier, e.getMessage());
+      return getSegmentDataDir(segmentName);
+    }
+  }
+
+  @Nullable
+  private String getSegmentCurrentTier(String segmentName) {
+    SegmentDataManager segment = _segmentDataManagerMap.get(segmentName);
+    if (segment != null && segment.getSegment() instanceof ImmutableSegment) {
+      return ((ImmutableSegment) segment.getSegment()).getTier();
+    }
+    return null;
+  }
+
+  @VisibleForTesting
+  protected File getTmpSegmentDataDir(String segmentName)
+      throws IOException {
+    File tmpDir = new File(_resourceTmpDir, segmentName);
+    if (tmpDir.exists()) {
+      FileUtils.deleteQuietly(tmpDir);
+    }
+    FileUtils.forceMkdir(tmpDir);
+    return tmpDir;
   }
 
   /**
@@ -599,20 +725,18 @@ public abstract class BaseTableDataManager implements TableDataManager {
    * object may be created when trying to load the segment, but it's closed if the method
    * returns false; otherwise it's opened and to be referred by ImmutableSegment object.
    */
-  private boolean tryLoadExistingSegment(String segmentName, IndexLoadingConfig indexLoadingConfig,
+  protected boolean tryLoadExistingSegment(String segmentName, IndexLoadingConfig indexLoadingConfig,
       SegmentZKMetadata zkMetadata) {
     // Try to recover the segment from potential segment reloading failure.
-    File indexDir = getSegmentDataDir(segmentName);
+    String segmentTier = zkMetadata.getTier();
+    File indexDir = getSegmentDataDir(segmentName, segmentTier, indexLoadingConfig.getTableConfig());
     recoverReloadFailureQuietly(_tableNameWithType, segmentName, indexDir);
 
-    Schema schema = ZKMetadataProvider.getTableSchema(_propertyStore, _tableNameWithType);
-    schema = SegmentGeneratorConfig.updateSchemaWithTimestampIndexes(schema,
-        SegmentGeneratorConfig.extractTimestampIndexConfigsFromTableConfig(indexLoadingConfig.getTableConfig()));
-
     // Creates the SegmentDirectory object to access the segment metadata.
     // The metadata is null if the segment doesn't exist yet.
+
     SegmentDirectory segmentDirectory =
-        tryInitSegmentDirectory(segmentName, String.valueOf(zkMetadata.getCrc()), indexLoadingConfig, schema);
+        tryInitSegmentDirectory(segmentName, String.valueOf(zkMetadata.getCrc()), indexLoadingConfig);
     SegmentMetadataImpl segmentMetadata = (segmentDirectory == null) ? null : segmentDirectory.getSegmentMetadata();
 
     // If the segment doesn't exist on server or its CRC has changed, then we
@@ -632,6 +756,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
       // If the segment is still kept by the server, then we can
       // either load it directly if it's still consistent with latest table config and schema;
       // or reprocess it to reflect latest table config and schema before loading.
+      Schema schema = indexLoadingConfig.getSchema();
       if (!ImmutableSegmentLoader.needPreprocess(segmentDirectory, indexLoadingConfig, schema)) {
         LOGGER.info("Segment: {} of table: {} is consistent with latest table config and schema", segmentName,
             _tableNameWithType);
@@ -642,26 +767,25 @@ public abstract class BaseTableDataManager implements TableDataManager {
         // Close the stale SegmentDirectory object and recreate it with reprocessed segment.
         closeSegmentDirectoryQuietly(segmentDirectory);
         ImmutableSegmentLoader.preprocess(indexDir, indexLoadingConfig, schema);
-        segmentDirectory =
-            initSegmentDirectory(segmentName, String.valueOf(zkMetadata.getCrc()), indexLoadingConfig, schema);
+        segmentDirectory = initSegmentDirectory(segmentName, String.valueOf(zkMetadata.getCrc()), indexLoadingConfig);
       }
       ImmutableSegment segment = ImmutableSegmentLoader.load(segmentDirectory, indexLoadingConfig, schema);
       addSegment(segment);
-      LOGGER.info("Loaded existing segment: {} of table: {} with crc: {}", segmentName, _tableNameWithType,
-          zkMetadata.getCrc());
+      LOGGER.info("Loaded existing segment: {} of table: {} with crc: {} on tier: {}", segmentName, _tableNameWithType,
+          zkMetadata.getCrc(), TierConfigUtils.normalizeTierName(segmentTier));
       return true;
     } catch (Exception e) {
-      LOGGER.error("Failed to load existing segment: {} of table: {} with crc: {}", segmentName, _tableNameWithType,
-          zkMetadata.getCrc(), e);
+      LOGGER.error("Failed to load existing segment: {} of table: {} with crc: {} on tier: {}", segmentName,
+          _tableNameWithType, zkMetadata.getCrc(), TierConfigUtils.normalizeTierName(segmentTier), e);
       closeSegmentDirectoryQuietly(segmentDirectory);
       return false;
     }
   }
 
   private SegmentDirectory tryInitSegmentDirectory(String segmentName, String segmentCrc,
-      IndexLoadingConfig indexLoadingConfig, Schema schema) {
+      IndexLoadingConfig indexLoadingConfig) {
     try {
-      return initSegmentDirectory(segmentName, segmentCrc, indexLoadingConfig, schema);
+      return initSegmentDirectory(segmentName, segmentCrc, indexLoadingConfig);
     } catch (Exception e) {
       LOGGER.warn("Failed to initialize SegmentDirectory for segment: {} of table: {} with error: {}", segmentName,
           _tableNameWithType, e.getMessage());
@@ -670,16 +794,19 @@ public abstract class BaseTableDataManager implements TableDataManager {
   }
 
   private SegmentDirectory initSegmentDirectory(String segmentName, String segmentCrc,
-      IndexLoadingConfig indexLoadingConfig, Schema schema)
+      IndexLoadingConfig indexLoadingConfig)
       throws Exception {
     SegmentDirectoryLoaderContext loaderContext =
         new SegmentDirectoryLoaderContext.Builder().setTableConfig(indexLoadingConfig.getTableConfig())
-            .setSchema(schema).setInstanceId(indexLoadingConfig.getInstanceId()).setSegmentName(segmentName)
-            .setSegmentCrc(segmentCrc).setSegmentDirectoryConfigs(indexLoadingConfig.getSegmentDirectoryConfigs())
-            .build();
+            .setSchema(indexLoadingConfig.getSchema()).setInstanceId(indexLoadingConfig.getInstanceId())
+            .setTableDataDir(indexLoadingConfig.getTableDataDir()).setSegmentName(segmentName).setSegmentCrc(segmentCrc)
+            .setSegmentTier(indexLoadingConfig.getSegmentTier())
+            .setInstanceTierConfigs(indexLoadingConfig.getInstanceTierConfigs())
+            .setSegmentDirectoryConfigs(indexLoadingConfig.getSegmentDirectoryConfigs()).build();
     SegmentDirectoryLoader segmentDirectoryLoader =
         SegmentDirectoryLoaderRegistry.getSegmentDirectoryLoader(indexLoadingConfig.getSegmentDirectoryLoader());
-    File indexDir = getSegmentDataDir(segmentName);
+    File indexDir =
+        getSegmentDataDir(segmentName, indexLoadingConfig.getSegmentTier(), indexLoadingConfig.getTableConfig());
     return segmentDirectoryLoader.load(indexDir.toURI(), loaderContext);
   }
 
