@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.core.minion;
 
+import com.google.common.base.Preconditions;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.List;
@@ -203,7 +204,9 @@ public class RawIndexConverter {
 
     // Create the raw index
     DataSource dataSource = _originalImmutableSegment.getDataSource(columnName);
-    ForwardIndexReader reader = dataSource.getForwardIndex();
+    ForwardIndexReader forwardIndexReader = dataSource.getForwardIndex();
+    Preconditions.checkState(forwardIndexReader != null,
+        "Forward index disabled for column: %s, raw index conversion operation unsupported!", columnName);
     Dictionary dictionary = dataSource.getDictionary();
     assert dictionary != null;
     DataType storedType = dictionary.getValueType();
@@ -213,36 +216,36 @@ public class RawIndexConverter {
         IndexCreationContext.builder().withIndexDir(_convertedIndexDir).withColumnMetadata(columnMetadata)
             .withFieldSpec(new DimensionFieldSpec(columnName, storedType, columnMetadata.isSingleValue()))
             .withDictionary(false).build().forForwardIndex(ChunkCompressionType.LZ4, null));
-        ForwardIndexReaderContext readerContext = reader.createContext()) {
+        ForwardIndexReaderContext readerContext = forwardIndexReader.createContext()) {
       switch (storedType) {
         case INT:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putInt(dictionary.getIntValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putInt(dictionary.getIntValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         case LONG:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putLong(dictionary.getLongValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putLong(dictionary.getLongValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         case FLOAT:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putFloat(dictionary.getFloatValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putFloat(dictionary.getFloatValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         case DOUBLE:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putDouble(dictionary.getDoubleValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putDouble(dictionary.getDoubleValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         case STRING:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putString(dictionary.getStringValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putString(dictionary.getStringValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         case BYTES:
           for (int docId = 0; docId < numDocs; docId++) {
-            rawIndexCreator.putBytes(dictionary.getBytesValue(reader.getDictId(docId, readerContext)));
+            rawIndexCreator.putBytes(dictionary.getBytesValue(forwardIndexReader.getDictId(docId, readerContext)));
           }
           break;
         default:
