@@ -34,7 +34,9 @@ import org.apache.pinot.common.request.context.OrderByExpressionContext;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.operator.AcquireReleaseColumnsSegmentOperator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.BaseResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.ExceptionResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.query.selection.SelectionOperatorUtils;
 import org.apache.pinot.segment.spi.datasource.DataSourceMetadata;
@@ -53,26 +55,27 @@ import org.slf4j.LoggerFactory;
  * </ul>
  */
 @SuppressWarnings({"rawtypes", "unchecked"})
-public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombineOperator {
+public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombineOperator<SelectionResultsBlock> {
   private static final Logger LOGGER = LoggerFactory.getLogger(MinMaxValueBasedSelectionOrderByCombineOperator.class);
 
   private static final String EXPLAIN_NAME = "COMBINE_SELECT_ORDERBY_MINMAX";
 
-  // For min/max value based combine, when a thread detects that no more segments need to be processed, it inserts this
-  // special IntermediateResultsBlock into the BlockingQueue to awake the main thread
-  private static final IntermediateResultsBlock LAST_RESULTS_BLOCK =
-      new IntermediateResultsBlock(new DataSchema(new String[0], new DataSchema.ColumnDataType[0]),
-          Collections.emptyList(), false);
+  // For min/max value based combine, when a thread detects that no more segment needs to be processed, it inserts this
+  // special results block, which can be skipped during the merge phase
+  private static final BaseResultsBlock EMPTY_RESULTS_BLOCK =
+      new SelectionResultsBlock(new DataSchema(new String[0], new DataSchema.ColumnDataType[0]),
+          Collections.emptyList());
 
-  // Use an AtomicInteger to track the number of operators skipped (no result inserted into the BlockingQueue)
-  private final AtomicInteger _numOperatorsSkipped = new AtomicInteger();
-  private final AtomicReference<Comparable> _globalBoundaryValue = new AtomicReference<>();
+  // Use an AtomicInteger to track the end operator id, beyond which no operator needs to be processed
+  private final AtomicInteger _endOperatorId;
   private final int _numRowsToKeep;
   private final List<MinMaxValueContext> _minMaxValueContexts;
+  private final AtomicReference<Comparable> _globalBoundaryValue = new AtomicReference<>();
 
   MinMaxValueBasedSelectionOrderByCombineOperator(List<Operator> operators, QueryContext queryContext,
       ExecutorService executorService) {
     super(operators, queryContext, executorService);
+    _endOperatorId = new AtomicInteger(_numOperators);
     _numRowsToKeep = queryContext.getLimit() + queryContext.getOffset();
 
     List<OrderByExpressionContext> orderByExpressions = _queryContext.getOrderByExpressions();
@@ -84,7 +87,7 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
     String firstOrderByColumn = firstOrderByExpression.getExpression().getIdentifier();
 
     _minMaxValueContexts = new ArrayList<>(_numOperators);
-    for (Operator<IntermediateResultsBlock> operator : _operators) {
+    for (Operator<BaseResultsBlock> operator : _operators) {
       _minMaxValueContexts.add(new MinMaxValueContext(operator, firstOrderByColumn));
     }
     if (firstOrderByExpression.isAsc()) {
@@ -114,7 +117,6 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
     }
   }
 
-
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
@@ -128,7 +130,7 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
    * documents to fulfill the LIMIT and OFFSET requirement.
    */
   @Override
-  protected void processSegments(int taskIndex) {
+  protected void processSegments() {
     List<OrderByExpressionContext> orderByExpressions = _queryContext.getOrderByExpressions();
     assert orderByExpressions != null;
     int numOrderByExpressions = orderByExpressions.size();
@@ -143,7 +145,13 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
     //       segment result is merged.
     Comparable threadBoundaryValue = null;
 
-    for (int operatorIndex = taskIndex; operatorIndex < _numOperators; operatorIndex += _numTasks) {
+    int operatorId;
+    while ((operatorId = _nextOperatorId.getAndIncrement()) < _numOperators) {
+      if (operatorId >= _endOperatorId.get()) {
+        _blockingQueue.offer(EMPTY_RESULTS_BLOCK);
+        continue;
+      }
+
       // Calculate the boundary value from global boundary and thread boundary
       Comparable boundaryValue = _globalBoundaryValue.get();
       if (boundaryValue == null) {
@@ -163,7 +171,7 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
       }
 
       // Check if the segment can be skipped
-      MinMaxValueContext minMaxValueContext = _minMaxValueContexts.get(operatorIndex);
+      MinMaxValueContext minMaxValueContext = _minMaxValueContexts.get(operatorId);
       if (boundaryValue != null) {
         if (asc) {
           // For ascending order, no need to process more segments if the column min value is larger than the
@@ -171,9 +179,9 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
           if (minMaxValueContext._minValue != null) {
             int result = minMaxValueContext._minValue.compareTo(boundaryValue);
             if (result > 0 || (result == 0 && numOrderByExpressions == 1)) {
-              _numOperatorsSkipped.getAndAdd((_numOperators - operatorIndex - 1) / _numTasks);
-              _blockingQueue.offer(LAST_RESULTS_BLOCK);
-              return;
+              _endOperatorId.set(operatorId);
+              _blockingQueue.offer(EMPTY_RESULTS_BLOCK);
+              continue;
             }
           }
         } else {
@@ -182,9 +190,9 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
           if (minMaxValueContext._maxValue != null) {
             int result = minMaxValueContext._maxValue.compareTo(boundaryValue);
             if (result < 0 || (result == 0 && numOrderByExpressions == 1)) {
-              _numOperatorsSkipped.getAndAdd((_numOperators - operatorIndex - 1) / _numTasks);
-              _blockingQueue.offer(LAST_RESULTS_BLOCK);
-              return;
+              _endOperatorId.set(operatorId);
+              _blockingQueue.offer(EMPTY_RESULTS_BLOCK);
+              continue;
             }
           }
         }
@@ -192,22 +200,32 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
 
       // Process the segment
       Operator operator = minMaxValueContext._operator;
-      IntermediateResultsBlock resultsBlock;
+      SelectionResultsBlock resultsBlock;
       try {
         if (operator instanceof AcquireReleaseColumnsSegmentOperator) {
           ((AcquireReleaseColumnsSegmentOperator) operator).acquire();
         }
-        resultsBlock = (IntermediateResultsBlock) operator.nextBlock();
+        resultsBlock = (SelectionResultsBlock) operator.nextBlock();
       } finally {
         if (operator instanceof AcquireReleaseColumnsSegmentOperator) {
           ((AcquireReleaseColumnsSegmentOperator) operator).release();
         }
       }
-      PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) resultsBlock.getSelectionResult();
-      if (selectionResult != null && selectionResult.size() == _numRowsToKeep) {
+      Collection<Object[]> rows = resultsBlock.getRows();
+      if (rows != null && rows.size() >= _numRowsToKeep) {
         // Segment result has enough rows, update the boundary value
-        assert selectionResult.peek() != null;
-        Comparable segmentBoundaryValue = (Comparable) selectionResult.peek()[0];
+
+        Comparable segmentBoundaryValue;
+        if (rows instanceof PriorityQueue) {
+          // Results from SelectionOrderByOperator
+          assert ((PriorityQueue<Object[]>) rows).peek() != null;
+          segmentBoundaryValue = (Comparable) ((PriorityQueue<Object[]>) rows).peek()[0];
+        } else {
+          // Results from LinearSelectionOrderByOperator
+          assert rows instanceof List;
+          segmentBoundaryValue = (Comparable) ((List<Object[]>) rows).get(rows.size() - 1)[0];
+        }
+
         if (boundaryValue == null) {
           boundaryValue = segmentBoundaryValue;
         } else {
@@ -241,37 +259,39 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
    * </ul>
    */
   @Override
-  protected IntermediateResultsBlock mergeResults()
+  protected BaseResultsBlock mergeResults()
       throws Exception {
-    IntermediateResultsBlock mergedBlock = null;
+    SelectionResultsBlock mergedBlock = null;
     int numBlocksMerged = 0;
     long endTimeMs = _queryContext.getEndTimeMs();
-    while (numBlocksMerged + _numOperatorsSkipped.get() < _numOperators) {
-      IntermediateResultsBlock blockToMerge =
+    while (numBlocksMerged < _numOperators) {
+      BaseResultsBlock blockToMerge =
           _blockingQueue.poll(endTimeMs - System.currentTimeMillis(), TimeUnit.MILLISECONDS);
       if (blockToMerge == null) {
         // Query times out, skip merging the remaining results blocks
         LOGGER.error("Timed out while polling results block, numBlocksMerged: {} (query: {})", numBlocksMerged,
             _queryContext);
-        return new IntermediateResultsBlock(QueryException.getException(QueryException.EXECUTION_TIMEOUT_ERROR,
+        return new ExceptionResultsBlock(QueryException.getException(QueryException.EXECUTION_TIMEOUT_ERROR,
             new TimeoutException("Timed out while polling results block")));
       }
+      if (blockToMerge == EMPTY_RESULTS_BLOCK) {
+        numBlocksMerged++;
+        continue;
+      }
       if (blockToMerge.getProcessingExceptions() != null) {
         // Caught exception while processing segment, skip merging the remaining results blocks and directly return
         // the exception
         return blockToMerge;
       }
       if (mergedBlock == null) {
-        mergedBlock = blockToMerge;
+        mergedBlock = convertToMergeableBlock((SelectionResultsBlock) blockToMerge);
       } else {
-        if (blockToMerge != LAST_RESULTS_BLOCK) {
-          mergeResultsBlocks(mergedBlock, blockToMerge);
-        }
+        mergeResultsBlocks(mergedBlock, (SelectionResultsBlock) blockToMerge);
       }
       numBlocksMerged++;
 
       // Update the boundary value if enough rows are collected
-      PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) mergedBlock.getSelectionResult();
+      PriorityQueue<Object[]> selectionResult = mergedBlock.getRowsAsPriorityQueue();
       if (selectionResult != null && selectionResult.size() == _numRowsToKeep) {
         assert selectionResult.peek() != null;
         _globalBoundaryValue.set((Comparable) selectionResult.peek()[0]);
@@ -281,7 +301,7 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
   }
 
   @Override
-  protected void mergeResultsBlocks(IntermediateResultsBlock mergedBlock, IntermediateResultsBlock blockToMerge) {
+  protected void mergeResultsBlocks(SelectionResultsBlock mergedBlock, SelectionResultsBlock blockToMerge) {
     DataSchema mergedDataSchema = mergedBlock.getDataSchema();
     DataSchema dataSchemaToMerge = blockToMerge.getDataSchema();
     assert mergedDataSchema != null && dataSchemaToMerge != null;
@@ -296,18 +316,25 @@ public class MinMaxValueBasedSelectionOrderByCombineOperator extends BaseCombine
       return;
     }
 
-    PriorityQueue<Object[]> mergedRows = (PriorityQueue<Object[]>) mergedBlock.getSelectionResult();
-    Collection<Object[]> rowsToMerge = blockToMerge.getSelectionResult();
+    PriorityQueue<Object[]> mergedRows = mergedBlock.getRowsAsPriorityQueue();
+    Collection<Object[]> rowsToMerge = blockToMerge.getRows();
     assert mergedRows != null && rowsToMerge != null;
     SelectionOperatorUtils.mergeWithOrdering(mergedRows, rowsToMerge, _numRowsToKeep);
   }
 
+  @Override
+  protected SelectionResultsBlock convertToMergeableBlock(SelectionResultsBlock resultsBlock) {
+    // This may create a copy or return the same instance. Anyway, this operator is the owner of the
+    // value now, so it can mutate it.
+    return resultsBlock.convertToPriorityQueueBased();
+  }
+
   private static class MinMaxValueContext {
-    final Operator<IntermediateResultsBlock> _operator;
+    final Operator<BaseResultsBlock> _operator;
     final Comparable _minValue;
     final Comparable _maxValue;
 
-    MinMaxValueContext(Operator<IntermediateResultsBlock> operator, String column) {
+    MinMaxValueContext(Operator<BaseResultsBlock> operator, String column) {
       _operator = operator;
       DataSourceMetadata dataSourceMetadata = operator.getIndexSegment().getDataSource(column).getDataSourceMetadata();
       _minValue = dataSourceMetadata.getMinValue();
