@@ -107,7 +107,7 @@ public class FilterOperatorUtils {
     } else {
       // Return the AND filter operator with re-ordered child filter operators
       FilterOperatorUtils.reorderAndFilterChildOperators(queryContext, childFilterOperators);
-      return new AndFilterOperator(childFilterOperators);
+      return new AndFilterOperator(childFilterOperators, queryContext.getQueryOptions());
     }
   }
 
