@@ -104,6 +104,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _dictionary = dictionary;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public boolean applySV(int dictId) {
       return _nonMatchingDictId != dictId;
@@ -158,6 +163,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.INT;
@@ -190,6 +200,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.LONG;
@@ -222,6 +237,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.FLOAT;
@@ -254,6 +274,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.DOUBLE;
@@ -286,6 +311,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BIG_DECIMAL;
@@ -305,6 +335,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.STRING;
@@ -324,6 +359,11 @@ public class NotEqualsPredicateEvaluatorFactory {
       _nonMatchingValue = nonMatchingValue;
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return -1;
+    }
+
     @Override
     public DataType getDataType() {
       return DataType.BYTES;
