@@ -209,6 +209,11 @@ public class RangePredicateEvaluatorFactory {
       }
       return _matchingDictIds;
     }
+
+    @Override
+    public int getNumMatchingItems() {
+      return Math.max(_numMatchingDictIds, 0);
+    }
   }
 
   private static final class UnsortedDictionaryBasedRangePredicateEvaluator
@@ -278,6 +283,11 @@ public class RangePredicateEvaluatorFactory {
       }
     }
 
+    @Override
+    public int getNumMatchingItems() {
+      return _matchingDictIdSet == null ? super.getNumMatchingItems() : _matchingDictIdSet.size();
+    }
+
     @Override
     public int[] getMatchingDictIds() {
       throw new UnsupportedOperationException();
