@@ -29,20 +29,19 @@ import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.common.RowBasedBlockValueFetcher;
 import org.apache.pinot.core.operator.BaseOperator;
 import org.apache.pinot.core.operator.ExecutionStatistics;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
 import org.apache.pinot.core.operator.blocks.TransformBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.operator.transform.TransformOperator;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.segment.spi.IndexSegment;
 
 
-public class StreamingSelectionOnlyOperator extends BaseOperator<IntermediateResultsBlock> {
+public class StreamingSelectionOnlyOperator extends BaseOperator<SelectionResultsBlock> {
 
   private static final String EXPLAIN_NAME = "SELECT_STREAMING";
 
   private final IndexSegment _indexSegment;
-  private final boolean _nullHandlingEnabled;
   private final TransformOperator _transformOperator;
   private final List<ExpressionContext> _expressions;
   private final BlockValSet[] _blockValSets;
@@ -54,7 +53,6 @@ public class StreamingSelectionOnlyOperator extends BaseOperator<IntermediateRes
   public StreamingSelectionOnlyOperator(IndexSegment indexSegment, QueryContext queryContext,
       List<ExpressionContext> expressions, TransformOperator transformOperator) {
     _indexSegment = indexSegment;
-    _nullHandlingEnabled = queryContext.isNullHandlingEnabled();
     _transformOperator = transformOperator;
     _expressions = expressions;
 
@@ -76,7 +74,7 @@ public class StreamingSelectionOnlyOperator extends BaseOperator<IntermediateRes
 
   @Nullable
   @Override
-  protected IntermediateResultsBlock getNextBlock() {
+  protected SelectionResultsBlock getNextBlock() {
     if (_numDocsScanned >= _limit) {
       // Already returned enough documents
       return null;
@@ -97,10 +95,9 @@ public class StreamingSelectionOnlyOperator extends BaseOperator<IntermediateRes
       rows.add(blockValueFetcher.getRow(i));
     }
     _numDocsScanned += numDocs;
-    return new IntermediateResultsBlock(_dataSchema, rows, _nullHandlingEnabled);
+    return new SelectionResultsBlock(_dataSchema, rows);
   }
 
-
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
