@@ -36,7 +36,6 @@ import org.apache.pinot.segment.spi.datasource.DataSource;
 public class ArrayLengthTransformFunction extends BaseTransformFunction {
   public static final String FUNCTION_NAME = "arrayLength";
 
-  private int[] _results;
   private TransformFunction _argument;
 
   @Override
@@ -68,45 +67,43 @@ public class ArrayLengthTransformFunction extends BaseTransformFunction {
   @Override
   public int[] transformToIntValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
-    if (_results == null || _results.length < length) {
-      _results = new int[length];
+    if (_intValuesSV == null) {
+      _intValuesSV = new int[length];
     }
-
     switch (_argument.getResultMetadata().getDataType().getStoredType()) {
       case INT:
         int[][] intValuesMV = _argument.transformToIntValuesMV(projectionBlock);
         for (int i = 0; i < length; i++) {
-          _results[i] = intValuesMV[i].length;
+          _intValuesSV[i] = intValuesMV[i].length;
         }
         break;
       case LONG:
         long[][] longValuesMV = _argument.transformToLongValuesMV(projectionBlock);
         for (int i = 0; i < length; i++) {
-          _results[i] = longValuesMV[i].length;
+          _intValuesSV[i] = longValuesMV[i].length;
         }
         break;
       case FLOAT:
         float[][] floatValuesMV = _argument.transformToFloatValuesMV(projectionBlock);
         for (int i = 0; i < length; i++) {
-          _results[i] = floatValuesMV[i].length;
+          _intValuesSV[i] = floatValuesMV[i].length;
         }
         break;
       case DOUBLE:
         double[][] doubleValuesMV = _argument.transformToDoubleValuesMV(projectionBlock);
         for (int i = 0; i < length; i++) {
-          _results[i] = doubleValuesMV[i].length;
+          _intValuesSV[i] = doubleValuesMV[i].length;
         }
         break;
       case STRING:
         String[][] stringValuesMV = _argument.transformToStringValuesMV(projectionBlock);
         for (int i = 0; i < length; i++) {
-          _results[i] = stringValuesMV[i].length;
+          _intValuesSV[i] = stringValuesMV[i].length;
         }
         break;
       default:
         throw new IllegalStateException();
     }
-    return _results;
+    return _intValuesSV;
   }
 }
