@@ -38,11 +38,6 @@ import org.apache.pinot.spi.data.FieldSpec.DataType;
 public class ArrayMaxTransformFunction extends BaseTransformFunction {
   public static final String FUNCTION_NAME = "arrayMax";
 
-  private int[] _intValuesSV;
-  private long[] _longValuesSV;
-  private float[] _floatValuesSV;
-  private double[] _doubleValuesSV;
-  private String[] _stringValuesSV;
   private TransformFunction _argument;
   private TransformResultMetadata _resultMetadata;
 
@@ -78,10 +73,8 @@ public class ArrayMaxTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.INT) {
       return super.transformToIntValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_intValuesSV == null || _intValuesSV.length < length) {
+    if (_intValuesSV == null) {
       _intValuesSV = new int[length];
     }
     int[][] intValuesMV = _argument.transformToIntValuesMV(projectionBlock);
@@ -100,10 +93,8 @@ public class ArrayMaxTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.LONG) {
       return super.transformToLongValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_longValuesSV == null || _longValuesSV.length < length) {
+    if (_longValuesSV == null) {
       _longValuesSV = new long[length];
     }
     long[][] longValuesMV = _argument.transformToLongValuesMV(projectionBlock);
@@ -122,10 +113,8 @@ public class ArrayMaxTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.FLOAT) {
       return super.transformToFloatValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_floatValuesSV == null || _floatValuesSV.length < length) {
+    if (_floatValuesSV == null) {
       _floatValuesSV = new float[length];
     }
     float[][] floatValuesMV = _argument.transformToFloatValuesMV(projectionBlock);
@@ -144,10 +133,8 @@ public class ArrayMaxTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.DOUBLE) {
       return super.transformToDoubleValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_doubleValuesSV == null || _doubleValuesSV.length < length) {
+    if (_doubleValuesSV == null) {
       _doubleValuesSV = new double[length];
     }
     double[][] doubleValuesMV = _argument.transformToDoubleValuesMV(projectionBlock);
@@ -166,10 +153,8 @@ public class ArrayMaxTransformFunction extends BaseTransformFunction {
     if (_argument.getResultMetadata().getDataType().getStoredType() != DataType.STRING) {
       return super.transformToStringValuesSV(projectionBlock);
     }
-
     int length = projectionBlock.getNumDocs();
-
-    if (_stringValuesSV == null || _stringValuesSV.length < length) {
+    if (_stringValuesSV == null) {
       _stringValuesSV = new String[length];
     }
     String[][] stringValuesMV = _argument.transformToStringValuesMV(projectionBlock);
