@@ -18,7 +18,6 @@
  */
 package org.apache.pinot.core.operator.transform.function;
 
-import com.google.common.base.Preconditions;
 import java.math.BigDecimal;
 import org.apache.pinot.core.operator.blocks.ProjectionBlock;
 import org.apache.pinot.core.operator.transform.TransformResultMetadata;
@@ -47,25 +46,50 @@ public abstract class BaseTransformFunction implements TransformFunction {
       new TransformResultMetadata(DataType.TIMESTAMP, true, false);
   protected static final TransformResultMetadata STRING_SV_NO_DICTIONARY_METADATA =
       new TransformResultMetadata(DataType.STRING, true, false);
-  protected static final TransformResultMetadata STRING_MV_NO_DICTIONARY_METADATA =
-      new TransformResultMetadata(DataType.STRING, false, false);
   protected static final TransformResultMetadata JSON_SV_NO_DICTIONARY_METADATA =
       new TransformResultMetadata(DataType.JSON, true, false);
   protected static final TransformResultMetadata BYTES_SV_NO_DICTIONARY_METADATA =
       new TransformResultMetadata(DataType.BYTES, true, false);
 
+  protected static final TransformResultMetadata INT_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.INT, false, false);
+  protected static final TransformResultMetadata LONG_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.LONG, false, false);
+  protected static final TransformResultMetadata FLOAT_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.FLOAT, false, false);
+  protected static final TransformResultMetadata DOUBLE_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.DOUBLE, false, false);
+  // TODO: Support MV BIG_DECIMAL
+  protected static final TransformResultMetadata BIG_DECIMAL_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.BIG_DECIMAL, false, false);
+  protected static final TransformResultMetadata BOOLEAN_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.BOOLEAN, false, false);
+  protected static final TransformResultMetadata TIMESTAMP_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.TIMESTAMP, false, false);
+  protected static final TransformResultMetadata STRING_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.STRING, false, false);
+  protected static final TransformResultMetadata JSON_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.JSON, false, false);
+  // TODO: Support MV BYTES
+  protected static final TransformResultMetadata BYTES_MV_NO_DICTIONARY_METADATA =
+      new TransformResultMetadata(DataType.BYTES, false, false);
+
+  // These buffers are used to hold the result for different result types. When the subclass overrides a method, it can
+  // reuse the buffer for that method. E.g. if transformToIntValuesSV is overridden, the result can be written into
+  // _intValuesSV.
   protected int[] _intValuesSV;
   protected long[] _longValuesSV;
   protected float[] _floatValuesSV;
   protected double[] _doubleValuesSV;
   protected BigDecimal[] _bigDecimalValuesSV;
   protected String[] _stringValuesSV;
-  protected byte[][] _byteValuesSV;
+  protected byte[][] _bytesValuesSV;
   protected int[][] _intValuesMV;
   protected long[][] _longValuesMV;
   protected float[][] _floatValuesMV;
   protected double[][] _doubleValuesMV;
   protected String[][] _stringValuesMV;
+  protected byte[][][] _bytesValuesMV;
 
   @Override
   public Dictionary getDictionary() {
@@ -85,17 +109,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public int[] transformToIntValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_intValuesSV == null) {
       _intValuesSV = new int[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readIntValues(dictIds, length, _intValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case LONG:
           long[] longValues = transformToLongValuesSV(projectionBlock);
           ArrayCopyUtils.copy(longValues, _intValuesSV, length);
@@ -117,7 +140,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(stringValues, _intValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as INT", resultDataType));
       }
     }
     return _intValuesSV;
@@ -126,17 +149,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public long[] transformToLongValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_longValuesSV == null) {
       _longValuesSV = new long[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readLongValues(dictIds, length, _longValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[] intValues = transformToIntValuesSV(projectionBlock);
           ArrayCopyUtils.copy(intValues, _longValuesSV, length);
@@ -158,7 +180,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(stringValues, _longValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as LONG", resultDataType));
       }
     }
     return _longValuesSV;
@@ -167,17 +189,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public float[] transformToFloatValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_floatValuesSV == null) {
       _floatValuesSV = new float[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readFloatValues(dictIds, length, _floatValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[] intValues = transformToIntValuesSV(projectionBlock);
           ArrayCopyUtils.copy(intValues, _floatValuesSV, length);
@@ -199,7 +220,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(stringValues, _floatValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as FLOAT", resultDataType));
       }
     }
     return _floatValuesSV;
@@ -208,17 +229,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public double[] transformToDoubleValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_doubleValuesSV == null) {
       _doubleValuesSV = new double[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readDoubleValues(dictIds, length, _doubleValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[] intValues = transformToIntValuesSV(projectionBlock);
           ArrayCopyUtils.copy(intValues, _doubleValuesSV, length);
@@ -240,7 +260,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(stringValues, _doubleValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as DOUBLE", resultDataType));
       }
     }
     return _doubleValuesSV;
@@ -249,16 +269,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public BigDecimal[] transformToBigDecimalValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-    if (_bigDecimalValuesSV == null || _bigDecimalValuesSV.length < length) {
+    if (_bigDecimalValuesSV == null) {
       _bigDecimalValuesSV = new BigDecimal[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readBigDecimalValues(dictIds, length, _bigDecimalValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[] intValues = transformToIntValuesSV(projectionBlock);
           ArrayCopyUtils.copy(intValues, _bigDecimalValuesSV, length);
@@ -284,7 +304,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(bytesValues, _bigDecimalValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as BIG_DECIMAL", resultDataType));
       }
     }
     return _bigDecimalValuesSV;
@@ -293,17 +313,16 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public String[] transformToStringValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_stringValuesSV == null) {
       _stringValuesSV = new String[length];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
       dictionary.readStringValues(dictIds, length, _stringValuesSV);
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[] intValues = transformToIntValuesSV(projectionBlock);
           ArrayCopyUtils.copy(intValues, _stringValuesSV, length);
@@ -329,7 +348,7 @@ public abstract class BaseTransformFunction implements TransformFunction {
           ArrayCopyUtils.copy(bytesValues, _stringValuesSV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read SV %s as STRING", resultDataType));
       }
     }
     return _stringValuesSV;
@@ -338,36 +357,37 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public byte[][] transformToBytesValuesSV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
-    if (_byteValuesSV == null) {
-      _byteValuesSV = new byte[length][];
+    if (_bytesValuesSV == null) {
+      _bytesValuesSV = new byte[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[] dictIds = transformToDictIdsSV(projectionBlock);
-      dictionary.readBytesValues(dictIds, length, _byteValuesSV);
+      dictionary.readBytesValues(dictIds, length, _bytesValuesSV);
     } else {
-      if (getResultMetadata().getDataType().getStoredType() == DataType.BIG_DECIMAL) {
-        BigDecimal[] bigDecimalValues = transformToBigDecimalValuesSV(projectionBlock);
-        ArrayCopyUtils.copy(bigDecimalValues, _byteValuesSV, length);
-      } else {
-        Preconditions.checkState(getResultMetadata().getDataType().getStoredType() == DataType.STRING);
-        String[] stringValues = transformToStringValuesSV(projectionBlock);
-        ArrayCopyUtils.copy(stringValues, _byteValuesSV, length);
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
+        case BIG_DECIMAL:
+          BigDecimal[] bigDecimalValues = transformToBigDecimalValuesSV(projectionBlock);
+          ArrayCopyUtils.copy(bigDecimalValues, _bytesValuesSV, length);
+          break;
+        case STRING:
+          String[] stringValues = transformToStringValuesSV(projectionBlock);
+          ArrayCopyUtils.copy(stringValues, _bytesValuesSV, length);
+          break;
+        default:
+          throw new IllegalStateException(String.format("Cannot read SV %s as BYTES", resultDataType));
       }
     }
-    return _byteValuesSV;
+    return _bytesValuesSV;
   }
 
   @Override
   public int[][] transformToIntValuesMV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_intValuesMV == null) {
       _intValuesMV = new int[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
@@ -379,49 +399,26 @@ public abstract class BaseTransformFunction implements TransformFunction {
         _intValuesMV[i] = intValues;
       }
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case LONG:
           long[][] longValuesMV = transformToLongValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            long[] longValues = longValuesMV[i];
-            int numValues = longValues.length;
-            int[] intValues = new int[numValues];
-            ArrayCopyUtils.copy(longValues, intValues, numValues);
-            _intValuesMV[i] = intValues;
-          }
+          ArrayCopyUtils.copy(longValuesMV, _intValuesMV, length);
           break;
         case FLOAT:
           float[][] floatValuesMV = transformToFloatValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            float[] floatValues = floatValuesMV[i];
-            int numValues = floatValues.length;
-            int[] intValues = new int[numValues];
-            ArrayCopyUtils.copy(floatValues, intValues, numValues);
-            _intValuesMV[i] = intValues;
-          }
+          ArrayCopyUtils.copy(floatValuesMV, _intValuesMV, length);
           break;
         case DOUBLE:
           double[][] doubleValuesMV = transformToDoubleValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            double[] doubleValues = doubleValuesMV[i];
-            int numValues = doubleValues.length;
-            int[] intValues = new int[numValues];
-            ArrayCopyUtils.copy(doubleValues, intValues, numValues);
-            _intValuesMV[i] = intValues;
-          }
+          ArrayCopyUtils.copy(doubleValuesMV, _intValuesMV, length);
           break;
         case STRING:
           String[][] stringValuesMV = transformToStringValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            String[] stringValues = stringValuesMV[i];
-            int numValues = stringValues.length;
-            int[] intValues = new int[numValues];
-            ArrayCopyUtils.copy(stringValues, intValues, numValues);
-            _intValuesMV[i] = intValues;
-          }
+          ArrayCopyUtils.copy(stringValuesMV, _intValuesMV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read MV %s as INT", resultDataType));
       }
     }
     return _intValuesMV;
@@ -430,11 +427,9 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public long[][] transformToLongValuesMV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_longValuesMV == null) {
       _longValuesMV = new long[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
@@ -446,49 +441,26 @@ public abstract class BaseTransformFunction implements TransformFunction {
         _longValuesMV[i] = longValues;
       }
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[][] intValuesMV = transformToIntValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            int[] intValues = intValuesMV[i];
-            int numValues = intValues.length;
-            long[] longValues = new long[numValues];
-            ArrayCopyUtils.copy(intValues, longValues, numValues);
-            _longValuesMV[i] = longValues;
-          }
+          ArrayCopyUtils.copy(intValuesMV, _longValuesMV, length);
           break;
         case FLOAT:
           float[][] floatValuesMV = transformToFloatValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            float[] floatValues = floatValuesMV[i];
-            int numValues = floatValues.length;
-            long[] longValues = new long[numValues];
-            ArrayCopyUtils.copy(floatValues, longValues, numValues);
-            _longValuesMV[i] = longValues;
-          }
+          ArrayCopyUtils.copy(floatValuesMV, _longValuesMV, length);
           break;
         case DOUBLE:
           double[][] doubleValuesMV = transformToDoubleValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            double[] doubleValues = doubleValuesMV[i];
-            int numValues = doubleValues.length;
-            long[] longValues = new long[numValues];
-            ArrayCopyUtils.copy(doubleValues, longValues, numValues);
-            _longValuesMV[i] = longValues;
-          }
+          ArrayCopyUtils.copy(doubleValuesMV, _longValuesMV, length);
           break;
         case STRING:
           String[][] stringValuesMV = transformToStringValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            String[] stringValues = stringValuesMV[i];
-            int numValues = stringValues.length;
-            long[] longValues = new long[numValues];
-            ArrayCopyUtils.copy(stringValues, longValues, numValues);
-            _longValuesMV[i] = longValues;
-          }
+          ArrayCopyUtils.copy(stringValuesMV, _longValuesMV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read MV %s as LONG", resultDataType));
       }
     }
     return _longValuesMV;
@@ -497,11 +469,9 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public float[][] transformToFloatValuesMV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_floatValuesMV == null) {
       _floatValuesMV = new float[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
@@ -513,49 +483,26 @@ public abstract class BaseTransformFunction implements TransformFunction {
         _floatValuesMV[i] = floatValues;
       }
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[][] intValuesMV = transformToIntValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            int[] intValues = intValuesMV[i];
-            int numValues = intValues.length;
-            float[] floatValues = new float[numValues];
-            ArrayCopyUtils.copy(intValues, floatValues, numValues);
-            _floatValuesMV[i] = floatValues;
-          }
+          ArrayCopyUtils.copy(intValuesMV, _floatValuesMV, length);
           break;
         case LONG:
           long[][] longValuesMV = transformToLongValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            long[] longValues = longValuesMV[i];
-            int numValues = longValues.length;
-            float[] floatValues = new float[numValues];
-            ArrayCopyUtils.copy(longValues, floatValues, numValues);
-            _floatValuesMV[i] = floatValues;
-          }
+          ArrayCopyUtils.copy(longValuesMV, _floatValuesMV, length);
           break;
         case DOUBLE:
           double[][] doubleValuesMV = transformToDoubleValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            double[] doubleValues = doubleValuesMV[i];
-            int numValues = doubleValues.length;
-            float[] floatValues = new float[numValues];
-            ArrayCopyUtils.copy(doubleValues, floatValues, numValues);
-            _floatValuesMV[i] = floatValues;
-          }
+          ArrayCopyUtils.copy(doubleValuesMV, _floatValuesMV, length);
           break;
         case STRING:
           String[][] stringValuesMV = transformToStringValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            String[] stringValues = stringValuesMV[i];
-            int numValues = stringValues.length;
-            float[] floatValues = new float[numValues];
-            ArrayCopyUtils.copy(stringValues, floatValues, numValues);
-            _floatValuesMV[i] = floatValues;
-          }
+          ArrayCopyUtils.copy(stringValuesMV, _floatValuesMV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read MV %s as FLOAT", resultDataType));
       }
     }
     return _floatValuesMV;
@@ -564,11 +511,9 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public double[][] transformToDoubleValuesMV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_doubleValuesMV == null) {
       _doubleValuesMV = new double[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
@@ -580,49 +525,26 @@ public abstract class BaseTransformFunction implements TransformFunction {
         _doubleValuesMV[i] = doubleValues;
       }
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType.getStoredType()) {
         case INT:
           int[][] intValuesMV = transformToIntValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            int[] intValues = intValuesMV[i];
-            int numValues = intValues.length;
-            double[] doubleValues = new double[numValues];
-            ArrayCopyUtils.copy(intValues, doubleValues, numValues);
-            _doubleValuesMV[i] = doubleValues;
-          }
+          ArrayCopyUtils.copy(intValuesMV, _doubleValuesMV, length);
           break;
         case LONG:
           long[][] longValuesMV = transformToLongValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            long[] longValues = longValuesMV[i];
-            int numValues = longValues.length;
-            double[] doubleValues = new double[numValues];
-            ArrayCopyUtils.copy(longValues, doubleValues, numValues);
-            _doubleValuesMV[i] = doubleValues;
-          }
+          ArrayCopyUtils.copy(longValuesMV, _doubleValuesMV, length);
           break;
         case FLOAT:
           float[][] floatValuesMV = transformToFloatValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            float[] floatValues = floatValuesMV[i];
-            int numValues = floatValues.length;
-            double[] doubleValues = new double[numValues];
-            ArrayCopyUtils.copy(floatValues, doubleValues, numValues);
-            _doubleValuesMV[i] = doubleValues;
-          }
+          ArrayCopyUtils.copy(floatValuesMV, _doubleValuesMV, length);
           break;
         case STRING:
           String[][] stringValuesMV = transformToStringValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            String[] stringValues = stringValuesMV[i];
-            int numValues = stringValues.length;
-            double[] doubleValues = new double[numValues];
-            ArrayCopyUtils.copy(stringValues, doubleValues, numValues);
-            _doubleValuesMV[i] = doubleValues;
-          }
+          ArrayCopyUtils.copy(stringValuesMV, _doubleValuesMV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read MV %s as DOUBLE", resultDataType));
       }
     }
     return _doubleValuesMV;
@@ -631,11 +553,9 @@ public abstract class BaseTransformFunction implements TransformFunction {
   @Override
   public String[][] transformToStringValuesMV(ProjectionBlock projectionBlock) {
     int length = projectionBlock.getNumDocs();
-
     if (_stringValuesMV == null) {
       _stringValuesMV = new String[length][];
     }
-
     Dictionary dictionary = getDictionary();
     if (dictionary != null) {
       int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
@@ -647,51 +567,52 @@ public abstract class BaseTransformFunction implements TransformFunction {
         _stringValuesMV[i] = stringValues;
       }
     } else {
-      switch (getResultMetadata().getDataType().getStoredType()) {
+      DataType resultDataType = getResultMetadata().getDataType();
+      switch (resultDataType) {
         case INT:
           int[][] intValuesMV = transformToIntValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            int[] intValues = intValuesMV[i];
-            int numValues = intValues.length;
-            String[] stringValues = new String[numValues];
-            ArrayCopyUtils.copy(intValues, stringValues, numValues);
-            _stringValuesMV[i] = stringValues;
-          }
+          ArrayCopyUtils.copy(intValuesMV, _stringValuesMV, length);
           break;
         case LONG:
           long[][] longValuesMV = transformToLongValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            long[] longValues = longValuesMV[i];
-            int numValues = longValues.length;
-            String[] stringValues = new String[numValues];
-            ArrayCopyUtils.copy(longValues, stringValues, numValues);
-            _stringValuesMV[i] = stringValues;
-          }
+          ArrayCopyUtils.copy(longValuesMV, _stringValuesMV, length);
           break;
         case FLOAT:
           float[][] floatValuesMV = transformToFloatValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            float[] floatValues = floatValuesMV[i];
-            int numValues = floatValues.length;
-            String[] stringValues = new String[numValues];
-            ArrayCopyUtils.copy(floatValues, stringValues, numValues);
-            _stringValuesMV[i] = stringValues;
-          }
+          ArrayCopyUtils.copy(floatValuesMV, _stringValuesMV, length);
           break;
         case DOUBLE:
           double[][] doubleValuesMV = transformToDoubleValuesMV(projectionBlock);
-          for (int i = 0; i < length; i++) {
-            double[] doubleValues = doubleValuesMV[i];
-            int numValues = doubleValues.length;
-            String[] stringValues = new String[numValues];
-            ArrayCopyUtils.copy(doubleValues, stringValues, numValues);
-            _stringValuesMV[i] = stringValues;
-          }
+          ArrayCopyUtils.copy(doubleValuesMV, _stringValuesMV, length);
           break;
         default:
-          throw new IllegalStateException();
+          throw new IllegalStateException(String.format("Cannot read MV %s as STRING", resultDataType));
       }
     }
     return _stringValuesMV;
   }
+
+  @Override
+  public byte[][][] transformToBytesValuesMV(ProjectionBlock projectionBlock) {
+    int length = projectionBlock.getNumDocs();
+    if (_bytesValuesMV == null) {
+      _bytesValuesMV = new byte[length][][];
+    }
+    Dictionary dictionary = getDictionary();
+    if (dictionary != null) {
+      int[][] dictIdsMV = transformToDictIdsMV(projectionBlock);
+      for (int i = 0; i < length; i++) {
+        int[] dictIds = dictIdsMV[i];
+        int numValues = dictIds.length;
+        byte[][] bytesValues = new byte[numValues][];
+        dictionary.readBytesValues(dictIds, numValues, bytesValues);
+        _bytesValuesMV[i] = bytesValues;
+      }
+    } else {
+      assert getResultMetadata().getDataType().getStoredType() == DataType.STRING;
+      String[][] stringValuesMV = transformToStringValuesMV(projectionBlock);
+      ArrayCopyUtils.copy(stringValuesMV, _bytesValuesMV, length);
+    }
+    return _bytesValuesMV;
+  }
 }
