@@ -18,19 +18,19 @@
  */
 package org.apache.pinot.core.query.aggregation.function;
 
-import com.google.common.math.DoubleMath;
-import java.io.Serializable;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import javax.annotation.Nullable;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.request.context.ExpressionContext;
+import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
 import org.apache.pinot.core.common.BlockValSet;
+import org.apache.pinot.core.common.ObjectSerDeUtils;
 import org.apache.pinot.core.operator.blocks.TransformBlock;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.segment.spi.index.startree.AggregationFunctionColumnPair;
@@ -66,31 +66,6 @@ public class AggregationFunctionUtils {
     return null;
   }
 
-  public static String formatValue(Object value) {
-    if (value instanceof Double) {
-      double doubleValue = (double) value;
-
-      // NOTE: String.format() is very expensive, so avoid it for whole numbers that can fit in Long.
-      //       We simply append ".00000" to long, in order to keep the existing behavior.
-      if (doubleValue <= Long.MAX_VALUE && doubleValue >= Long.MIN_VALUE && DoubleMath.isMathematicalInteger(
-          doubleValue)) {
-        return (long) doubleValue + ".00000";
-      } else {
-        return String.format(Locale.US, "%1.5f", doubleValue);
-      }
-    } else {
-      return value.toString();
-    }
-  }
-
-  public static Serializable getSerializableValue(Object value) {
-    if (value instanceof Number) {
-      return (Number) value;
-    } else {
-      return value.toString();
-    }
-  }
-
   /**
    * Collects all transform expressions required for aggregation/group-by queries.
    * <p>NOTE: We don't need to consider order-by columns here as the ordering is only allowed for aggregation functions
@@ -143,4 +118,51 @@ public class AggregationFunctionUtils {
     BlockValSet blockValSet = transformBlock.getBlockValueSet(aggregationFunctionColumnPair.toColumnName());
     return Collections.singletonMap(expression, blockValSet);
   }
+
+  /**
+   * Reads the intermediate result from the {@link DataTable}.
+   *
+   * TODO: Move ser/de into AggregationFunction interface
+   */
+  public static Object getIntermediateResult(DataTable dataTable, ColumnDataType columnDataType, int rowId, int colId) {
+    switch (columnDataType) {
+      case LONG:
+        return dataTable.getLong(rowId, colId);
+      case DOUBLE:
+        return dataTable.getDouble(rowId, colId);
+      case OBJECT:
+        DataTable.CustomObject customObject = dataTable.getCustomObject(rowId, colId);
+        return customObject != null ? ObjectSerDeUtils.deserialize(customObject) : null;
+      default:
+        throw new IllegalStateException("Illegal column data type in intermediate result: " + columnDataType);
+    }
+  }
+
+  /**
+   * Reads the converted final result from the {@link DataTable}. It should be equivalent to running
+   * {@link AggregationFunction#extractFinalResult(Object)} and {@link ColumnDataType#convert(Object)}.
+   */
+  public static Object getConvertedFinalResult(DataTable dataTable, ColumnDataType columnDataType, int rowId,
+      int colId) {
+    switch (columnDataType) {
+      case INT:
+        return dataTable.getInt(rowId, colId);
+      case LONG:
+        return dataTable.getLong(rowId, colId);
+      case FLOAT:
+        return dataTable.getFloat(rowId, colId);
+      case DOUBLE:
+        return dataTable.getDouble(rowId, colId);
+      case BIG_DECIMAL:
+        return dataTable.getBigDecimal(rowId, colId);
+      case STRING:
+        return dataTable.getString(rowId, colId);
+      case BYTES:
+        return dataTable.getBytes(rowId, colId).getBytes();
+      case DOUBLE_ARRAY:
+        return dataTable.getDoubleArray(rowId, colId);
+      default:
+        throw new IllegalStateException("Illegal column data type in final result: " + columnDataType);
+    }
+  }
 }
