@@ -32,15 +32,16 @@ import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 import javax.annotation.Nullable;
+import org.apache.pinot.common.datatable.DataTable;
+import org.apache.pinot.common.datatable.DataTableFactory;
 import org.apache.pinot.common.request.context.OrderByExpressionContext;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
-import org.apache.pinot.common.utils.DataTable;
 import org.apache.pinot.core.common.datatable.DataTableBuilder;
-import org.apache.pinot.core.common.datatable.DataTableFactory;
+import org.apache.pinot.core.common.datatable.DataTableBuilderFactory;
 import org.apache.pinot.core.data.table.Record;
 import org.apache.pinot.spi.utils.ByteArray;
-import org.apache.pinot.spi.utils.NullValueUtils;
+import org.apache.pinot.spi.utils.LoopUtils;
 import org.roaringbitmap.RoaringBitmap;
 
 
@@ -239,9 +240,12 @@ public class DistinctTable {
    */
   public void mergeTable(DistinctTable distinctTable) {
     assert _isMainTable;
+    int mergedRecords = 0;
     if (hasOrderBy()) {
       for (Record record : distinctTable._records) {
         addWithOrderBy(record);
+        mergedRecords++;
+        LoopUtils.checkMergePhaseInterruption(mergedRecords);
       }
     } else {
       if (_recordSet.size() < _limit) {
@@ -249,6 +253,8 @@ public class DistinctTable {
           if (addWithoutOrderBy(record)) {
             return;
           }
+          mergedRecords++;
+          LoopUtils.checkMergePhaseInterruption(mergedRecords);
         }
       }
     }
@@ -277,16 +283,15 @@ public class DistinctTable {
   public byte[] toBytes()
       throws IOException {
     // NOTE: Serialize the DistinctTable as a DataTable
-    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(
-        _dataSchema);
+    DataTableBuilder dataTableBuilder = DataTableBuilderFactory.getDataTableBuilder(_dataSchema);
     ColumnDataType[] storedColumnDataTypes = _dataSchema.getStoredColumnDataTypes();
     int numColumns = storedColumnDataTypes.length;
     RoaringBitmap[] nullBitmaps = null;
     if (_nullHandlingEnabled) {
       nullBitmaps = new RoaringBitmap[numColumns];
-      Object[] colDefaultNullValues = new Object[numColumns];
+      Object[] nullPlaceholders = new Object[numColumns];
       for (int colId = 0; colId < numColumns; colId++) {
-        colDefaultNullValues[colId] = NullValueUtils.getDefaultNullValue(storedColumnDataTypes[colId].toDataType());
+        nullPlaceholders[colId] = storedColumnDataTypes[colId].getNullPlaceholder();
         nullBitmaps[colId] = new RoaringBitmap();
       }
 
@@ -295,7 +300,7 @@ public class DistinctTable {
         Object[] values = record.getValues();
         for (int colId = 0; colId < numColumns; colId++) {
           if (values[colId] == null) {
-            values[colId] = colDefaultNullValues[colId];
+            values[colId] = nullPlaceholders[colId];
             nullBitmaps[colId].add(rowId);
           }
         }
