@@ -18,15 +18,18 @@
  */
 package org.apache.pinot.core.query.reduce;
 
+import com.google.common.base.Preconditions;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Map;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.metrics.BrokerMetrics;
 import org.apache.pinot.common.response.broker.BrokerResponseNative;
 import org.apache.pinot.common.response.broker.ResultTable;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
-import org.apache.pinot.common.utils.DataTable;
 import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
+import org.apache.pinot.core.query.aggregation.function.AggregationFunctionUtils;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.transport.ServerRoutingInstance;
 import org.roaringbitmap.RoaringBitmap;
@@ -52,6 +55,8 @@ public class AggregationDataTableReducer implements DataTableReducer {
   public void reduceAndSetResults(String tableName, DataSchema dataSchema,
       Map<ServerRoutingInstance, DataTable> dataTableMap, BrokerResponseNative brokerResponseNative,
       DataTableReducerContext reducerContext, BrokerMetrics brokerMetrics) {
+    assert dataSchema != null;
+
     if (dataTableMap.isEmpty()) {
       DataSchema resultTableSchema =
           new PostAggregationHandler(_queryContext, getPrePostAggregationDataSchema()).getResultDataSchema();
@@ -59,43 +64,31 @@ public class AggregationDataTableReducer implements DataTableReducer {
       return;
     }
 
-    // Merge results from all data tables
+    if (!_queryContext.isServerReturnFinalResult()) {
+      reduceWithIntermediateResult(dataSchema, dataTableMap.values(), brokerResponseNative);
+    } else {
+      Preconditions.checkState(dataTableMap.size() == 1, "Cannot merge final results from multiple servers");
+      reduceWithFinalResult(dataSchema, dataTableMap.values().iterator().next(), brokerResponseNative);
+    }
+  }
+
+  private void reduceWithIntermediateResult(DataSchema dataSchema, Collection<DataTable> dataTables,
+      BrokerResponseNative brokerResponseNative) {
     int numAggregationFunctions = _aggregationFunctions.length;
     Object[] intermediateResults = new Object[numAggregationFunctions];
-    for (DataTable dataTable : dataTableMap.values()) {
+    for (DataTable dataTable : dataTables) {
       for (int i = 0; i < numAggregationFunctions; i++) {
         Object intermediateResultToMerge;
         ColumnDataType columnDataType = dataSchema.getColumnDataType(i);
         if (_queryContext.isNullHandlingEnabled()) {
           RoaringBitmap nullBitmap = dataTable.getNullRowIds(i);
-          boolean isNull = nullBitmap != null && nullBitmap.contains(0);
-          switch (columnDataType) {
-            case LONG:
-              intermediateResultToMerge = isNull ? null : dataTable.getLong(0, i);
-              break;
-            case DOUBLE:
-              intermediateResultToMerge = isNull ? null : dataTable.getDouble(0, i);
-              break;
-            case OBJECT:
-              intermediateResultToMerge = isNull ? null : dataTable.getObject(0, i);
-              break;
-            default:
-              throw new IllegalStateException("Illegal column data type in aggregation results: " + columnDataType);
+          if (nullBitmap != null && nullBitmap.contains(0)) {
+            intermediateResultToMerge = null;
+          } else {
+            intermediateResultToMerge = AggregationFunctionUtils.getIntermediateResult(dataTable, columnDataType, 0, i);
           }
         } else {
-          switch (columnDataType) {
-            case LONG:
-              intermediateResultToMerge = dataTable.getLong(0, i);
-              break;
-            case DOUBLE:
-              intermediateResultToMerge = dataTable.getDouble(0, i);
-              break;
-            case OBJECT:
-              intermediateResultToMerge = dataTable.getObject(0, i);
-              break;
-            default:
-              throw new IllegalStateException("Illegal column data type in aggregation results: " + columnDataType);
-          }
+          intermediateResultToMerge = AggregationFunctionUtils.getIntermediateResult(dataTable, columnDataType, 0, i);
         }
         Object mergedIntermediateResult = intermediateResults[i];
         if (mergedIntermediateResult == null) {
@@ -114,6 +107,26 @@ public class AggregationDataTableReducer implements DataTableReducer {
     brokerResponseNative.setResultTable(reduceToResultTable(finalResults));
   }
 
+  private void reduceWithFinalResult(DataSchema dataSchema, DataTable dataTable,
+      BrokerResponseNative brokerResponseNative) {
+    int numAggregationFunctions = _aggregationFunctions.length;
+    Object[] finalResults = new Object[numAggregationFunctions];
+    for (int i = 0; i < numAggregationFunctions; i++) {
+      ColumnDataType columnDataType = dataSchema.getColumnDataType(i);
+      if (_queryContext.isNullHandlingEnabled()) {
+        RoaringBitmap nullBitmap = dataTable.getNullRowIds(i);
+        if (nullBitmap != null && nullBitmap.contains(0)) {
+          finalResults[i] = null;
+        } else {
+          finalResults[i] = AggregationFunctionUtils.getConvertedFinalResult(dataTable, columnDataType, 0, i);
+        }
+      } else {
+        finalResults[i] = AggregationFunctionUtils.getConvertedFinalResult(dataTable, columnDataType, 0, i);
+      }
+    }
+    brokerResponseNative.setResultTable(reduceToResultTable(finalResults));
+  }
+
   /**
    * Sets aggregation results into ResultsTable
    */
