@@ -33,13 +33,14 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import org.apache.pinot.common.config.GrpcConfig;
 import org.apache.pinot.common.config.TlsConfig;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.metrics.ServerMeter;
 import org.apache.pinot.common.metrics.ServerMetrics;
 import org.apache.pinot.common.proto.PinotQueryServerGrpc;
 import org.apache.pinot.common.proto.Server.ServerRequest;
 import org.apache.pinot.common.proto.Server.ServerResponse;
-import org.apache.pinot.common.utils.DataTable;
 import org.apache.pinot.common.utils.TlsUtils;
+import org.apache.pinot.core.operator.blocks.InstanceResponseBlock;
 import org.apache.pinot.core.operator.streaming.StreamingResponseUtils;
 import org.apache.pinot.core.query.executor.QueryExecutor;
 import org.apache.pinot.core.query.request.ServerQueryRequest;
@@ -68,8 +69,7 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
     if (tlsConfig != null) {
       try {
         _server = NettyServerBuilder.forPort(port).sslContext(buildGRpcSslContext(tlsConfig))
-            .maxInboundMessageSize(config.getMaxInboundMessageSizeBytes())
-            .addService(this).build();
+            .maxInboundMessageSize(config.getMaxInboundMessageSizeBytes()).addService(this).build();
       } catch (Exception e) {
         throw new RuntimeException("Failed to start secure grpcQueryServer", e);
       }
@@ -144,9 +144,9 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
     }
 
     // Process the query
-    DataTable dataTable;
+    InstanceResponseBlock instanceResponse;
     try {
-      dataTable = _queryExecutor.processQuery(queryRequest, _executorService, responseObserver);
+      instanceResponse = _queryExecutor.execute(queryRequest, _executorService, responseObserver);
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing request {}: {} from broker: {}", queryRequest.getRequestId(),
           queryRequest.getQueryContext(), queryRequest.getBrokerId(), e);
@@ -155,18 +155,19 @@ public class GrpcQueryServer extends PinotQueryServerGrpc.PinotQueryServerImplBa
       return;
     }
 
-    ServerResponse response;
+    ServerResponse serverResponse;
     try {
-      response = queryRequest.isEnableStreaming() ? StreamingResponseUtils.getMetadataResponse(dataTable)
+      DataTable dataTable = instanceResponse.toDataTable();
+      serverResponse = queryRequest.isEnableStreaming() ? StreamingResponseUtils.getMetadataResponse(dataTable)
           : StreamingResponseUtils.getNonStreamingResponse(dataTable);
     } catch (Exception e) {
-      LOGGER.error("Caught exception while constructing response from data table for request {}: {} from broker: {}",
+      LOGGER.error("Caught exception while serializing response for request {}: {} from broker: {}",
           queryRequest.getRequestId(), queryRequest.getQueryContext(), queryRequest.getBrokerId(), e);
       _serverMetrics.addMeteredGlobalValue(ServerMeter.RESPONSE_SERIALIZATION_EXCEPTIONS, 1);
       responseObserver.onError(Status.INTERNAL.withCause(e).asException());
       return;
     }
-    responseObserver.onNext(response);
+    responseObserver.onNext(serverResponse);
     responseObserver.onCompleted();
   }
 }
