@@ -64,7 +64,11 @@ public class PeerServerSegmentFinder {
     LLCSegmentName llcSegmentName = new LLCSegmentName(segmentName);
     String tableNameWithType =
         TableNameBuilder.forType(TableType.REALTIME).tableNameWithType(llcSegmentName.getTableName());
+    return getPeerServerURIs(segmentName, downloadScheme, helixManager, tableNameWithType);
+  }
 
+  public static List<URI> getPeerServerURIs(String segmentName, String downloadScheme,
+      HelixManager helixManager, String tableNameWithType) {
     HelixAdmin helixAdmin = helixManager.getClusterManagmentTool();
     String clusterName = helixManager.getClusterName();
     if (clusterName == null) {
