@@ -31,15 +31,18 @@ import java.util.Map;
 import java.util.Random;
 import org.apache.commons.lang.RandomStringUtils;
 import org.apache.commons.lang3.StringUtils;
+import org.apache.pinot.common.datatable.DataTable;
+import org.apache.pinot.common.datatable.DataTable.MetadataKey;
+import org.apache.pinot.common.datatable.DataTableFactory;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.utils.DataSchema;
-import org.apache.pinot.common.utils.DataTable;
-import org.apache.pinot.common.utils.DataTable.MetadataKey;
-import org.apache.pinot.core.query.request.context.ThreadTimer;
+import org.apache.pinot.core.common.ObjectSerDeUtils;
+import org.apache.pinot.spi.accounting.ThreadResourceUsageProvider;
 import org.apache.pinot.spi.utils.ByteArray;
 import org.roaringbitmap.RoaringBitmap;
 import org.testng.Assert;
+import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -85,28 +88,29 @@ public class DataTableSerDeTest {
           .put(MetadataKey.TOTAL_DOCS.getName(), String.valueOf(200L))
           .put(MetadataKey.NUM_GROUPS_LIMIT_REACHED.getName(), "true")
           .put(MetadataKey.TIME_USED_MS.getName(), String.valueOf(20000L)).put(MetadataKey.TRACE_INFO.getName(),
-          "StudentException: Error finding students\n"
-              + "        at StudentManager.findStudents(StudentManager.java:13)\n"
-              + "        at StudentProgram.main(StudentProgram.java:9)\n"
-              + "Caused by: DAOException: Error querying students from database\n"
-              + "        at StudentDAO.list(StudentDAO.java:11)\n"
-              + "        at StudentManager.findStudents(StudentManager.java:11)\n" + "        ... 1 more\n"
-              + "Caused by: java.sql.SQLException: Syntax Error\n"
-              + "        at DatabaseUtils.executeQuery(DatabaseUtils.java:5)\n"
-              + "        at StudentDAO.list(StudentDAO.java:8)\n" + "        ... 2 more")
+              "StudentException: Error finding students\n"
+                  + "        at StudentManager.findStudents(StudentManager.java:13)\n"
+                  + "        at StudentProgram.main(StudentProgram.java:9)\n"
+                  + "Caused by: DAOException: Error querying students from database\n"
+                  + "        at StudentDAO.list(StudentDAO.java:11)\n"
+                  + "        at StudentManager.findStudents(StudentManager.java:11)\n" + "        ... 1 more\n"
+                  + "Caused by: java.sql.SQLException: Syntax Error\n"
+                  + "        at DatabaseUtils.executeQuery(DatabaseUtils.java:5)\n"
+                  + "        at StudentDAO.list(StudentDAO.java:8)\n" + "        ... 2 more")
           .put(MetadataKey.REQUEST_ID.getName(), String.valueOf(90181881818L))
           .put(MetadataKey.NUM_RESIZES.getName(), String.valueOf(900L))
           .put(MetadataKey.RESIZE_TIME_MS.getName(), String.valueOf(1919199L)).build();
 
-  @Test
-  public void testException()
+  @Test(dataProvider = "versionProvider")
+  public void testException(int dataTableVersion)
       throws IOException {
+    DataTableBuilderFactory.setDataTableVersion(dataTableVersion);
     Exception exception = new UnsupportedOperationException("Caught exception.");
     ProcessingException processingException =
         QueryException.getException(QueryException.QUERY_EXECUTION_ERROR, exception);
     String expected = processingException.getMessage();
 
-    DataTable dataTable = DataTableFactory.getEmptyDataTable();
+    DataTable dataTable = DataTableBuilderFactory.getEmptyDataTable();
     dataTable.addException(processingException);
     DataTable newDataTable = DataTableFactory.getDataTable(dataTable.toBytes());
     Assert.assertNull(newDataTable.getDataSchema());
@@ -114,17 +118,19 @@ public class DataTableSerDeTest {
 
     String actual = newDataTable.getExceptions().get(QueryException.QUERY_EXECUTION_ERROR.getErrorCode());
     Assert.assertEquals(actual, expected);
+    DataTableBuilderFactory.setDataTableVersion(DataTableBuilderFactory.DEFAULT_VERSION);
   }
 
-  @Test
-  public void testEmptyStrings()
+  @Test(dataProvider = "versionProvider")
+  public void testEmptyStrings(int dataTableVersion)
       throws IOException {
+    DataTableBuilderFactory.setDataTableVersion(dataTableVersion);
     String emptyString = StringUtils.EMPTY;
     String[] emptyStringArray = {StringUtils.EMPTY};
 
     DataSchema dataSchema = new DataSchema(new String[]{"SV", "MV"},
         new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.STRING, DataSchema.ColumnDataType.STRING_ARRAY});
-    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilder = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     for (int rowId = 0; rowId < NUM_ROWS; rowId++) {
       dataTableBuilder.startRow();
       dataTableBuilder.setColumn(0, emptyString);
@@ -141,11 +147,13 @@ public class DataTableSerDeTest {
       Assert.assertEquals(newDataTable.getString(rowId, 0), emptyString);
       Assert.assertEquals(newDataTable.getStringArray(rowId, 1), emptyStringArray);
     }
+    DataTableBuilderFactory.setDataTableVersion(DataTableBuilderFactory.DEFAULT_VERSION);
   }
 
-  @Test
-  public void testAllDataTypes()
+  @Test(dataProvider = "versionProvider")
+  public void testAllDataTypes(int dataTableVersion)
       throws IOException {
+    DataTableBuilderFactory.setDataTableVersion(dataTableVersion);
     DataSchema.ColumnDataType[] columnDataTypes = DataSchema.ColumnDataType.values();
     int numColumns = columnDataTypes.length;
     String[] columnNames = new String[numColumns];
@@ -154,7 +162,7 @@ public class DataTableSerDeTest {
     }
 
     DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
-    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilder = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilder, columnDataTypes, numColumns);
 
     DataTable dataTable = dataTableBuilder.build();
@@ -162,6 +170,44 @@ public class DataTableSerDeTest {
     Assert.assertEquals(newDataTable.getDataSchema(), dataSchema, ERROR_MESSAGE);
     Assert.assertEquals(newDataTable.getNumberOfRows(), NUM_ROWS, ERROR_MESSAGE);
     verifyDataIsSame(newDataTable, columnDataTypes, numColumns);
+    DataTableBuilderFactory.setDataTableVersion(DataTableBuilderFactory.DEFAULT_VERSION);
+  }
+
+  @Test(dataProvider = "versionProvider")
+  public void testExecutionThreadCpuTimeNs(int dataTableVersion)
+      throws IOException {
+    DataTableBuilderFactory.setDataTableVersion(dataTableVersion);
+    DataSchema.ColumnDataType[] columnDataTypes = DataSchema.ColumnDataType.values();
+    int numColumns = columnDataTypes.length;
+    String[] columnNames = new String[numColumns];
+    for (int i = 0; i < numColumns; i++) {
+      columnNames[i] = columnDataTypes[i].name();
+    }
+
+    DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
+    DataTableBuilder dataTableBuilder = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
+    fillDataTableWithRandomData(dataTableBuilder, columnDataTypes, numColumns);
+
+    DataTable dataTable = dataTableBuilder.build();
+
+    // Disable ThreadCpuTimeMeasurement, serialize/de-serialize data table.
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
+    DataTable newDataTable = DataTableFactory.getDataTable(dataTable.toBytes());
+    // When ThreadCpuTimeMeasurement is disabled, no value for
+    // threadCpuTimeNs/systemActivitiesCpuTimeNs/responseSerializationCpuTimeNs.
+    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.THREAD_CPU_TIME_NS.getName()));
+    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.SYSTEM_ACTIVITIES_CPU_TIME_NS.getName()));
+    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName()));
+
+    // Enable ThreadCpuTimeMeasurement, serialize/de-serialize data table.
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(true);
+    newDataTable = DataTableFactory.getDataTable(dataTable.toBytes());
+    // When ThreadCpuTimeMeasurement is enabled, value of responseSerializationCpuTimeNs is not 0.
+    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.THREAD_CPU_TIME_NS.getName()));
+    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.SYSTEM_ACTIVITIES_CPU_TIME_NS.getName()));
+    Assert.assertTrue(
+        Integer.parseInt(newDataTable.getMetadata().get(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName())) > 0);
+    DataTableBuilderFactory.setDataTableVersion(DataTableBuilderFactory.DEFAULT_VERSION);
   }
 
   @Test
@@ -180,9 +226,9 @@ public class DataTableSerDeTest {
     // TODO: see https://github.com/apache/pinot/pull/8874/files#r894806085
 
     // Verify V4 broker can deserialize data table (has data, but has no metadata) send by V3 server
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_3);
-    DataTableBuilder dataTableBuilderV3WithDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_3);
+    DataTableBuilder dataTableBuilderV3WithDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilderV3WithDataOnly, columnDataTypes, numColumns);
 
     DataTable dataTableV3 = dataTableBuilderV3WithDataOnly.build(); // create a V3 data table
@@ -204,7 +250,7 @@ public class DataTableSerDeTest {
     Assert.assertEquals(newDataTable.getMetadata(), EXPECTED_METADATA);
 
     // Verify V4 broker can deserialize data table (only has metadata) send by V3 server
-    DataTableBuilder dataTableBuilderV3WithMetadataDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilderV3WithMetadataDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     dataTableV3 = dataTableBuilderV3WithMetadataDataOnly.build(); // create a V3 data table
     for (String key : EXPECTED_METADATA.keySet()) {
       dataTableV3.getMetadata().put(key, EXPECTED_METADATA.get(key));
@@ -216,9 +262,9 @@ public class DataTableSerDeTest {
 
     // Verify V4 broker can deserialize (has data, but has no metadata) send by V4 server(with ThreadCpuTimeMeasurement
     // disabled)
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_4);
-    DataTableBuilder dataTableBuilderV4WithDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_4);
+    DataTableBuilder dataTableBuilderV4WithDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilderV4WithDataOnly, columnDataTypes, numColumns);
     DataTable dataTableV4 = dataTableBuilderV4WithDataOnly.build(); // create a V4 data table
     // Deserialize data table bytes as V4
@@ -242,7 +288,7 @@ public class DataTableSerDeTest {
 
     // Verify V4 broker can deserialize data table (only has metadata) send by V4 server(with
     // ThreadCpuTimeMeasurement disabled)
-    DataTableBuilder dataTableBuilderV4WithMetadataDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilderV4WithMetadataDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     dataTableV4 = dataTableBuilderV4WithMetadataDataOnly.build(); // create a V4 data table
     for (String key : EXPECTED_METADATA.keySet()) {
       dataTableV4.getMetadata().put(key, EXPECTED_METADATA.get(key));
@@ -254,8 +300,8 @@ public class DataTableSerDeTest {
 
     // Verify V4 broker can deserialize (has data, but has no metadata) send by V4 server(with ThreadCpuTimeMeasurement
     // enabled)
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(true);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_4);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(true);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_4);
     dataTableV4 = dataTableBuilderV4WithDataOnly.build(); // create a V4 data table
     // Deserialize data table bytes as V4
     newDataTable = DataTableFactory.getDataTable(dataTableV4.toBytes());
@@ -275,7 +321,7 @@ public class DataTableSerDeTest {
     Assert.assertEquals(newDataTable.getDataSchema(), dataSchema, ERROR_MESSAGE);
     Assert.assertEquals(newDataTable.getNumberOfRows(), NUM_ROWS, ERROR_MESSAGE);
     verifyDataIsSame(newDataTable, columnDataTypes, numColumns);
-    if (ThreadTimer.isThreadCpuTimeMeasurementEnabled()) {
+    if (ThreadResourceUsageProvider.isThreadCpuTimeMeasurementEnabled()) {
       Assert.assertEquals(newDataTable.getMetadata().size(), EXPECTED_METADATA.keySet().size() + 1);
       newDataTable.getMetadata().remove(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName());
     }
@@ -290,7 +336,7 @@ public class DataTableSerDeTest {
     newDataTable = DataTableFactory.getDataTable(dataTableV4.toBytes()); // Broker deserialize data table bytes as V4
     Assert.assertEquals(newDataTable.getDataSchema(), dataSchema, ERROR_MESSAGE);
     Assert.assertEquals(newDataTable.getNumberOfRows(), 0, 0);
-    if (ThreadTimer.isThreadCpuTimeMeasurementEnabled()) {
+    if (ThreadResourceUsageProvider.isThreadCpuTimeMeasurementEnabled()) {
       Assert.assertEquals(newDataTable.getMetadata().size(), EXPECTED_METADATA.keySet().size() + 1);
       newDataTable.getMetadata().remove(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName());
     }
@@ -310,9 +356,9 @@ public class DataTableSerDeTest {
     DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
 
     // Verify V3 broker can deserialize data table (has data, but has no metadata) send by V2 server
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_2);
-    DataTableBuilder dataTableBuilderV2WithDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_2);
+    DataTableBuilder dataTableBuilderV2WithDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilderV2WithDataOnly, columnDataTypes, numColumns);
 
     DataTable dataTableV2 = dataTableBuilderV2WithDataOnly.build(); // create a V2 data table
@@ -334,7 +380,7 @@ public class DataTableSerDeTest {
     Assert.assertEquals(newDataTable.getMetadata(), EXPECTED_METADATA);
 
     // Verify V3 broker can deserialize data table (only has metadata) send by V2 server
-    DataTableBuilder dataTableBuilderV2WithMetadataDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilderV2WithMetadataDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     dataTableV2 = dataTableBuilderV2WithMetadataDataOnly.build(); // create a V2 data table
     for (String key : EXPECTED_METADATA.keySet()) {
       dataTableV2.getMetadata().put(key, EXPECTED_METADATA.get(key));
@@ -346,9 +392,9 @@ public class DataTableSerDeTest {
 
     // Verify V3 broker can deserialize (has data, but has no metadata) send by V3 server(with ThreadCpuTimeMeasurement
     // disabled)
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_3);
-    DataTableBuilder dataTableBuilderV3WithDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_3);
+    DataTableBuilder dataTableBuilderV3WithDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilderV3WithDataOnly, columnDataTypes, numColumns);
     DataTable dataTableV3 = dataTableBuilderV3WithDataOnly.build(); // create a V3 data table
     // Deserialize data table bytes as V3
@@ -372,7 +418,7 @@ public class DataTableSerDeTest {
 
     // Verify V3 broker can deserialize data table (only has metadata) send by V3 server(with
     // ThreadCpuTimeMeasurement disabled)
-    DataTableBuilder dataTableBuilderV3WithMetadataDataOnly = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilder dataTableBuilderV3WithMetadataDataOnly = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     dataTableV3 = dataTableBuilderV3WithMetadataDataOnly.build(); // create a V3 data table
     for (String key : EXPECTED_METADATA.keySet()) {
       dataTableV3.getMetadata().put(key, EXPECTED_METADATA.get(key));
@@ -384,8 +430,8 @@ public class DataTableSerDeTest {
 
     // Verify V3 broker can deserialize (has data, but has no metadata) send by V3 server(with ThreadCpuTimeMeasurement
     // enabled)
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(true);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_3);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(true);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_3);
     dataTableV3 = dataTableBuilderV3WithDataOnly.build(); // create a V3 data table
     // Deserialize data table bytes as V3
     newDataTable = DataTableFactory.getDataTable(dataTableV3.toBytes());
@@ -405,7 +451,7 @@ public class DataTableSerDeTest {
     Assert.assertEquals(newDataTable.getDataSchema(), dataSchema, ERROR_MESSAGE);
     Assert.assertEquals(newDataTable.getNumberOfRows(), NUM_ROWS, ERROR_MESSAGE);
     verifyDataIsSame(newDataTable, columnDataTypes, numColumns);
-    if (ThreadTimer.isThreadCpuTimeMeasurementEnabled()) {
+    if (ThreadResourceUsageProvider.isThreadCpuTimeMeasurementEnabled()) {
       Assert.assertEquals(newDataTable.getMetadata().size(), EXPECTED_METADATA.keySet().size() + 1);
       newDataTable.getMetadata().remove(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName());
     }
@@ -420,48 +466,13 @@ public class DataTableSerDeTest {
     newDataTable = DataTableFactory.getDataTable(dataTableV3.toBytes()); // Broker deserialize data table bytes as V3
     Assert.assertEquals(newDataTable.getDataSchema(), dataSchema, ERROR_MESSAGE);
     Assert.assertEquals(newDataTable.getNumberOfRows(), 0, 0);
-    if (ThreadTimer.isThreadCpuTimeMeasurementEnabled()) {
+    if (ThreadResourceUsageProvider.isThreadCpuTimeMeasurementEnabled()) {
       Assert.assertEquals(newDataTable.getMetadata().size(), EXPECTED_METADATA.keySet().size() + 1);
       newDataTable.getMetadata().remove(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName());
     }
     Assert.assertEquals(newDataTable.getMetadata(), EXPECTED_METADATA);
   }
 
-  @Test
-  public void testExecutionThreadCpuTimeNs()
-      throws IOException {
-    DataSchema.ColumnDataType[] columnDataTypes = DataSchema.ColumnDataType.values();
-    int numColumns = columnDataTypes.length;
-    String[] columnNames = new String[numColumns];
-    for (int i = 0; i < numColumns; i++) {
-      columnNames[i] = columnDataTypes[i].name();
-    }
-
-    DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
-    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(dataSchema);
-    fillDataTableWithRandomData(dataTableBuilder, columnDataTypes, numColumns);
-
-    DataTable dataTable = dataTableBuilder.build();
-
-    // Disable ThreadCpuTimeMeasurement, serialize/de-serialize data table.
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
-    DataTable newDataTable = DataTableFactory.getDataTable(dataTable.toBytes());
-    // When ThreadCpuTimeMeasurement is disabled, no value for
-    // threadCpuTimeNs/systemActivitiesCpuTimeNs/responseSerializationCpuTimeNs.
-    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.THREAD_CPU_TIME_NS.getName()));
-    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.SYSTEM_ACTIVITIES_CPU_TIME_NS.getName()));
-    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName()));
-
-    // Enable ThreadCpuTimeMeasurement, serialize/de-serialize data table.
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(true);
-    newDataTable = DataTableFactory.getDataTable(dataTable.toBytes());
-    // When ThreadCpuTimeMeasurement is enabled, value of responseSerializationCpuTimeNs is not 0.
-    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.THREAD_CPU_TIME_NS.getName()));
-    Assert.assertNull(newDataTable.getMetadata().get(MetadataKey.SYSTEM_ACTIVITIES_CPU_TIME_NS.getName()));
-    Assert.assertTrue(
-        Integer.parseInt(newDataTable.getMetadata().get(MetadataKey.RESPONSE_SER_CPU_TIME_NS.getName())) > 0);
-  }
-
   @Test
   public void testDataTableVer3MetadataBytesLayout()
       throws IOException {
@@ -472,10 +483,10 @@ public class DataTableSerDeTest {
       columnNames[i] = columnDataTypes[i].name();
     }
 
-    ThreadTimer.setThreadCpuTimeMeasurementEnabled(false);
+    ThreadResourceUsageProvider.setThreadCpuTimeMeasurementEnabled(false);
     DataSchema dataSchema = new DataSchema(columnNames, columnDataTypes);
-    DataTableFactory.setDataTableVersion(DataTableFactory.VERSION_3);
-    DataTableBuilder dataTableBuilder = DataTableFactory.getDataTableBuilder(dataSchema);
+    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_3);
+    DataTableBuilder dataTableBuilder = DataTableBuilderFactory.getDataTableBuilder(dataSchema);
     fillDataTableWithRandomData(dataTableBuilder, columnDataTypes, numColumns);
 
     DataTable dataTable = dataTableBuilder.build();
@@ -542,7 +553,7 @@ public class DataTableSerDeTest {
       DataSchema.ColumnDataType[] columnDataTypes, int numColumns)
       throws IOException {
     RoaringBitmap[] nullBitmaps = null;
-    if (DataTableFactory.getDataTableVersion() >= DataTableFactory.VERSION_4) {
+    if (DataTableBuilderFactory.getDataTableVersion() >= DataTableFactory.VERSION_4) {
       nullBitmaps = new RoaringBitmap[numColumns];
       for (int colId = 0; colId < numColumns; colId++) {
         nullBitmaps[colId] = new RoaringBitmap();
@@ -703,8 +714,8 @@ public class DataTableSerDeTest {
             Assert.assertEquals(newDataTable.getDouble(rowId, colId), isNull ? 0.0 : DOUBLES[rowId], ERROR_MESSAGE);
             break;
           case BIG_DECIMAL:
-            Assert.assertEquals(newDataTable.getBigDecimal(rowId, colId), isNull ? BigDecimal.ZERO
-                : BIG_DECIMALS[rowId], ERROR_MESSAGE);
+            Assert.assertEquals(newDataTable.getBigDecimal(rowId, colId),
+                isNull ? BigDecimal.ZERO : BIG_DECIMALS[rowId], ERROR_MESSAGE);
             break;
           case BOOLEAN:
             Assert.assertEquals(newDataTable.getInt(rowId, colId), isNull ? 0 : BOOLEANS[rowId], ERROR_MESSAGE);
@@ -723,7 +734,13 @@ public class DataTableSerDeTest {
                 ERROR_MESSAGE);
             break;
           case OBJECT:
-            Assert.assertEquals(newDataTable.getObject(rowId, colId), isNull ? null : OBJECTS[rowId], ERROR_MESSAGE);
+            DataTable.CustomObject customObject = newDataTable.getCustomObject(rowId, colId);
+            if (isNull) {
+              Assert.assertNull(customObject, ERROR_MESSAGE);
+            } else {
+              Assert.assertNotNull(customObject);
+              Assert.assertEquals(ObjectSerDeUtils.deserialize(customObject), OBJECTS[rowId], ERROR_MESSAGE);
+            }
             break;
           case INT_ARRAY:
             Assert.assertTrue(Arrays.equals(newDataTable.getIntArray(rowId, colId), INT_ARRAYS[rowId]), ERROR_MESSAGE);
@@ -761,4 +778,12 @@ public class DataTableSerDeTest {
       }
     }
   }
+
+  @DataProvider(name = "versionProvider")
+  public Object[][] provideVersion() {
+    return new Object[][]{
+        new Object[]{DataTableFactory.VERSION_4},
+        new Object[]{DataTableFactory.VERSION_3},
+    };
+  }
 }
