@@ -18,10 +18,10 @@
  */
 package org.apache.pinot.core.data.manager;
 
-import java.io.DataInputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -35,9 +35,12 @@ import org.apache.helix.HelixManager;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.metrics.ServerMetrics;
+import org.apache.pinot.common.tier.TierFactory;
 import org.apache.pinot.common.utils.TarGzCompressionUtils;
+import org.apache.pinot.common.utils.fetcher.BaseSegmentFetcher;
 import org.apache.pinot.common.utils.fetcher.SegmentFetcherFactory;
 import org.apache.pinot.core.data.manager.offline.OfflineTableDataManager;
+import org.apache.pinot.core.util.PeerServerSegmentFinder;
 import org.apache.pinot.segment.local.data.manager.TableDataManagerConfig;
 import org.apache.pinot.segment.local.data.manager.TableDataManagerParams;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentIndexCreationDriverImpl;
@@ -50,6 +53,7 @@ import org.apache.pinot.segment.spi.creator.SegmentVersion;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
+import org.apache.pinot.spi.config.table.TierConfig;
 import org.apache.pinot.spi.crypt.PinotCrypter;
 import org.apache.pinot.spi.crypt.PinotCrypterFactory;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -58,18 +62,20 @@ import org.apache.pinot.spi.data.readers.GenericRow;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.metrics.PinotMetricUtils;
 import org.apache.pinot.spi.utils.CommonConstants;
-import org.apache.pinot.spi.utils.ReadMode;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
 import org.apache.pinot.spi.utils.retry.AttemptsExceededException;
 import org.apache.pinot.util.TestUtils;
+import org.mockito.MockedStatic;
+import org.mockito.Mockito;
 import org.testng.annotations.AfterMethod;
 import org.testng.annotations.BeforeMethod;
 import org.testng.annotations.Test;
 
-import static org.apache.pinot.common.utils.fetcher.BaseSegmentFetcher.RETRY_COUNT_CONFIG_KEY;
-import static org.apache.pinot.common.utils.fetcher.BaseSegmentFetcher.RETRY_DELAY_SCALE_FACTOR_CONFIG_KEY;
-import static org.apache.pinot.common.utils.fetcher.BaseSegmentFetcher.RETRY_WAIT_MS_CONFIG_KEY;
+import static org.mockito.Mockito.doNothing;
 import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.mockStatic;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
@@ -80,7 +86,8 @@ import static org.testng.Assert.fail;
 public class BaseTableDataManagerTest {
   private static final File TEMP_DIR = new File(FileUtils.getTempDirectory(), "BaseTableDataManagerTest");
   private static final String TABLE_NAME = "table01";
-  private static final File TABLE_DATA_DIR = new File(TEMP_DIR, TABLE_NAME);
+  private static final String TABLE_NAME_WITH_TYPE = "table01_OFFLINE";
+  private static final File TABLE_DATA_DIR = new File(TEMP_DIR, TABLE_NAME_WITH_TYPE);
   private static final String STRING_COLUMN = "col1";
   private static final String[] STRING_VALUES = {"A", "D", "E", "B", "C"};
   private static final String LONG_COLUMN = "col2";
@@ -90,7 +97,7 @@ public class BaseTableDataManagerTest {
   public void setUp()
       throws Exception {
     TestUtils.ensureDirectoriesExistAndEmpty(TEMP_DIR);
-    initSegmentFetcher();
+    TableDataManagerTestUtils.initSegmentFetcher();
   }
 
   @AfterMethod
@@ -113,35 +120,74 @@ public class BaseTableDataManagerTest {
 
     BaseTableDataManager tmgr = createTableManager();
     assertFalse(tmgr.getSegmentDataDir(segName).exists());
-    tmgr.reloadSegment(segName, createIndexLoadingConfig(), zkmd, llmd, null, false);
+    tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, false);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
   }
 
+  @Test
+  public void testReloadSegmentNewDataNewTier()
+      throws Exception {
+    TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
+    String segName = "seg01";
+    SegmentZKMetadata zkmd = createRawSegment(tableConfig, segName, SegmentVersion.v3, 5);
+    String tierName = "coolTier";
+    zkmd.setTier(tierName);
+
+    // Mock the case where segment is loaded but its CRC is different from
+    // the one in zk, thus raw segment is downloaded and loaded.
+    SegmentMetadata llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn("0");
+
+    // No dataDir for coolTier, thus stay on default tier.
+    BaseTableDataManager tmgr = createTableManager();
+    File defaultSegDir = tmgr.getSegmentDataDir(segName);
+    assertFalse(defaultSegDir.exists());
+    tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null),
+        zkmd, llmd, null, false);
+    assertTrue(defaultSegDir.exists());
+    llmd = new SegmentMetadataImpl(defaultSegDir);
+    assertEquals(llmd.getTotalDocs(), 5);
+
+    // Configured dataDir for coolTier, thus move to new dir.
+    llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn("0");
+    tableConfig = createTableConfigWithTier(tierName, new File(TEMP_DIR, tierName));
+    tmgr = createTableManager();
+    IndexLoadingConfig loadingCfg = TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null);
+    tmgr.reloadSegment(segName, loadingCfg, zkmd, llmd, null, false);
+    File segDirOnTier = tmgr.getSegmentDataDir(segName, tierName, loadingCfg.getTableConfig());
+    assertTrue(segDirOnTier.exists());
+    assertFalse(defaultSegDir.exists());
+    llmd = new SegmentMetadataImpl(segDirOnTier);
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), segDirOnTier);
+  }
+
   @Test
   public void testReloadSegmentUseLocalCopy()
       throws Exception {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v1, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v1);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v1);
 
     // Same CRCs so load the local segment directory directly.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
     SegmentMetadata llmd = mock(SegmentMetadata.class);
-    when(llmd.getCrc()).thenReturn(segCrc);
+    when(llmd.getCrc()).thenReturn(Long.toString(segCrc));
 
     BaseTableDataManager tmgr = createTableManager();
-    tmgr.reloadSegment(segName, createIndexLoadingConfig(), zkmd, llmd, null, false);
+    tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, false);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
 
     FileUtils.deleteQuietly(localSegDir);
     try {
-      tmgr.reloadSegment(segName, createIndexLoadingConfig(), zkmd, llmd, null, false);
+      tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, false);
       fail();
     } catch (Exception e) {
       // As expected, segment reloading fails due to missing the local segment dir.
@@ -149,22 +195,62 @@ public class BaseTableDataManagerTest {
     }
   }
 
+  @Test
+  public void testReloadSegmentUseLocalCopyNewTier()
+      throws Exception {
+    TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
+    String segName = "seg01";
+    File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v1, 5);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v1);
+
+    // Same CRCs so load the local segment directory directly.
+    SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
+    when(zkmd.getCrc()).thenReturn(segCrc);
+    String tierName = "coolTier";
+    when(zkmd.getTier()).thenReturn(tierName);
+    SegmentMetadata llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn(Long.toString(segCrc));
+
+    // No dataDir for coolTier, thus stay on default tier.
+    BaseTableDataManager tmgr = createTableManager();
+    tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null),
+        zkmd, llmd, null, false);
+    assertTrue(tmgr.getSegmentDataDir(segName).exists());
+    llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), localSegDir);
+
+    // Configured dataDir for coolTier, thus move to new dir.
+    llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn(Long.toString(segCrc));
+    tableConfig = createTableConfigWithTier(tierName, new File(TEMP_DIR, tierName));
+    tmgr = createTableManager();
+    IndexLoadingConfig loadingCfg = TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null);
+    tmgr.reloadSegment(segName, loadingCfg, zkmd, llmd, null, false);
+    File segDirOnTier = tmgr.getSegmentDataDir(segName, tierName, loadingCfg.getTableConfig());
+    assertTrue(segDirOnTier.exists());
+    assertFalse(localSegDir.exists());
+    llmd = new SegmentMetadataImpl(segDirOnTier);
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), segDirOnTier);
+  }
+
   @Test
   public void testReloadSegmentConvertVersion()
       throws Exception {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v1, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v1);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v1);
 
     // Same CRCs so load the local segment directory directly.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
     SegmentMetadata llmd = mock(SegmentMetadata.class);
-    when(llmd.getCrc()).thenReturn(segCrc);
+    when(llmd.getCrc()).thenReturn(Long.toString(segCrc));
 
     // Require to use v3 format.
-    IndexLoadingConfig idxCfg = createIndexLoadingConfig();
+    IndexLoadingConfig idxCfg = TableDataManagerTestUtils.createIndexLoadingConfig();
     idxCfg.setSegmentVersion(SegmentVersion.v3);
 
     BaseTableDataManager tmgr = createTableManager();
@@ -181,18 +267,18 @@ public class BaseTableDataManagerTest {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v3);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
     assertFalse(hasInvertedIndex(localSegDir, STRING_COLUMN, SegmentVersion.v3));
     assertFalse(hasInvertedIndex(localSegDir, LONG_COLUMN, SegmentVersion.v3));
 
     // Same CRCs so load the local segment directory directly.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
     SegmentMetadata llmd = mock(SegmentMetadata.class);
-    when(llmd.getCrc()).thenReturn(segCrc);
+    when(llmd.getCrc()).thenReturn(Long.toString(segCrc));
 
     // Require to add indices.
-    IndexLoadingConfig idxCfg = createIndexLoadingConfig();
+    IndexLoadingConfig idxCfg = TableDataManagerTestUtils.createIndexLoadingConfig();
     idxCfg.setSegmentVersion(SegmentVersion.v3);
     idxCfg.setInvertedIndexColumns(new HashSet<>(Arrays.asList(STRING_COLUMN, LONG_COLUMN)));
 
@@ -210,8 +296,9 @@ public class BaseTableDataManagerTest {
       throws Exception {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
-    SegmentZKMetadata zkmd = createRawSegment(tableConfig, segName, SegmentVersion.v3, 5);
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
+    SegmentZKMetadata zkmd = TableDataManagerTestUtils.makeRawSegment(segName, localSegDir,
+        new File(TEMP_DIR, segName + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION), false);
 
     // Same CRC but force to download.
     BaseTableDataManager tmgr = createTableManager();
@@ -221,14 +308,14 @@ public class BaseTableDataManagerTest {
     // Remove the local segment dir. Segment reloading fails unless force to download.
     FileUtils.deleteQuietly(localSegDir);
     try {
-      tmgr.reloadSegment(segName, createIndexLoadingConfig(), zkmd, llmd, null, false);
+      tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, false);
       fail();
     } catch (Exception e) {
       // As expected, segment reloading fails due to missing the local segment dir.
       assertTrue(e.getMessage().contains("does not exist or is not a directory"));
     }
 
-    tmgr.reloadSegment(segName, createIndexLoadingConfig(), zkmd, llmd, null, true);
+    tmgr.reloadSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd, null, true);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
 
     llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
@@ -250,12 +337,51 @@ public class BaseTableDataManagerTest {
 
     BaseTableDataManager tmgr = createTableManager();
     assertFalse(tmgr.getSegmentDataDir(segName).exists());
-    tmgr.addOrReplaceSegment(segName, createIndexLoadingConfig(), zkmd, llmd);
+    tmgr.addOrReplaceSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
   }
 
+  @Test
+  public void testAddOrReplaceSegmentNewDataNewTier()
+      throws Exception {
+    TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
+    String segName = "seg01";
+    SegmentZKMetadata zkmd = createRawSegment(tableConfig, segName, SegmentVersion.v3, 5);
+    String tierName = "coolTier";
+    zkmd.setTier(tierName);
+
+    // Mock the case where segment is loaded but its CRC is different from
+    // the one in zk, thus raw segment is downloaded and loaded.
+    SegmentMetadata llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn("0");
+
+    // No dataDir for coolTier, thus stay on default tier.
+    BaseTableDataManager tmgr = createTableManager();
+    File defaultSegDir = tmgr.getSegmentDataDir(segName);
+    assertFalse(defaultSegDir.exists());
+    tmgr.addOrReplaceSegment(segName,
+        TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null), zkmd, llmd);
+    assertTrue(defaultSegDir.exists());
+    llmd = new SegmentMetadataImpl(defaultSegDir);
+    assertEquals(llmd.getTotalDocs(), 5);
+
+    // Configured dataDir for coolTier, thus move to new dir.
+    llmd = mock(SegmentMetadata.class);
+    when(llmd.getCrc()).thenReturn("0");
+    tableConfig = createTableConfigWithTier(tierName, new File(TEMP_DIR, tierName));
+    tmgr = createTableManager();
+    IndexLoadingConfig loadingCfg = TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null);
+    tmgr.addOrReplaceSegment(segName, loadingCfg, zkmd, llmd);
+    File segDirOnTier = tmgr.getSegmentDataDir(segName, tierName, loadingCfg.getTableConfig());
+    assertTrue(segDirOnTier.exists());
+    assertFalse(defaultSegDir.exists());
+    llmd = new SegmentMetadataImpl(segDirOnTier);
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), segDirOnTier);
+  }
+
   @Test
   public void testAddOrReplaceSegmentNoop()
       throws Exception {
@@ -267,7 +393,7 @@ public class BaseTableDataManagerTest {
 
     BaseTableDataManager tmgr = createTableManager();
     assertFalse(tmgr.getSegmentDataDir("seg01").exists());
-    tmgr.addOrReplaceSegment("seg01", createIndexLoadingConfig(), zkmd, llmd);
+    tmgr.addOrReplaceSegment("seg01", TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, llmd);
     // As CRC is same, the index dir is left as is, so not get created by the test.
     assertFalse(tmgr.getSegmentDataDir("seg01").exists());
   }
@@ -278,22 +404,22 @@ public class BaseTableDataManagerTest {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v3);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
 
     // Make local and remote CRC same to skip downloading raw segment.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
     when(zkmd.getDownloadUrl()).thenReturn("file://somewhere");
 
     BaseTableDataManager tmgr = createTableManager();
-    tmgr.addOrReplaceSegment(segName, createIndexLoadingConfig(), zkmd, null);
+    tmgr.addOrReplaceSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, null);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     SegmentMetadataImpl llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
 
     FileUtils.deleteQuietly(localSegDir);
     try {
-      tmgr.addOrReplaceSegment(segName, createIndexLoadingConfig(), zkmd, null);
+      tmgr.addOrReplaceSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, null);
       fail();
     } catch (Exception e) {
       // As expected, when local segment dir is missing, it tries to download
@@ -302,24 +428,62 @@ public class BaseTableDataManagerTest {
     }
   }
 
+  @Test
+  public void testAddOrReplaceSegmentUseLocalCopyNewTier()
+      throws Exception {
+    TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
+    String segName = "seg01";
+    File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
+
+    // Make local and remote CRC same to skip downloading raw segment.
+    SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
+    when(zkmd.getCrc()).thenReturn(segCrc);
+    when(zkmd.getDownloadUrl()).thenReturn("file://somewhere");
+    String tierName = "coolTier";
+    when(zkmd.getTier()).thenReturn(tierName);
+
+    // No dataDir for coolTier, thus stay on default tier.
+    BaseTableDataManager tmgr = createTableManager();
+    tmgr.addOrReplaceSegment(segName,
+        TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null), zkmd, null);
+    assertTrue(tmgr.getSegmentDataDir(segName).exists());
+    SegmentMetadataImpl llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), localSegDir);
+
+    // Configured dataDir for coolTier, thus move to new dir.
+    tableConfig = createTableConfigWithTier(tierName, new File(TEMP_DIR, tierName));
+    IndexLoadingConfig loadingCfg = TableDataManagerTestUtils.createIndexLoadingConfig("tierBased", tableConfig, null);
+    File segDirOnTier = tmgr.getSegmentDataDir(segName, tierName, loadingCfg.getTableConfig());
+    assertFalse(segDirOnTier.exists());
+    // Move segDir to new tier to see if addOrReplaceSegment() can load segDir from new tier directly.
+    FileUtils.moveDirectory(localSegDir, segDirOnTier);
+    tmgr = createTableManager();
+    tmgr.addOrReplaceSegment(segName, loadingCfg, zkmd, null);
+    llmd = new SegmentMetadataImpl(segDirOnTier);
+    assertEquals(llmd.getTotalDocs(), 5);
+    assertEquals(llmd.getIndexDir(), segDirOnTier);
+  }
+
   @Test
   public void testAddOrReplaceSegmentUseBackupCopy()
       throws Exception {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v3);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
 
     // Make local and remote CRC same to skip downloading raw segment.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
 
     BaseTableDataManager tmgr = createTableManager();
     File backup = tmgr.getSegmentDataDir(segName + CommonConstants.Segment.SEGMENT_BACKUP_DIR_SUFFIX);
     localSegDir.renameTo(backup);
 
     assertFalse(tmgr.getSegmentDataDir(segName).exists());
-    tmgr.addOrReplaceSegment(segName, createIndexLoadingConfig(), zkmd, null);
+    tmgr.addOrReplaceSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, null);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     SegmentMetadataImpl llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
@@ -340,7 +504,7 @@ public class BaseTableDataManagerTest {
     localSegDir.renameTo(backup);
 
     assertFalse(tmgr.getSegmentDataDir(segName).exists());
-    tmgr.addOrReplaceSegment(segName, createIndexLoadingConfig(), zkmd, null);
+    tmgr.addOrReplaceSegment(segName, TableDataManagerTestUtils.createIndexLoadingConfig(), zkmd, null);
     assertTrue(tmgr.getSegmentDataDir(segName).exists());
     SegmentMetadataImpl llmd = new SegmentMetadataImpl(tmgr.getSegmentDataDir(segName));
     assertEquals(llmd.getTotalDocs(), 5);
@@ -352,14 +516,14 @@ public class BaseTableDataManagerTest {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v1, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v1);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v1);
 
     // Make local and remote CRC same to skip downloading raw segment.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
 
     // Require to use v3 format.
-    IndexLoadingConfig idxCfg = createIndexLoadingConfig();
+    IndexLoadingConfig idxCfg = TableDataManagerTestUtils.createIndexLoadingConfig();
     idxCfg.setSegmentVersion(SegmentVersion.v3);
 
     BaseTableDataManager tmgr = createTableManager();
@@ -376,14 +540,14 @@ public class BaseTableDataManagerTest {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v3);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
 
     // Make local and remote CRC same to skip downloading raw segment.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
 
     // Require to use v1 format.
-    IndexLoadingConfig idxCfg = createIndexLoadingConfig();
+    IndexLoadingConfig idxCfg = TableDataManagerTestUtils.createIndexLoadingConfig();
     idxCfg.setSegmentVersion(SegmentVersion.v1);
 
     BaseTableDataManager tmgr = createTableManager();
@@ -401,16 +565,16 @@ public class BaseTableDataManagerTest {
     TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).build();
     String segName = "seg01";
     File localSegDir = createSegment(tableConfig, segName, SegmentVersion.v3, 5);
-    String segCrc = getCRC(localSegDir, SegmentVersion.v3);
+    long segCrc = TableDataManagerTestUtils.getCRC(localSegDir, SegmentVersion.v3);
     assertFalse(hasInvertedIndex(localSegDir, STRING_COLUMN, SegmentVersion.v3));
     assertFalse(hasInvertedIndex(localSegDir, LONG_COLUMN, SegmentVersion.v3));
 
     // Make local and remote CRC same to skip downloading raw segment.
     SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
+    when(zkmd.getCrc()).thenReturn(segCrc);
 
     // Require to add indices.
-    IndexLoadingConfig idxCfg = createIndexLoadingConfig();
+    IndexLoadingConfig idxCfg = TableDataManagerTestUtils.createIndexLoadingConfig();
     idxCfg.setSegmentVersion(SegmentVersion.v3);
     idxCfg.setInvertedIndexColumns(new HashSet<>(Arrays.asList(STRING_COLUMN, LONG_COLUMN)));
 
@@ -443,14 +607,14 @@ public class BaseTableDataManagerTest {
     assertEquals(FileUtils.readFileToString(tarFile), "this is from somewhere remote");
 
     FakePinotCrypter fakeCrypter = (FakePinotCrypter) PinotCrypterFactory.create("fakePinotCrypter");
-    String parentDir = TABLE_NAME + "/tmp/test-download-decrypt/";
+    String parentDir = TABLE_NAME_WITH_TYPE + "/tmp/test-download-decrypt/";
     assertTrue(fakeCrypter._origFile.getAbsolutePath().endsWith(parentDir + "seg01.tar.gz.enc"));
     assertTrue(fakeCrypter._decFile.getAbsolutePath().endsWith(parentDir + "seg01.tar.gz"));
 
     try {
       // Set maxRetry to 0 to cause retry failure immediately.
       Map<String, Object> properties = new HashMap<>();
-      properties.put(RETRY_COUNT_CONFIG_KEY, 0);
+      properties.put(BaseSegmentFetcher.RETRY_COUNT_CONFIG_KEY, 0);
       SegmentFetcherFactory.init(new PinotConfiguration(properties));
       tmgr.downloadAndDecrypt("seg01", zkmd, tempRootDir);
       fail();
@@ -459,6 +623,53 @@ public class BaseTableDataManagerTest {
     }
   }
 
+  // case 2: if the attempt to download from deep storage exceeds, invoke downloadFromPeers.
+  @Test
+  public void testDownloadAndDecryptPeerDownload() throws Exception {
+    String backupCopyURI = mockRemoteCopy().toString();
+    SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
+    when(zkmd.getDownloadUrl()).thenReturn(backupCopyURI);
+
+    TableDataManagerConfig config = createDefaultTableDataManagerConfig();
+    when(config.getTablePeerDownloadScheme()).thenReturn("http");
+    BaseTableDataManager tmgr = createSpyOfflineTableManager(config);
+    File tempRootDir = tmgr.getTmpSegmentDataDir("test-download-decrypt-peer");
+
+    // As the case 2 description says, we need to mock the static method fetchAndDecryptSegmentToLocal to
+    // throw the AttemptExceed exception; Due to the constraint that mockito static cannot do argument matching,
+    // e.g., any(), we have to pass exact argument value when mocking fetchAndDecryptSegmentToLocal.
+    // However, the second argument of File is internally created, which cannot be mocked.
+    // Luckily, the File class's equal method only compares the path. Thus, we can create a file with identical path
+    // and use it to mock the fetchAndDecryptSegmentToLocal
+    File destFile = new File(tempRootDir, "seg01" + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION);
+    doNothing().when(tmgr).downloadFromPeersWithoutStreaming("seg01", zkmd, destFile);
+    try (MockedStatic<SegmentFetcherFactory> mockSegFactory = mockStatic(SegmentFetcherFactory.class)) {
+      mockSegFactory.when(() -> SegmentFetcherFactory.fetchAndDecryptSegmentToLocal(backupCopyURI, destFile, null))
+          .thenThrow(new AttemptsExceededException("fake attempt exceeds exception"));
+      tmgr.downloadAndDecrypt("seg01", zkmd, tempRootDir);
+    }
+    verify(tmgr, times(1)).downloadFromPeersWithoutStreaming("seg01", zkmd, destFile);
+  }
+
+  // happy case: download from peers
+  @Test
+  public void testDownloadFromPeersWithoutStreaming() throws Exception {
+    URI uri = mockRemoteCopy();
+    TableDataManagerConfig config = createDefaultTableDataManagerConfig();
+    when(config.getTablePeerDownloadScheme()).thenReturn("http");
+    HelixManager mockedHelix = mock(HelixManager.class);
+    BaseTableDataManager tmgr = createTableManager(config, mockedHelix);
+    File tempRootDir = tmgr.getTmpSegmentDataDir("test-download-peer-without-streaming");
+    File destFile = new File(tempRootDir, "seg01" + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION);
+    try (MockedStatic<PeerServerSegmentFinder> mockPeerSegFinder = mockStatic(PeerServerSegmentFinder.class)) {
+      mockPeerSegFinder.when(() -> PeerServerSegmentFinder.getPeerServerURIs(
+          "seg01", "http", mockedHelix, TABLE_NAME_WITH_TYPE))
+          .thenReturn(Collections.singletonList(uri));
+      tmgr.downloadFromPeersWithoutStreaming("seg01", mock(SegmentZKMetadata.class), destFile);
+    }
+    assertEquals(FileUtils.readFileToString(destFile), "this is from somewhere remote");
+  }
+
   @Test
   public void testUntarAndMoveSegment()
       throws IOException {
@@ -505,31 +716,8 @@ public class BaseTableDataManagerTest {
     }
   }
 
-  private static void initSegmentFetcher()
-      throws Exception {
-    Map<String, Object> properties = new HashMap<>();
-    properties.put(RETRY_COUNT_CONFIG_KEY, 3);
-    properties.put(RETRY_WAIT_MS_CONFIG_KEY, 100);
-    properties.put(RETRY_DELAY_SCALE_FACTOR_CONFIG_KEY, 5);
-    SegmentFetcherFactory.init(new PinotConfiguration(properties));
-
-    // Setup crypter
-    properties.put("class.fakePinotCrypter", FakePinotCrypter.class.getName());
-    PinotCrypterFactory.init(new PinotConfiguration(properties));
-  }
-
-  private static IndexLoadingConfig createIndexLoadingConfig() {
-    IndexLoadingConfig indexLoadingConfig = new IndexLoadingConfig();
-    indexLoadingConfig.setSegmentVersion(SegmentVersion.v3);
-    indexLoadingConfig.setReadMode(ReadMode.mmap);
-    return indexLoadingConfig;
-  }
-
   private static BaseTableDataManager createTableManager() {
-    TableDataManagerConfig config = mock(TableDataManagerConfig.class);
-    when(config.getTableName()).thenReturn(TABLE_NAME);
-    when(config.getDataDir()).thenReturn(TABLE_DATA_DIR.getAbsolutePath());
-    when(config.getAuthConfig()).thenReturn(new MapConfiguration(Collections.emptyMap()));
+    TableDataManagerConfig config = createDefaultTableDataManagerConfig();
 
     OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
     tableDataManager.init(config, "dummyInstance", mock(ZkHelixPropertyStore.class),
@@ -539,20 +727,38 @@ public class BaseTableDataManagerTest {
     return tableDataManager;
   }
 
+  private static BaseTableDataManager createTableManager(TableDataManagerConfig config, HelixManager helixManager) {
+    OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
+    tableDataManager.init(config, "dummyInstance", mock(ZkHelixPropertyStore.class),
+        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), helixManager, null,
+        new TableDataManagerParams(0, false, -1));
+    tableDataManager.start();
+    return tableDataManager;
+  }
+
+  private static OfflineTableDataManager createSpyOfflineTableManager(TableDataManagerConfig tableDataManagerConfig) {
+    OfflineTableDataManager tableDataManager = new OfflineTableDataManager();
+    tableDataManager.init(tableDataManagerConfig, "dummyInstance", mock(ZkHelixPropertyStore.class),
+        new ServerMetrics(PinotMetricUtils.getPinotMetricsRegistry()), mock(HelixManager.class), null,
+        new TableDataManagerParams(0, false, -1));
+    tableDataManager.start();
+    return Mockito.spy(tableDataManager);
+  }
+
+  private static TableDataManagerConfig createDefaultTableDataManagerConfig() {
+    TableDataManagerConfig config = mock(TableDataManagerConfig.class);
+    when(config.getTableName()).thenReturn(TABLE_NAME_WITH_TYPE);
+    when(config.getDataDir()).thenReturn(TABLE_DATA_DIR.getAbsolutePath());
+    when(config.getAuthConfig()).thenReturn(new MapConfiguration(Collections.emptyMap()));
+    return config;
+  }
+
   private static SegmentZKMetadata createRawSegment(TableConfig tableConfig, String segName, SegmentVersion segVer,
       int rowCnt)
       throws Exception {
-    File segDir = createSegment(tableConfig, segName, segVer, rowCnt);
-    String segCrc = getCRC(segDir, SegmentVersion.v3);
-
-    SegmentZKMetadata zkmd = mock(SegmentZKMetadata.class);
-    File tempTar = new File(TEMP_DIR, segName + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION);
-    TarGzCompressionUtils.createTarGzFile(segDir, tempTar);
-    when(zkmd.getDownloadUrl()).thenReturn("file://" + tempTar.getAbsolutePath());
-    when(zkmd.getCrc()).thenReturn(Long.valueOf(segCrc));
-
-    FileUtils.deleteQuietly(segDir);
-    return zkmd;
+    File localSegDir = createSegment(tableConfig, segName, segVer, rowCnt);
+    return TableDataManagerTestUtils.makeRawSegment(segName, localSegDir,
+        new File(TEMP_DIR, segName + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION), true);
   }
 
   private static File createSegment(TableConfig tableConfig, String segName, SegmentVersion segVer, int rowCnt)
@@ -576,18 +782,6 @@ public class BaseTableDataManagerTest {
     return new File(TABLE_DATA_DIR, segName);
   }
 
-  private static String getCRC(File segDir, SegmentVersion segVer)
-      throws IOException {
-    File parentDir = segDir;
-    if (segVer == SegmentVersion.v3) {
-      parentDir = new File(segDir, "v3");
-    }
-    File crcFile = new File(parentDir, V1Constants.SEGMENT_CREATION_META);
-    try (DataInputStream ds = new DataInputStream(new FileInputStream(crcFile))) {
-      return String.valueOf(ds.readLong());
-    }
-  }
-
   private static boolean hasInvertedIndex(File segDir, String colName, SegmentVersion segVer)
       throws IOException {
     File parentDir = segDir;
@@ -597,4 +791,20 @@ public class BaseTableDataManagerTest {
     File idxMapFile = new File(parentDir, V1Constants.INDEX_MAP_FILE_NAME);
     return FileUtils.readFileToString(idxMapFile).contains(colName + ".inverted_index");
   }
+
+  private TableConfig createTableConfigWithTier(String tierName, File dataDir) {
+    return new TableConfigBuilder(TableType.OFFLINE).setTableName(TABLE_NAME).setTierConfigList(
+        Collections.singletonList(new TierConfig(tierName, TierFactory.TIME_SEGMENT_SELECTOR_TYPE, "3d", null,
+            TierFactory.PINOT_SERVER_STORAGE_TYPE, "tag_OFFLINE", null,
+            Collections.singletonMap("dataDir", dataDir.getAbsolutePath())))).build();
+  }
+
+  private static URI mockRemoteCopy() throws IOException, URISyntaxException {
+    File tempInput = new File(TEMP_DIR, "tmp.txt");
+    FileUtils.write(tempInput, "this is from somewhere remote");
+
+    String backupCopyURI = "file://" + tempInput.getAbsolutePath();
+    URI uri = new URI(backupCopyURI);
+    return uri;
+  }
 }
