@@ -30,6 +30,7 @@ import org.apache.helix.zookeeper.datamodel.ZNRecord;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.metrics.ServerMetrics;
 import org.apache.pinot.common.utils.SchemaUtils;
+import org.apache.pinot.common.utils.config.TableConfigUtils;
 import org.apache.pinot.segment.local.data.manager.SegmentDataManager;
 import org.apache.pinot.segment.local.data.manager.TableDataManagerConfig;
 import org.apache.pinot.segment.local.data.manager.TableDataManagerParams;
@@ -41,12 +42,17 @@ import org.apache.pinot.segment.spi.SegmentMetadata;
 import org.apache.pinot.segment.spi.creator.SegmentGeneratorConfig;
 import org.apache.pinot.segment.spi.creator.SegmentIndexCreationDriver;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
+import org.apache.pinot.spi.config.table.DimensionTableConfig;
+import org.apache.pinot.spi.config.table.TableConfig;
+import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.readers.GenericRow;
 import org.apache.pinot.spi.data.readers.PrimaryKey;
 import org.apache.pinot.spi.metrics.PinotMetricUtils;
+import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
+import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 import org.testng.annotations.AfterClass;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
@@ -61,7 +67,8 @@ import static org.testng.Assert.assertNull;
 @SuppressWarnings("unchecked")
 public class DimensionTableDataManagerTest {
   private static final File TEMP_DIR = new File(FileUtils.getTempDirectory(), LoaderTest.class.getName());
-  private static final String TABLE_NAME = "dimBaseballTeams";
+  private static final String RAW_TABLE_NAME = "dimBaseballTeams";
+  private static final String OFFLINE_TABLE_NAME = TableNameBuilder.OFFLINE.tableNameWithType(RAW_TABLE_NAME);
   private static final String AVRO_DATA_PATH = "data/dimBaseballTeams.avro";
 
   private File _indexDir;
@@ -79,7 +86,7 @@ public class DimensionTableDataManagerTest {
 
     // create segment
     SegmentGeneratorConfig segmentGeneratorConfig =
-        SegmentTestUtils.getSegmentGeneratorConfigWithoutTimeColumn(avroFile, TEMP_DIR, TABLE_NAME);
+        SegmentTestUtils.getSegmentGeneratorConfigWithoutTimeColumn(avroFile, TEMP_DIR, RAW_TABLE_NAME);
     SegmentIndexCreationDriver driver = SegmentCreationDriverFactory.get(null);
     driver.init(segmentGeneratorConfig);
     driver.build();
@@ -103,6 +110,12 @@ public class DimensionTableDataManagerTest {
         .setPrimaryKeyColumns(Collections.singletonList("teamID")).build();
   }
 
+  private TableConfig getTableConfig(boolean disablePreload) {
+    DimensionTableConfig dimensionTableConfig = new DimensionTableConfig(disablePreload);
+    return new TableConfigBuilder(TableType.OFFLINE).setTableName("dimBaseballTeams").setSchemaName("dimBaseballTeams")
+        .setDimensionTableConfig(dimensionTableConfig).build();
+  }
+
   private Schema getSchemaWithExtraColumn() {
     return new Schema.SchemaBuilder().setSchemaName("dimBaseballTeams")
         .addSingleValueDimension("teamID", DataType.STRING).addSingleValueDimension("teamName", DataType.STRING)
@@ -111,11 +124,12 @@ public class DimensionTableDataManagerTest {
   }
 
   private DimensionTableDataManager makeTableDataManager(HelixManager helixManager) {
-    DimensionTableDataManager tableDataManager = DimensionTableDataManager.createInstanceByTableName(TABLE_NAME);
+    DimensionTableDataManager tableDataManager =
+        DimensionTableDataManager.createInstanceByTableName(OFFLINE_TABLE_NAME);
     TableDataManagerConfig config;
     {
       config = mock(TableDataManagerConfig.class);
-      when(config.getTableName()).thenReturn(TABLE_NAME);
+      when(config.getTableName()).thenReturn(OFFLINE_TABLE_NAME);
       when(config.getDataDir()).thenReturn(TEMP_DIR.getAbsolutePath());
     }
     tableDataManager.init(config, "dummyInstance", helixManager.getHelixPropertyStore(),
@@ -134,10 +148,10 @@ public class DimensionTableDataManagerTest {
         SchemaUtils.toZNRecord(getSchema()));
     when(helixManager.getHelixPropertyStore()).thenReturn(propertyStore);
     DimensionTableDataManager tableDataManager = makeTableDataManager(helixManager);
-    assertEquals(tableDataManager.getTableName(), TABLE_NAME);
+    assertEquals(tableDataManager.getTableName(), OFFLINE_TABLE_NAME);
 
     // fetch the same instance via static method
-    DimensionTableDataManager returnedManager = DimensionTableDataManager.getInstanceByTableName(TABLE_NAME);
+    DimensionTableDataManager returnedManager = DimensionTableDataManager.getInstanceByTableName(OFFLINE_TABLE_NAME);
     assertNotNull(returnedManager, "Manager should find instance");
     assertEquals(tableDataManager, returnedManager, "Manager should return already created instance");
 
@@ -239,4 +253,49 @@ public class DimensionTableDataManagerTest {
     assertEquals(resp.getValue("teamName"), "San Francisco Giants");
     assertEquals(resp.getValue("teamCity"), "null");
   }
+
+  @Test
+  public void testLookupWithoutPreLoad()
+      throws Exception {
+    HelixManager helixManager = mock(HelixManager.class);
+    ZkHelixPropertyStore<ZNRecord> propertyStore = mock(ZkHelixPropertyStore.class);
+    when(propertyStore.get("/SCHEMAS/dimBaseballTeams", null, AccessOption.PERSISTENT)).thenReturn(
+        SchemaUtils.toZNRecord(getSchema()));
+    when(propertyStore.get("/CONFIGS/TABLE/dimBaseballTeams", null, AccessOption.PERSISTENT)).thenReturn(
+        TableConfigUtils.toZNRecord(getTableConfig(true)));
+    when(helixManager.getHelixPropertyStore()).thenReturn(propertyStore);
+    DimensionTableDataManager tableDataManager = makeTableDataManager(helixManager);
+
+    // try fetching data BEFORE loading segment
+    GenericRow resp = tableDataManager.lookupRowByPrimaryKey(new PrimaryKey(new String[]{"SF"}));
+    assertNull(resp, "Response should be null if no segment is loaded");
+
+    tableDataManager.addSegment(_indexDir, _indexLoadingConfig);
+
+    // Confirm table is loaded and available for lookup
+    resp = tableDataManager.lookupRowByPrimaryKey(new PrimaryKey(new String[]{"SF"}));
+    assertNotNull(resp, "Should return response after segment load");
+    assertEquals(resp.getFieldToValueMap().size(), 2);
+    assertEquals(resp.getValue("teamID"), "SF");
+    assertEquals(resp.getValue("teamName"), "San Francisco Giants");
+
+    // Confirm we can get FieldSpec for loaded tables columns.
+    FieldSpec spec = tableDataManager.getColumnFieldSpec("teamName");
+    assertNotNull(spec, "Should return spec for existing column");
+    assertEquals(spec.getDataType(), DataType.STRING, "Should return correct data type for teamName column");
+
+    // Confirm we can read primary column list
+    List<String> pkColumns = tableDataManager.getPrimaryKeyColumns();
+    assertEquals(pkColumns, Collections.singletonList("teamID"), "Should return PK column list");
+
+    // Remove the segment
+    List<SegmentDataManager> segmentManagers = tableDataManager.acquireAllSegments();
+    assertEquals(segmentManagers.size(), 1, "Should have exactly one segment manager");
+    SegmentDataManager segMgr = segmentManagers.get(0);
+    String segmentName = segMgr.getSegmentName();
+    tableDataManager.removeSegment(segmentName);
+    // confirm table is cleaned up
+    resp = tableDataManager.lookupRowByPrimaryKey(new PrimaryKey(new String[]{"SF"}));
+    assertNull(resp, "Response should be null if no segment is loaded");
+  }
 }
