@@ -29,7 +29,7 @@ import java.util.concurrent.Executors;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.utils.DataSchema;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.SelectionResultsBlock;
 import org.apache.pinot.core.plan.CombinePlanNode;
 import org.apache.pinot.core.plan.PlanNode;
 import org.apache.pinot.core.plan.maker.InstancePlanMakerImplV2;
@@ -105,7 +105,8 @@ public class SelectionCombineOperatorTest {
     }
   }
 
-  private IndexSegment createRealtimeSegment(int index) throws Exception {
+  private IndexSegment createRealtimeSegment(int index)
+      throws Exception {
     RealtimeSegmentStatsHistory statsHistory = mock(RealtimeSegmentStatsHistory.class);
     when(statsHistory.getEstimatedCardinality(anyString())).thenReturn(200);
     when(statsHistory.getEstimatedAvgColSize(anyString())).thenReturn(32);
@@ -115,7 +116,7 @@ public class SelectionCombineOperatorTest {
     RealtimeSegmentConfig realtimeSegmentConfig = new RealtimeSegmentConfig.Builder()
         .setTableNameWithType(REALTIME_TABLE_NAME).setSegmentName(segmentName).setSchema(SCHEMA).setCapacity(100000)
         .setAvgNumMultiValues(2).setNoDictionaryColumns(Collections.emptySet())
-        .setJsonIndexColumns(Collections.emptySet()).setVarLengthDictionaryColumns(Collections.emptySet())
+        .setJsonIndexConfigs(Collections.emptyMap()).setVarLengthDictionaryColumns(Collections.emptySet())
         .setInvertedIndexColumns(Collections.emptySet()).setSegmentZKMetadata(new SegmentZKMetadata(segmentName))
         .setMemoryManager(new DirectMemoryManager(segmentName)).setStatsHistory(statsHistory).setAggregateMetrics(false)
         .setNullHandlingEnabled(true).setIngestionAggregationConfigs(Collections.emptyList()).build();
@@ -154,11 +155,10 @@ public class SelectionCombineOperatorTest {
 
   @Test
   public void testSelectionLimit0() {
-    IntermediateResultsBlock combineResult = getCombineResult("SELECT * FROM testTable LIMIT 0");
+    SelectionResultsBlock combineResult = getCombineResult("SELECT * FROM testTable LIMIT 0");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    assertNotNull(combineResult.getSelectionResult());
-    assertTrue(combineResult.getSelectionResult().isEmpty());
+    assertTrue(combineResult.getRows().isEmpty());
     assertEquals(combineResult.getNumDocsScanned(), 0);
     assertEquals(combineResult.getNumEntriesScannedInFilter(), 0);
     assertEquals(combineResult.getNumEntriesScannedPostFilter(), 0);
@@ -171,11 +171,10 @@ public class SelectionCombineOperatorTest {
 
   @Test
   public void testSelectionOnly() {
-    IntermediateResultsBlock combineResult = getCombineResult("SELECT * FROM testTable");
+    SelectionResultsBlock combineResult = getCombineResult("SELECT * FROM testTable");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    assertNotNull(combineResult.getSelectionResult());
-    assertEquals(combineResult.getSelectionResult().size(), 10);
+    assertEquals(combineResult.getRows().size(), 10);
     // Should early-terminate after processing the result of the first segment. Each thread should process at most 1
     // segment.
     long numDocsScanned = combineResult.getNumDocsScanned();
@@ -184,16 +183,30 @@ public class SelectionCombineOperatorTest {
     assertEquals(combineResult.getNumEntriesScannedPostFilter(), numDocsScanned);
     assertEquals(combineResult.getNumSegmentsProcessed(), NUM_SEGMENTS);
     assertEquals(combineResult.getNumConsumingSegmentsProcessed(), NUM_CONSUMING_SEGMENTS);
-    assertEquals(combineResult.getNumConsumingSegmentsMatched(), 0);
     int numSegmentsMatched = combineResult.getNumSegmentsMatched();
     assertTrue(numSegmentsMatched >= 1 && numSegmentsMatched <= CombineOperatorUtils.MAX_NUM_THREADS_PER_QUERY);
+    // The check below depends on the order of segment processing. When segments# <= 10 (the value of
+    // CombinePlanNode.TARGET_NUM_PLANS_PER_THREAD to be specific), the segments are processed in the order as they
+    // are prepared, which is OFFLINE segments followed by RT segments and this case makes the value here equal to 0.
+    // But when segments# > 10, the segments are processed in a different order and some RT segments can be processed
+    // ahead of the other OFFLINE segments, but no more than CombineOperatorUtils.MAX_NUM_THREADS_PER_QUERY for sure
+    // as each thread only processes one segment.
+    int numConsumingSegmentsMatched = combineResult.getNumConsumingSegmentsMatched();
+    if (NUM_SEGMENTS <= 10) {
+      assertEquals(numConsumingSegmentsMatched, 0, "numSegments: " + NUM_SEGMENTS);
+    } else {
+      assertTrue(numConsumingSegmentsMatched >= 0
+          && numConsumingSegmentsMatched <= CombineOperatorUtils.MAX_NUM_THREADS_PER_QUERY, String
+          .format("numConsumingSegmentsMatched: %d, maxThreadsPerQuery: %d, numSegments: %d",
+              combineResult.getNumConsumingSegmentsMatched(), CombineOperatorUtils.MAX_NUM_THREADS_PER_QUERY,
+              NUM_SEGMENTS));
+    }
     assertEquals(combineResult.getNumTotalDocs(), NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
 
     combineResult = getCombineResult("SELECT * FROM testTable LIMIT 10000");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    assertNotNull(combineResult.getSelectionResult());
-    assertEquals(combineResult.getSelectionResult().size(), NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
+    assertEquals(combineResult.getRows().size(), NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
     // Should not early-terminate
     numDocsScanned = combineResult.getNumDocsScanned();
     assertEquals(numDocsScanned, NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
@@ -208,10 +221,10 @@ public class SelectionCombineOperatorTest {
 
   @Test
   public void testSelectionOrderBy() {
-    IntermediateResultsBlock combineResult = getCombineResult("SELECT * FROM testTable ORDER BY intColumn");
+    SelectionResultsBlock combineResult = getCombineResult("SELECT * FROM testTable ORDER BY intColumn");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    PriorityQueue<Object[]> selectionResult = (PriorityQueue<Object[]>) combineResult.getSelectionResult();
+    PriorityQueue<Object[]> selectionResult = combineResult.getRowsAsPriorityQueue();
     assertNotNull(selectionResult);
     assertEquals(selectionResult.size(), 10);
     int expectedValue = 9;
@@ -235,7 +248,7 @@ public class SelectionCombineOperatorTest {
     combineResult = getCombineResult("SELECT * FROM testTable ORDER BY intColumn DESC");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    selectionResult = (PriorityQueue<Object[]>) combineResult.getSelectionResult();
+    selectionResult = combineResult.getRowsAsPriorityQueue();
     assertNotNull(selectionResult);
     assertEquals(selectionResult.size(), 10);
     expectedValue = NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT / 2 + 40;
@@ -259,7 +272,7 @@ public class SelectionCombineOperatorTest {
     combineResult = getCombineResult("SELECT * FROM testTable ORDER BY intColumn DESC LIMIT 10000");
     assertEquals(combineResult.getDataSchema(),
         new DataSchema(new String[]{INT_COLUMN}, new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT}));
-    selectionResult = (PriorityQueue<Object[]>) combineResult.getSelectionResult();
+    selectionResult = combineResult.getRowsAsPriorityQueue();
     assertNotNull(selectionResult);
     assertEquals(selectionResult.size(), NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
     // Should not early-terminate
@@ -274,7 +287,7 @@ public class SelectionCombineOperatorTest {
     assertEquals(combineResult.getNumTotalDocs(), NUM_SEGMENTS * NUM_RECORDS_PER_SEGMENT);
   }
 
-  private IntermediateResultsBlock getCombineResult(String query) {
+  private SelectionResultsBlock getCombineResult(String query) {
     QueryContext queryContext = QueryContextConverterUtils.getQueryContext(query);
     List<PlanNode> planNodes = new ArrayList<>(NUM_SEGMENTS);
     for (IndexSegment indexSegment : _indexSegments) {
@@ -282,7 +295,7 @@ public class SelectionCombineOperatorTest {
     }
     queryContext.setEndTimeMs(System.currentTimeMillis() + Server.DEFAULT_QUERY_EXECUTOR_TIMEOUT_MS);
     CombinePlanNode combinePlanNode = new CombinePlanNode(planNodes, queryContext, EXECUTOR, null);
-    return combinePlanNode.run().nextBlock();
+    return (SelectionResultsBlock) combinePlanNode.run().nextBlock();
   }
 
   @AfterClass
