@@ -38,13 +38,14 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.LongAccumulator;
 import javax.annotation.Nullable;
 import org.apache.commons.configuration.PropertiesConfiguration;
+import org.apache.pinot.common.datatable.DataTable;
+import org.apache.pinot.common.datatable.DataTable.MetadataKey;
+import org.apache.pinot.common.datatable.DataTableFactory;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.metrics.ServerMetrics;
 import org.apache.pinot.common.proto.Server;
-import org.apache.pinot.common.utils.DataTable;
-import org.apache.pinot.common.utils.DataTable.MetadataKey;
-import org.apache.pinot.core.common.datatable.DataTableFactory;
 import org.apache.pinot.core.data.manager.InstanceDataManager;
+import org.apache.pinot.core.operator.blocks.InstanceResponseBlock;
 import org.apache.pinot.core.query.executor.QueryExecutor;
 import org.apache.pinot.core.query.request.ServerQueryRequest;
 import org.apache.pinot.core.query.scheduler.resources.PolicyBasedResourceManager;
@@ -296,7 +297,7 @@ public class PrioritySchedulerTest {
     }
 
     @Override
-    public DataTable processQuery(ServerQueryRequest queryRequest, ExecutorService executorService,
+    public InstanceResponseBlock execute(ServerQueryRequest queryRequest, ExecutorService executorService,
         @Nullable StreamObserver<Server.ServerResponse> responseObserver) {
       if (_useBarrier) {
         try {
@@ -305,8 +306,8 @@ public class PrioritySchedulerTest {
           throw new RuntimeException(e);
         }
       }
-      DataTable result = DataTableFactory.getEmptyDataTable();
-      result.getMetadata().put(MetadataKey.TABLE.getName(), queryRequest.getTableNameWithType());
+      InstanceResponseBlock instanceResponse = new InstanceResponseBlock();
+      instanceResponse.addMetadata(MetadataKey.TABLE.getName(), queryRequest.getTableNameWithType());
       if (_useBarrier) {
         try {
           _validationBarrier.await();
@@ -315,7 +316,7 @@ public class PrioritySchedulerTest {
         }
       }
       _numQueries.countDown();
-      return result;
+      return instanceResponse;
     }
   }
 }
