@@ -18,24 +18,14 @@
  */
 package org.apache.pinot.queries;
 
-import java.util.List;
-import org.apache.pinot.common.utils.DataSchema;
-import org.apache.pinot.core.data.table.Record;
-import org.apache.pinot.core.operator.BaseOperator;
-import org.apache.pinot.core.operator.blocks.IntermediateResultsBlock;
-import org.apache.pinot.core.operator.query.AggregationGroupByOrderByOperator;
+import org.apache.pinot.core.operator.blocks.results.AggregationResultsBlock;
+import org.apache.pinot.core.operator.blocks.results.GroupByResultsBlock;
 import org.apache.pinot.core.operator.query.AggregationOperator;
-import org.apache.pinot.core.operator.query.DistinctOperator;
 import org.apache.pinot.core.operator.query.FilteredAggregationOperator;
-import org.apache.pinot.core.query.distinct.DistinctTable;
+import org.apache.pinot.core.operator.query.GroupByOperator;
 import org.testng.annotations.Test;
 
-import static org.testng.Assert.assertEquals;
-import static org.testng.Assert.assertNotNull;
-import static org.testng.Assert.assertTrue;
 
-
-@SuppressWarnings("ConstantConditions")
 public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleValueQueriesTest {
   private static final String AGGREGATION_QUERY =
       "SELECT COUNT(*), SUM(column1), MAX(column3), MIN(column6), AVG(column7) FROM testTable";
@@ -54,19 +44,19 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
   public void testAggregationOnly() {
     // Test query without filter.
     AggregationOperator aggregationOperator = getOperator(AGGREGATION_QUERY);
-    IntermediateResultsBlock resultsBlock = aggregationOperator.nextBlock();
+    AggregationResultsBlock resultsBlock = aggregationOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 30000L, 0L,
         120000L, 30000L);
-    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getAggregationResult(), 30000L, 32317185437847L,
-        2147419555, 1689277, 28175373944314L, 30000L);
+    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getResults(), 30000L, 32317185437847L, 2147419555,
+        1689277, 28175373944314L, 30000L);
 
     // Test query with filter.
     aggregationOperator = getOperator(AGGREGATION_QUERY + FILTER);
     resultsBlock = aggregationOperator.nextBlock();
-    QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 6129L, 84134L,
+    QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 6129L, 63064L,
         24516L, 30000L);
-    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getAggregationResult(), 6129L, 6875947596072L,
-        999813884, 1980174, 4699510391301L, 6129L);
+    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getResults(), 6129L, 6875947596072L, 999813884,
+        1980174, 4699510391301L, 6129L);
   }
 
   @Test
@@ -75,11 +65,11 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
         + "MAX(column3) FILTER(WHERE column3 IS NOT NULL), SUM(column3), AVG(column7) FILTER(WHERE column7 > 0) "
         + "FROM testTable WHERE column3 > 0";
     FilteredAggregationOperator aggregationOperator = getOperator(query);
-    IntermediateResultsBlock resultsBlock = aggregationOperator.nextBlock();
+    AggregationResultsBlock resultsBlock = aggregationOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 180000L, 0L,
         540000L, 30000L);
-    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getAggregationResult(), 22266008882250L, 30000,
-        2147419555, 32289159189150L, 28175373944314L, 30000L);
+    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getResults(), 22266008882250L, 30000, 2147419555,
+        32289159189150L, 28175373944314L, 30000L);
 
     query = "SELECT SUM(column6) FILTER(WHERE column6 > 5), COUNT(*) FILTER(WHERE column1 IS NOT NULL), "
         + "MAX(column3) FILTER(WHERE column3 IS NOT NULL), SUM(column3), AVG(column7) FILTER(WHERE column7 > 0) "
@@ -88,8 +78,8 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     resultsBlock = aggregationOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 180000L, 0L,
         540000L, 30000L);
-    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getAggregationResult(), 22266008882250L, 30000,
-        2147419555, 32289159189150L, 28175373944314L, 30000L);
+    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getResults(), 22266008882250L, 30000, 2147419555,
+        32289159189150L, 28175373944314L, 30000L);
 
     query = "SELECT SUM(column6) FILTER(WHERE column6 > 5 OR column6 < 15), "
         + "COUNT(*) FILTER(WHERE column1 IS NOT NULL), MAX(column3) FILTER(WHERE column3 IS NOT NULL AND column3 > 0), "
@@ -98,15 +88,15 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     resultsBlock = aggregationOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(aggregationOperator.getExecutionStatistics(), 150000L, 0L,
         450000L, 30000L);
-    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getAggregationResult(), 22266008882250L, 30000,
-        2147419555, 32289159189150L, 0L, 0L);
+    QueriesTestUtils.testInnerSegmentAggregationResult(resultsBlock.getResults(), 22266008882250L, 30000, 2147419555,
+        32289159189150L, 0L, 0L);
   }
 
   @Test
   public void testSmallAggregationGroupBy() {
     // Test query without filter.
-    AggregationGroupByOrderByOperator groupByOperator = getOperator(AGGREGATION_QUERY + SMALL_GROUP_BY);
-    IntermediateResultsBlock resultsBlock = groupByOperator.nextBlock();
+    GroupByOperator groupByOperator = getOperator(AGGREGATION_QUERY + SMALL_GROUP_BY);
+    GroupByResultsBlock resultsBlock = groupByOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 30000L, 0L, 150000L,
         30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
@@ -115,7 +105,7 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     // Test query with filter.
     groupByOperator = getOperator(AGGREGATION_QUERY + FILTER + SMALL_GROUP_BY);
     resultsBlock = groupByOperator.nextBlock();
-    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 84134L,
+    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 63064L,
         30645L, 30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
         new Object[]{242920}, 3L, 4348938306L, 407993712, 296467636, 5803888725L, 3L);
@@ -124,8 +114,8 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
   @Test
   public void testMediumAggregationGroupBy() {
     // Test query without filter.
-    AggregationGroupByOrderByOperator groupByOperator = getOperator(AGGREGATION_QUERY + MEDIUM_GROUP_BY);
-    IntermediateResultsBlock resultsBlock = groupByOperator.nextBlock();
+    GroupByOperator groupByOperator = getOperator(AGGREGATION_QUERY + MEDIUM_GROUP_BY);
+    GroupByResultsBlock resultsBlock = groupByOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 30000L, 0L, 210000L,
         30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
@@ -134,7 +124,7 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     // Test query with filter.
     groupByOperator = getOperator(AGGREGATION_QUERY + FILTER + MEDIUM_GROUP_BY);
     resultsBlock = groupByOperator.nextBlock();
-    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 84134L,
+    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 63064L,
         42903L, 30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
         new Object[]{1176631727, "P", "KrNxpdycSiwoRohEiTIlLqDHnx"}, 1L, 716185211L, 489993380, 371110078, 487714191L,
@@ -144,8 +134,8 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
   @Test
   public void testLargeAggregationGroupBy() {
     // Test query without filter.
-    AggregationGroupByOrderByOperator groupByOperator = getOperator(AGGREGATION_QUERY + LARGE_GROUP_BY);
-    IntermediateResultsBlock resultsBlock = groupByOperator.nextBlock();
+    GroupByOperator groupByOperator = getOperator(AGGREGATION_QUERY + LARGE_GROUP_BY);
+    GroupByResultsBlock resultsBlock = groupByOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 30000L, 0L, 210000L,
         30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
@@ -155,7 +145,7 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     // Test query with filter.
     groupByOperator = getOperator(AGGREGATION_QUERY + FILTER + LARGE_GROUP_BY);
     resultsBlock = groupByOperator.nextBlock();
-    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 84134L,
+    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 63064L,
         42903L, 30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
         new Object[]{1318761745, 353175528, 1172307870, "P", "HEuxNvH"}, 2L, 2637523490L, 557154208, 353175528,
@@ -165,8 +155,8 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
   @Test
   public void testVeryLargeAggregationGroupBy() {
     // Test query without filter.
-    AggregationGroupByOrderByOperator groupByOperator = getOperator(AGGREGATION_QUERY + VERY_LARGE_GROUP_BY);
-    IntermediateResultsBlock resultsBlock = groupByOperator.nextBlock();
+    GroupByOperator groupByOperator = getOperator(AGGREGATION_QUERY + VERY_LARGE_GROUP_BY);
+    GroupByResultsBlock resultsBlock = groupByOperator.nextBlock();
     QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 30000L, 0L, 270000L,
         30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(),
@@ -176,69 +166,10 @@ public class InnerSegmentAggregationSingleValueQueriesTest extends BaseSingleVal
     // Test query with filter.
     groupByOperator = getOperator(AGGREGATION_QUERY + FILTER + VERY_LARGE_GROUP_BY);
     resultsBlock = groupByOperator.nextBlock();
-    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 84134L,
+    QueriesTestUtils.testInnerSegmentExecutionStatistics(groupByOperator.getExecutionStatistics(), 6129L, 63064L,
         55161L, 30000L);
     QueriesTestUtils.testInnerSegmentAggregationGroupByResult(resultsBlock.getAggregationGroupByResult(), new Object[]{
         1361199163, 178133991, 296467636, 788414092, 1719301234, "P", "MaztCmmxxgguBUxPti", 1284373442, 752388855
     }, 1L, 1361199163L, 178133991, 296467636, 788414092L, 1L);
   }
-
-  /**
-   * Test DISTINCT on single column single segment. Since the dataset
-   * is Avro files, the only thing we currently check
-   * for correctness is the actual number of DISTINCT
-   * records returned
-   */
-  @Test
-  public void testSingleColumnDistinct() {
-    String query = "SELECT DISTINCT column1 FROM testTable LIMIT 1000000";
-    BaseOperator<IntermediateResultsBlock> distinctOperator = getOperator(query);
-    IntermediateResultsBlock resultsBlock = distinctOperator.nextBlock();
-    List<Object> operatorResult = resultsBlock.getAggregationResult();
-
-    assertEquals(operatorResult.size(), 1);
-    assertTrue(operatorResult.get(0) instanceof DistinctTable);
-
-    DistinctTable distinctTable = (DistinctTable) operatorResult.get(0);
-    assertEquals(distinctTable.size(), 6582);
-
-    DataSchema dataSchema = distinctTable.getDataSchema();
-    assertEquals(dataSchema.getColumnNames(), new String[]{"column1"});
-    assertEquals(dataSchema.getColumnDataTypes(), new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT});
-
-    for (Record record : distinctTable.getRecords()) {
-      assertNotNull(record);
-      assertEquals(record.getValues().length, 1);
-    }
-  }
-
-  /**
-   * Test DISTINCT on multiple column single segment. Since the dataset
-   * is Avro files, the only thing we currently check
-   * for correctness is the actual number of DISTINCT
-   * records returned
-   */
-  @Test
-  public void testMultiColumnDistinct() {
-    String query = "SELECT DISTINCT column1, column3 FROM testTable LIMIT 1000000";
-    DistinctOperator distinctOperator = getOperator(query);
-    IntermediateResultsBlock resultsBlock = distinctOperator.nextBlock();
-    List<Object> operatorResult = resultsBlock.getAggregationResult();
-
-    assertEquals(operatorResult.size(), 1);
-    assertTrue(operatorResult.get(0) instanceof DistinctTable);
-
-    DistinctTable distinctTable = (DistinctTable) operatorResult.get(0);
-    assertEquals(distinctTable.size(), 21968);
-
-    DataSchema dataSchema = distinctTable.getDataSchema();
-    assertEquals(dataSchema.getColumnNames(), new String[]{"column1", "column3"});
-    assertEquals(dataSchema.getColumnDataTypes(),
-        new DataSchema.ColumnDataType[]{DataSchema.ColumnDataType.INT, DataSchema.ColumnDataType.INT});
-
-    for (Record record : distinctTable.getRecords()) {
-      assertNotNull(record);
-      assertEquals(record.getValues().length, 2);
-    }
-  }
 }
