@@ -83,12 +83,6 @@ public class AggregateMetricsClusterIntegrationTest extends BaseClusterIntegrati
     waitForAllDocsLoaded(600_000L);
   }
 
-  @Override
-  protected boolean useLlc() {
-    // NOTE: Aggregate metrics is only available with LLC.
-    return true;
-  }
-
   @Override
   protected void waitForAllDocsLoaded(long timeoutMs) {
     // NOTE: For aggregate metrics, we need to test the aggregation result instead of the document count because
