@@ -79,11 +79,6 @@ public class ControllerPeriodicTasksIntegrationTest extends BaseClusterIntegrati
     return _currentTable;
   }
 
-  @Override
-  protected boolean useLlc() {
-    return true;
-  }
-
   @Override
   protected int getNumReplicas() {
     return NUM_REPLICAS;
