@@ -36,6 +36,7 @@ import org.apache.pinot.common.metadata.ZKMetadataProvider;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
 import org.apache.pinot.common.utils.FileUploadDownloadClient;
 import org.apache.pinot.controller.ControllerConf;
+import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.data.Schema;
@@ -55,13 +56,11 @@ import static org.testng.Assert.assertTrue;
 
 
 /**
- * Integration test that extends RealtimeClusterIntegrationTest but uses low-level Kafka consumer.
+ * Integration test for low-level Kafka consumer.
  * TODO: Add separate module-level tests and remove the randomness of this test
  */
-public class LLCRealtimeClusterIntegrationTest extends RealtimeClusterIntegrationTest {
+public class LLCRealtimeClusterIntegrationTest extends BaseRealtimeClusterIntegrationTest {
   private static final String CONSUMER_DIRECTORY = "/tmp/consumer-test";
-  private static final String TEST_UPDATED_INVERTED_INDEX_QUERY =
-      "SELECT COUNT(*) FROM mytable WHERE DivActualElapsedTime = 305";
   private static final List<String> UPDATED_INVERTED_INDEX_COLUMNS = Collections.singletonList("DivActualElapsedTime");
   private static final long RANDOM_SEED = System.currentTimeMillis();
   private static final Random RANDOM = new Random(RANDOM_SEED);
@@ -77,11 +76,6 @@ public class LLCRealtimeClusterIntegrationTest extends RealtimeClusterIntegratio
     return true;
   }
 
-  @Override
-  protected boolean useLlc() {
-    return true;
-  }
-
   @Override
   protected String getLoadMode() {
     return ReadMode.mmap.name();
@@ -155,17 +149,17 @@ public class LLCRealtimeClusterIntegrationTest extends RealtimeClusterIntegratio
         if (changeCrc) {
           changeCrcInSegmentZKMetadata(tableName, segmentTarFile.toString());
         }
-        assertEquals(fileUploadDownloadClient
-            .uploadSegment(uploadSegmentHttpURI, segmentTarFile.getName(), segmentTarFile, tableName,
-                TableType.REALTIME).getStatusCode(), HttpStatus.SC_OK);
+        assertEquals(
+            fileUploadDownloadClient.uploadSegment(uploadSegmentHttpURI, segmentTarFile.getName(), segmentTarFile,
+                tableName, TableType.REALTIME).getStatusCode(), HttpStatus.SC_OK);
       } else {
         // Upload segments in parallel
         ExecutorService executorService = Executors.newFixedThreadPool(numSegments);
         List<Future<Integer>> futures = new ArrayList<>(numSegments);
         for (File segmentTarFile : segmentTarFiles) {
-          futures.add(executorService.submit(() -> fileUploadDownloadClient
-              .uploadSegment(uploadSegmentHttpURI, segmentTarFile.getName(), segmentTarFile, tableName,
-                  TableType.REALTIME).getStatusCode()));
+          futures.add(executorService.submit(
+              () -> fileUploadDownloadClient.uploadSegment(uploadSegmentHttpURI, segmentTarFile.getName(),
+                  segmentTarFile, tableName, TableType.REALTIME).getStatusCode()));
         }
         executorService.shutdown();
         for (Future<Integer> future : futures) {
@@ -234,35 +228,6 @@ public class LLCRealtimeClusterIntegrationTest extends RealtimeClusterIntegratio
     }
   }
 
-  @Test
-  public void testInvertedIndexTriggering()
-      throws Exception {
-    long numTotalDocs = getCountStarResult();
-
-    JsonNode queryResponse = postQuery(TEST_UPDATED_INVERTED_INDEX_QUERY);
-    assertEquals(queryResponse.get("totalDocs").asLong(), numTotalDocs);
-    assertTrue(queryResponse.get("numEntriesScannedInFilter").asLong() > 0L);
-
-    TableConfig tableConfig = getRealtimeTableConfig();
-    tableConfig.getIndexingConfig().setInvertedIndexColumns(UPDATED_INVERTED_INDEX_COLUMNS);
-    updateTableConfig(tableConfig);
-    reloadRealtimeTable(getTableName());
-
-    TestUtils.waitForCondition(aVoid -> {
-      try {
-        JsonNode queryResponse1 = postQuery(TEST_UPDATED_INVERTED_INDEX_QUERY);
-        // Total docs should not change during reload
-        assertEquals(queryResponse1.get("totalDocs").asLong(), numTotalDocs);
-        assertEquals(queryResponse1.get("numConsumingSegmentsQueried").asLong(), 2);
-        assertTrue(queryResponse1.get("minConsumingFreshnessTimeMs").asLong() > _startTime);
-        assertTrue(queryResponse1.get("minConsumingFreshnessTimeMs").asLong() < System.currentTimeMillis());
-        return queryResponse1.get("numEntriesScannedInFilter").asLong() == 0;
-      } catch (Exception e) {
-        throw new RuntimeException(e);
-      }
-    }, 600_000L, "Failed to generate inverted index");
-  }
-
   @Test(expectedExceptions = IOException.class)
   public void testAddHLCTableShouldFail()
       throws IOException {
@@ -277,6 +242,67 @@ public class LLCRealtimeClusterIntegrationTest extends RealtimeClusterIntegratio
     testReload(false);
   }
 
+  @Test
+  public void testAddRemoveDictionaryAndInvertedIndex()
+      throws Exception {
+    String query = "SELECT COUNT(*) FROM myTable WHERE ActualElapsedTime = -9999";
+    long numTotalDocs = getCountStarResult();
+
+    JsonNode queryResponse = postQuery(query);
+    assertEquals(queryResponse.get("totalDocs").asLong(), numTotalDocs);
+    // Full table scan without dictionary
+    assertEquals(queryResponse.get("numEntriesScannedInFilter").asLong(), numTotalDocs);
+    long queryResult = queryResponse.get("resultTable").get("rows").get(0).get(0).asLong();
+
+    // Enable dictionary and inverted index.
+    TableConfig tableConfig = getRealtimeTableConfig();
+    IndexingConfig indexingConfig = tableConfig.getIndexingConfig();
+    assertNotNull(indexingConfig.getNoDictionaryColumns());
+    assertNotNull(indexingConfig.getInvertedIndexColumns());
+    indexingConfig.getNoDictionaryColumns().remove("ActualElapsedTime");
+    indexingConfig.getInvertedIndexColumns().add("ActualElapsedTime");
+    updateTableConfig(tableConfig);
+    String enableDictReloadId = reloadTableAndValidateResponse(getTableName(), TableType.REALTIME, false);
+    TestUtils.waitForCondition(aVoid -> {
+      try {
+        // Query result and total docs should not change during reload
+        JsonNode queryResponse1 = postQuery(query);
+        assertEquals(queryResponse1.get("resultTable").get("rows").get(0).get(0).asLong(), queryResult);
+        assertEquals(queryResponse1.get("totalDocs").asLong(), numTotalDocs);
+
+        long numConsumingSegmentsQueried = queryResponse1.get("numConsumingSegmentsQueried").asLong();
+        long minConsumingFreshnessTimeMs = queryResponse1.get("minConsumingFreshnessTimeMs").asLong();
+        return numConsumingSegmentsQueried == 2 && minConsumingFreshnessTimeMs > _startTime
+            && minConsumingFreshnessTimeMs < System.currentTimeMillis() && isReloadJobCompleted(enableDictReloadId);
+      } catch (Exception e) {
+        throw new RuntimeException(e);
+      }
+    }, 600_000L, "Failed to generate dictionary and inverted index");
+    // numEntriesScannedInFilter should be zero with inverted index
+    queryResponse = postQuery(query);
+    assertEquals(queryResponse.get("numEntriesScannedInFilter").asLong(), 0L);
+
+    // Disable dictionary and inverted index.
+    indexingConfig.getNoDictionaryColumns().add("ActualElapsedTime");
+    indexingConfig.getInvertedIndexColumns().remove("ActualElapsedTime");
+    updateTableConfig(tableConfig);
+    String disableDictReloadId = reloadTableAndValidateResponse(getTableName(), TableType.REALTIME, false);
+    TestUtils.waitForCondition(aVoid -> {
+      try {
+        // Query result and total docs should not change during reload
+        JsonNode queryResponse1 = postQuery(query);
+        assertEquals(queryResponse1.get("resultTable").get("rows").get(0).get(0).asLong(), queryResult);
+        assertEquals(queryResponse1.get("totalDocs").asLong(), numTotalDocs);
+        return isReloadJobCompleted(disableDictReloadId);
+      } catch (Exception e) {
+        throw new RuntimeException(e);
+      }
+    }, 60_000L, "Failed to remove dictionary and inverted index");
+    // Should get back to full table scan
+    queryResponse = postQuery(query);
+    assertEquals(queryResponse.get("numEntriesScannedInFilter").asLong(), numTotalDocs);
+  }
+
   @Test
   public void testReset()
       throws Exception {
