@@ -19,7 +19,6 @@
 package org.apache.pinot.integration.tests;
 
 import com.fasterxml.jackson.databind.JsonNode;
-import com.google.common.collect.Lists;
 import java.io.File;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -80,11 +79,14 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         .addMultiValueDimension(INT_KEY_MAP_FIELD_NAME + SchemaUtils.MAP_VALUE_COLUMN_SUFFIX, DataType.INT)
         .addSingleValueDimension(STRING_KEY_MAP_STR_FIELD_NAME, DataType.STRING)
         .addSingleValueDimension(INT_KEY_MAP_STR_FIELD_NAME, DataType.STRING).build();
-    List<TransformConfig> transformConfigs = Lists.newArrayList(
+    List<TransformConfig> transformConfigs = Arrays.asList(
         new TransformConfig(STRING_KEY_MAP_STR_FIELD_NAME, "toJsonMapStr(" + STRING_KEY_MAP_FIELD_NAME + ")"),
         new TransformConfig(INT_KEY_MAP_STR_FIELD_NAME, "toJsonMapStr(" + INT_KEY_MAP_FIELD_NAME + ")"));
-    TableConfig tableConfig = new TableConfigBuilder(TableType.OFFLINE).setTableName(rawTableName)
-        .setIngestionConfig(new IngestionConfig(null, null, null, transformConfigs, null)).build();
+    IngestionConfig ingestionConfig = new IngestionConfig();
+    ingestionConfig.setTransformConfigs(transformConfigs);
+    TableConfig tableConfig =
+        new TableConfigBuilder(TableType.OFFLINE).setTableName(rawTableName).setIngestionConfig(ingestionConfig)
+            .build();
     addTableConfig(tableConfig);
 
     // Create and upload segments
@@ -134,26 +136,26 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
     String query = "SELECT stringKeyMapStr FROM " + getTableName();
     JsonNode pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    JsonNode rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(selectionResults.get(i).get(0).textValue(), String.format("{\"k1\":%d,\"k2\":100%d}", i, i));
+      assertEquals(rows.get(i).get(0).textValue(), String.format("{\"k1\":%d,\"k2\":100%d}", i, i));
     }
     query = "SELECT jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT') FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
     }
     query = "SELECT jsonExtractScalar(intKeyMapStr, '$.95', 'INT') FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
     }
 
     // Selection order-by
@@ -161,55 +163,55 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " ORDER BY jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT')";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), NUM_DOCS + i);
     }
     query = "SELECT jsonExtractScalar(intKeyMapStr, '$.717', 'INT') FROM " + getTableName()
         + " ORDER BY jsonExtractScalar(intKeyMapStr, '$.95', 'INT')";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), NUM_DOCS + i);
     }
 
     // Aggregation only
     query = "SELECT MAX(jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT')) FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode aggregationResult = pinotResponse.get("aggregationResults").get(0).get("value");
-    assertEquals((int) Double.parseDouble(aggregationResult.textValue()), NUM_DOCS - 1);
+    JsonNode aggregationResult = pinotResponse.get("resultTable").get("rows").get(0).get(0);
+    assertEquals(aggregationResult.intValue(), NUM_DOCS - 1);
     query = "SELECT MAX(jsonExtractScalar(intKeyMapStr, '$.95', 'INT')) FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    aggregationResult = pinotResponse.get("aggregationResults").get(0).get("value");
-    assertEquals((int) Double.parseDouble(aggregationResult.textValue()), NUM_DOCS - 1);
+    aggregationResult = pinotResponse.get("resultTable").get("rows").get(0).get(0);
+    assertEquals(aggregationResult.intValue(), NUM_DOCS - 1);
 
     // Aggregation group-by
-    query = "SELECT MIN(jsonExtractScalar(stringKeyMapStr, '$.k2', 'INT')) FROM " + getTableName()
-        + " GROUP BY jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT')";
+    query = "SELECT jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT') AS key, "
+        + "MIN(jsonExtractScalar(stringKeyMapStr, '$.k2', 'INT')) AS value FROM " + getTableName()
+        + " GROUP BY key ORDER BY value";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode groupByResults = pinotResponse.get("aggregationResults").get(0).get("groupByResult");
-    assertEquals(groupByResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      JsonNode groupByResult = groupByResults.get(i);
-      assertEquals(Integer.parseInt(groupByResult.get("group").get(0).asText()), i);
-      assertEquals((int) Double.parseDouble(groupByResult.get("value").asText()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
+      assertEquals(rows.get(i).get(1).intValue(), NUM_DOCS + i);
     }
-    query = "SELECT MIN(jsonExtractScalar(intKeyMapStr, '$.717', 'INT')) FROM " + getTableName()
-        + " GROUP BY jsonExtractScalar(intKeyMapStr, '$.95', 'INT')";
+    query = "SELECT jsonExtractScalar(intKeyMapStr, '$.95', 'INT') AS key, "
+        + "MIN(jsonExtractScalar(intKeyMapStr, '$.717', 'INT')) AS value FROM " + getTableName()
+        + " GROUP BY key ORDER BY value";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    groupByResults = pinotResponse.get("aggregationResults").get(0).get("groupByResult");
-    assertEquals(groupByResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      JsonNode groupByResult = groupByResults.get(i);
-      assertEquals(Integer.parseInt(groupByResult.get("group").get(0).asText()), i);
-      assertEquals((int) Double.parseDouble(groupByResult.get("value").asText()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
+      assertEquals(rows.get(i).get(1).intValue(), NUM_DOCS + i);
     }
 
     // Filter
@@ -217,30 +219,30 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " WHERE jsonExtractScalar(stringKeyMapStr, '$.k1', 'INT') = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 1);
-    assertEquals(Integer.parseInt(selectionResults.get(0).get(0).textValue()), NUM_DOCS + 25);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    assertEquals(rows.get(0).get(0).intValue(), NUM_DOCS + 25);
     query = "SELECT jsonExtractScalar(intKeyMapStr, '$.717', 'INT') FROM " + getTableName()
         + " WHERE jsonExtractScalar(intKeyMapStr, '$.95', 'INT') = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 1);
-    assertEquals(Integer.parseInt(selectionResults.get(0).get(0).textValue()), NUM_DOCS + 25);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    assertEquals(rows.get(0).get(0).intValue(), NUM_DOCS + 25);
 
     // Filter on non-existing key
     query = "SELECT jsonExtractScalar(stringKeyMapStr, '$.k2', 'INT') FROM " + getTableName()
         + " WHERE jsonExtractScalar(stringKeyMapStr, '$.k3', 'INT_ARRAY') = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
     query = "SELECT jsonExtractScalar(intKeyMapStr, '$.717', 'INT') FROM " + getTableName()
         + " WHERE jsonExtractScalar(intKeyMapStr, '$.123', 'INT_ARRAY') = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
 
     // Select non-existing key (illegal query)
     query = "SELECT jsonExtractScalar(stringKeyMapStr, '$.k3', 'INT') FROM " + getTableName();
@@ -263,14 +265,14 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " WHERE jsonExtractKey(intKeyMapStr, '$.*') = \"$['123']\"";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
     query = "SELECT jsonExtractScalar(stringKeyMapStr, '$.k3', 'INT') FROM " + getTableName()
         + " WHERE jsonExtractKey(stringKeyMapStr, '$.*') = \"$['k3']\"";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
   }
 
   @Test
@@ -280,18 +282,18 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
     String query = "SELECT mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES) FROM " + getTableName();
     JsonNode pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    JsonNode rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
     }
     query = "SELECT mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES) FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
     }
 
     // Selection order-by
@@ -299,55 +301,55 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " ORDER BY mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES)";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), NUM_DOCS + i);
     }
     query = "SELECT mapValue(intKeyMap__KEYS, 717, intKeyMap__VALUES) FROM " + getTableName()
         + " ORDER BY mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES)";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      assertEquals(Integer.parseInt(selectionResults.get(i).get(0).textValue()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), NUM_DOCS + i);
     }
 
     // Aggregation only
     query = "SELECT MAX(mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES)) FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode aggregationResult = pinotResponse.get("aggregationResults").get(0).get("value");
-    assertEquals((int) Double.parseDouble(aggregationResult.textValue()), NUM_DOCS - 1);
+    JsonNode aggregationResult = pinotResponse.get("resultTable").get("rows").get(0).get(0);
+    assertEquals(aggregationResult.intValue(), NUM_DOCS - 1);
     query = "SELECT MAX(mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES)) FROM " + getTableName();
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    aggregationResult = pinotResponse.get("aggregationResults").get(0).get("value");
-    assertEquals((int) Double.parseDouble(aggregationResult.textValue()), NUM_DOCS - 1);
+    aggregationResult = pinotResponse.get("resultTable").get("rows").get(0).get(0);
+    assertEquals(aggregationResult.intValue(), NUM_DOCS - 1);
 
     // Aggregation group-by
-    query = "SELECT MIN(mapValue(stringKeyMap__KEYS, 'k2', stringKeyMap__VALUES)) FROM " + getTableName()
-        + " GROUP BY mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES)";
+    query = "SELECT mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES) AS key, "
+        + "MIN(mapValue(stringKeyMap__KEYS, 'k2', stringKeyMap__VALUES)) AS value FROM " + getTableName()
+        + " GROUP BY key ORDER BY value";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    JsonNode groupByResults = pinotResponse.get("aggregationResults").get(0).get("groupByResult");
-    assertEquals(groupByResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      JsonNode groupByResult = groupByResults.get(i);
-      assertEquals(Integer.parseInt(groupByResult.get("group").get(0).asText()), i);
-      assertEquals((int) Double.parseDouble(groupByResult.get("value").asText()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
+      assertEquals(rows.get(i).get(1).intValue(), NUM_DOCS + i);
     }
-    query = "SELECT MIN(mapValue(intKeyMap__KEYS, 717, intKeyMap__VALUES)) FROM " + getTableName()
-        + " GROUP BY mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES)";
+    query = "SELECT mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES) AS key, "
+        + "MIN(mapValue(intKeyMap__KEYS, 717, intKeyMap__VALUES)) AS value FROM " + getTableName()
+        + " GROUP BY key ORDER BY value";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    groupByResults = pinotResponse.get("aggregationResults").get(0).get("groupByResult");
-    assertEquals(groupByResults.size(), 10);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 10);
     for (int i = 0; i < 10; i++) {
-      JsonNode groupByResult = groupByResults.get(i);
-      assertEquals(Integer.parseInt(groupByResult.get("group").get(0).asText()), i);
-      assertEquals((int) Double.parseDouble(groupByResult.get("value").asText()), NUM_DOCS + i);
+      assertEquals(rows.get(i).get(0).intValue(), i);
+      assertEquals(rows.get(i).get(1).intValue(), NUM_DOCS + i);
     }
 
     // Filter
@@ -355,30 +357,30 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " WHERE mapValue(stringKeyMap__KEYS, 'k1', stringKeyMap__VALUES) = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 1);
-    assertEquals(Integer.parseInt(selectionResults.get(0).get(0).textValue()), NUM_DOCS + 25);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    assertEquals(rows.get(0).get(0).intValue(), NUM_DOCS + 25);
     query = "SELECT mapValue(intKeyMap__KEYS, 717, intKeyMap__VALUES) FROM " + getTableName()
         + " WHERE mapValue(intKeyMap__KEYS, 95, intKeyMap__VALUES) = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 1);
-    assertEquals(Integer.parseInt(selectionResults.get(0).get(0).textValue()), NUM_DOCS + 25);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 1);
+    assertEquals(rows.get(0).get(0).intValue(), NUM_DOCS + 25);
 
     // Filter on non-existing key
     query = "SELECT mapValue(stringKeyMap__KEYS, 'k2', stringKeyMap__VALUES) FROM " + getTableName()
         + " WHERE mapValue(stringKeyMap__KEYS, 'k3', stringKeyMap__VALUES) = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
     query = "SELECT mapValue(intKeyMap__KEYS, 717, intKeyMap__VALUES) FROM " + getTableName()
         + " WHERE mapValue(intKeyMap__KEYS, 123, intKeyMap__VALUES) = 25";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
 
     // Select non-existing key (illegal query)
     query = "SELECT mapValue(stringKeyMap__KEYS, 'k3', stringKeyMap__VALUES) FROM " + getTableName();
@@ -393,14 +395,14 @@ public class MapTypeClusterIntegrationTest extends BaseClusterIntegrationTest {
         + " WHERE stringKeyMap__KEYS = 'k3'";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
     query = "SELECT mapValue(intKeyMap__KEYS, 123, intKeyMap__VALUES) FROM " + getTableName()
         + " WHERE stringKeyMap__KEYS = 123";
     pinotResponse = postQuery(query);
     assertEquals(pinotResponse.get("exceptions").size(), 0);
-    selectionResults = pinotResponse.get("selectionResults").get("results");
-    assertEquals(selectionResults.size(), 0);
+    rows = pinotResponse.get("resultTable").get("rows");
+    assertEquals(rows.size(), 0);
   }
 
   @AfterClass
