@@ -242,11 +242,6 @@ public class TlsIntegrationTest extends BaseClusterIntegrationTest {
     return new TableTaskConfig(Collections.singletonMap(MinionConstants.RealtimeToOfflineSegmentsTask.TASK_TYPE, prop));
   }
 
-  @Override
-  protected boolean useLlc() {
-    return true;
-  }
-
   @Override
   public void addSchema(Schema schema)
       throws IOException {
