@@ -37,6 +37,7 @@ import org.testng.annotations.BeforeClass;
 import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
+import static org.testng.Assert.assertFalse;
 
 
 public class UpsertTableSegmentUploadIntegrationTest extends BaseClusterIntegrationTestSet {
@@ -87,7 +88,24 @@ public class UpsertTableSegmentUploadIntegrationTest extends BaseClusterIntegrat
   @AfterClass
   public void tearDown()
       throws IOException {
-    dropRealtimeTable(getTableName());
+    String realtimeTableName = TableNameBuilder.REALTIME.tableNameWithType(getTableName());
+
+    // Test dropping all segments one by one
+    List<String> segments = listSegments(realtimeTableName);
+    assertFalse(segments.isEmpty());
+    for (String segment : segments) {
+      dropSegment(realtimeTableName, segment);
+    }
+    // NOTE: There is a delay to remove the segment from property store
+    TestUtils.waitForCondition((aVoid) -> {
+      try {
+        return listSegments(realtimeTableName).isEmpty();
+      } catch (IOException e) {
+        throw new RuntimeException(e);
+      }
+    }, 60_000L, "Failed to drop the segments");
+
+    dropRealtimeTable(realtimeTableName);
     stopServer();
     stopBroker();
     stopController();
@@ -111,11 +129,6 @@ public class UpsertTableSegmentUploadIntegrationTest extends BaseClusterIntegrat
     return "upsert_test.tar.gz";
   }
 
-  @Override
-  protected boolean useLlc() {
-    return true;
-  }
-
   @Override
   protected String getPartitionColumn() {
     return PRIMARY_KEY_COL;
