@@ -20,7 +20,6 @@ package org.apache.pinot.plugin.ingestion.batch.common;
 
 import com.fasterxml.jackson.databind.JsonNode;
 import java.io.Serializable;
-import java.net.URISyntaxException;
 import java.util.HashMap;
 import java.util.Map;
 import org.apache.pinot.segment.local.segment.creator.impl.SegmentIndexCreationDriverImpl;
@@ -47,11 +46,6 @@ import org.apache.pinot.spi.utils.JsonUtils;
 @SuppressWarnings("serial")
 public class SegmentGenerationTaskRunner implements Serializable {
 
-  public static final String FIXED_SEGMENT_NAME_GENERATOR = "fixed";
-  public static final String SIMPLE_SEGMENT_NAME_GENERATOR = "simple";
-  public static final String NORMALIZED_DATE_SEGMENT_NAME_GENERATOR = "normalizedDate";
-  public static final String INPUT_FILE_SEGMENT_NAME_GENERATOR = "inputFile";
-
   // For FixedSegmentNameGenerator
   public static final String SEGMENT_NAME = "segment.name";
 
@@ -98,14 +92,10 @@ public class SegmentGenerationTaskRunner implements Serializable {
       recordReaderConfig = (RecordReaderConfig) JsonUtils.jsonNodeToObject(jsonNode, clazz);
     }
 
-    //init segmentName Generator
-    SegmentNameGenerator segmentNameGenerator = getSegmentNameGenerator();
-
     //init segment generation config
     SegmentGeneratorConfig segmentGeneratorConfig = new SegmentGeneratorConfig(tableConfig, schema);
     segmentGeneratorConfig.setTableName(tableName);
     segmentGeneratorConfig.setOutDir(_taskSpec.getOutputDirectoryPath());
-    segmentGeneratorConfig.setSegmentNameGenerator(segmentNameGenerator);
     segmentGeneratorConfig.setSequenceId(_taskSpec.getSequenceId());
     segmentGeneratorConfig.setReaderConfig(recordReaderConfig);
     segmentGeneratorConfig.setRecordReaderPath(_taskSpec.getRecordReaderSpec().getClassName());
@@ -113,6 +103,10 @@ public class SegmentGenerationTaskRunner implements Serializable {
     segmentGeneratorConfig.setCustomProperties(_taskSpec.getCustomProperties());
     segmentGeneratorConfig.setFailOnEmptySegment(_taskSpec.isFailOnEmptySegment());
 
+    //init segmentName Generator
+    SegmentNameGenerator segmentNameGenerator = getSegmentNameGenerator(segmentGeneratorConfig);
+    segmentGeneratorConfig.setSegmentNameGenerator(segmentNameGenerator);
+
     //build segment
     SegmentIndexCreationDriverImpl segmentIndexCreationDriver = new SegmentIndexCreationDriverImpl();
     segmentIndexCreationDriver.init(segmentGeneratorConfig);
@@ -120,8 +114,7 @@ public class SegmentGenerationTaskRunner implements Serializable {
     return segmentIndexCreationDriver.getSegmentName();
   }
 
-  private SegmentNameGenerator getSegmentNameGenerator()
-      throws URISyntaxException {
+  private SegmentNameGenerator getSegmentNameGenerator(SegmentGeneratorConfig segmentGeneratorConfig) {
     TableConfig tableConfig = _taskSpec.getTableConfig();
     String tableName = tableConfig.getTableName();
 
@@ -132,7 +125,7 @@ public class SegmentGenerationTaskRunner implements Serializable {
     }
     String segmentNameGeneratorType = segmentNameGeneratorSpec.getType();
     if (segmentNameGeneratorType == null) {
-      segmentNameGeneratorType = SIMPLE_SEGMENT_NAME_GENERATOR;
+      segmentNameGeneratorType = segmentGeneratorConfig.inferSegmentNameGeneratorType();
     }
     Map<String, String> segmentNameGeneratorConfigs = segmentNameGeneratorSpec.getConfigs();
     if (segmentNameGeneratorConfigs == null) {
@@ -143,12 +136,12 @@ public class SegmentGenerationTaskRunner implements Serializable {
         Boolean.parseBoolean(segmentNameGeneratorConfigs.get(APPEND_UUID_TO_SEGMENT_NAME));
 
     switch (segmentNameGeneratorType) {
-      case FIXED_SEGMENT_NAME_GENERATOR:
+      case BatchConfigProperties.SegmentNameGeneratorType.FIXED:
         return new FixedSegmentNameGenerator(segmentNameGeneratorConfigs.get(SEGMENT_NAME));
-      case SIMPLE_SEGMENT_NAME_GENERATOR:
+      case BatchConfigProperties.SegmentNameGeneratorType.SIMPLE:
         return new SimpleSegmentNameGenerator(tableName, segmentNameGeneratorConfigs.get(SEGMENT_NAME_POSTFIX),
             appendUUIDToSegmentName);
-      case NORMALIZED_DATE_SEGMENT_NAME_GENERATOR:
+      case BatchConfigProperties.SegmentNameGeneratorType.NORMALIZED_DATE:
         SegmentsValidationAndRetentionConfig validationConfig = tableConfig.getValidationConfig();
         DateTimeFormatSpec dateTimeFormatSpec = null;
         String timeColumnName = validationConfig.getTimeColumnName();
@@ -165,7 +158,7 @@ public class SegmentGenerationTaskRunner implements Serializable {
             IngestionConfigUtils.getBatchSegmentIngestionFrequency(tableConfig), dateTimeFormatSpec,
             segmentNameGeneratorConfigs.get(SEGMENT_NAME_POSTFIX),
             appendUUIDToSegmentName);
-      case INPUT_FILE_SEGMENT_NAME_GENERATOR:
+      case BatchConfigProperties.SegmentNameGeneratorType.INPUT_FILE:
         String inputFileUri = _taskSpec.getCustomProperty(BatchConfigProperties.INPUT_DATA_FILE_URI_KEY);
         return new InputFileSegmentNameGenerator(segmentNameGeneratorConfigs.get(FILE_PATH_PATTERN),
             segmentNameGeneratorConfigs.get(SEGMENT_NAME_TEMPLATE),
