@@ -21,14 +21,11 @@ package org.apache.pinot.plugin.filesystem.test;
 import com.azure.core.http.rest.PagedIterable;
 import com.azure.core.http.rest.SimpleResponse;
 import com.azure.core.util.Context;
-import com.azure.storage.blob.BlobClient;
-import com.azure.storage.blob.BlobContainerClient;
-import com.azure.storage.blob.BlobServiceClient;
-import com.azure.storage.blob.specialized.BlobInputStream;
 import com.azure.storage.file.datalake.DataLakeDirectoryClient;
 import com.azure.storage.file.datalake.DataLakeFileClient;
 import com.azure.storage.file.datalake.DataLakeFileSystemClient;
 import com.azure.storage.file.datalake.DataLakeServiceClient;
+import com.azure.storage.file.datalake.models.DataLakeFileOpenInputStreamResult;
 import com.azure.storage.file.datalake.models.DataLakeStorageException;
 import com.azure.storage.file.datalake.models.PathItem;
 import com.azure.storage.file.datalake.models.PathProperties;
@@ -36,10 +33,16 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.URI;
 import java.net.URISyntaxException;
-import java.util.Collections;
+import java.time.Instant;
+import java.time.OffsetDateTime;
+import java.time.ZoneOffset;
 import java.util.HashMap;
+import java.util.List;
+import java.util.stream.Stream;
 import org.apache.pinot.plugin.filesystem.ADLSGen2PinotFS;
+import org.apache.pinot.plugin.filesystem.AzurePinotFSUtil;
 import org.apache.pinot.spi.env.PinotConfiguration;
+import org.apache.pinot.spi.filesystem.FileMetadata;
 import org.mockito.Mock;
 import org.mockito.MockitoAnnotations;
 import org.testng.Assert;
@@ -60,18 +63,14 @@ public class ADLSGen2PinotFSTest {
   @Mock
   private DataLakeDirectoryClient _mockDirectoryClient;
   @Mock
-  private BlobServiceClient _mockBlobServiceClient;
-  @Mock
-  private BlobContainerClient _mockBlobContainerClient;
-  @Mock
-  private BlobClient _mockBlobClient;
-  @Mock
-  private BlobInputStream _mockBlobInputStream;
-  @Mock
   private DataLakeServiceClient _mockServiceClient;
   @Mock
   private DataLakeFileClient _mockFileClient;
   @Mock
+  private DataLakeFileOpenInputStreamResult _mockFileOpenInputStreamResult;
+  @Mock
+  private InputStream _mockInputStream;
+  @Mock
   private DataLakeStorageException _mockDataLakeStorageException;
   @Mock
   private SimpleResponse _mockSimpleResponse;
@@ -90,8 +89,8 @@ public class ADLSGen2PinotFSTest {
   @BeforeMethod
   public void setup()
       throws URISyntaxException {
-    MockitoAnnotations.initMocks(this);
-    _adlsGen2PinotFsUnderTest = new ADLSGen2PinotFS(_mockFileSystemClient, _mockBlobServiceClient);
+    MockitoAnnotations.openMocks(this);
+    _adlsGen2PinotFsUnderTest = new ADLSGen2PinotFS(_mockFileSystemClient);
     _mockURI = new URI("mock://mock");
   }
 
@@ -99,7 +98,7 @@ public class ADLSGen2PinotFSTest {
   public void tearDown() {
     verifyNoMoreInteractions(_mockDataLakeStorageException, _mockServiceClient, _mockFileSystemClient,
         _mockSimpleResponse, _mockDirectoryClient, _mockPathItem, _mockPagedIterable, _mockPathProperties,
-        _mockFileClient, _mockBlobContainerClient, _mockBlobClient, _mockBlobServiceClient, _mockBlobInputStream);
+        _mockFileClient, _mockFileOpenInputStreamResult, _mockInputStream);
   }
 
   @Test(expectedExceptions = NullPointerException.class)
@@ -190,7 +189,7 @@ public class ADLSGen2PinotFSTest {
   public void testListFiles()
       throws IOException {
     when(_mockFileSystemClient.listPaths(any(), any())).thenReturn(_mockPagedIterable);
-    when(_mockPagedIterable.stream()).thenReturn(Collections.singletonList(_mockPathItem).stream());
+    when(_mockPagedIterable.stream()).thenReturn(Stream.of(_mockPathItem));
     when(_mockPathItem.getName()).thenReturn("foo");
 
     String[] actual = _adlsGen2PinotFsUnderTest.listFiles(_mockURI, true);
@@ -201,6 +200,49 @@ public class ADLSGen2PinotFSTest {
     verify(_mockPathItem).getName();
   }
 
+  @Test
+  public void testListFilesWithMetadata()
+      throws IOException {
+    when(_mockFileSystemClient.listPaths(any(), any())).thenReturn(_mockPagedIterable);
+    when(_mockPagedIterable.stream()).thenReturn(Stream.of(_mockPathItem));
+    when(_mockPathItem.getName()).thenReturn("foo");
+    when(_mockPathItem.isDirectory()).thenReturn(false);
+    when(_mockPathItem.getContentLength()).thenReturn(1024L);
+    OffsetDateTime mtime = OffsetDateTime.now();
+    when(_mockPathItem.getLastModified()).thenReturn(mtime);
+
+    List<FileMetadata> actual = _adlsGen2PinotFsUnderTest.listFilesWithMetadata(_mockURI, true);
+    FileMetadata fm = actual.get(0);
+    Assert.assertEquals(fm.getFilePath(), "/foo");
+    Assert.assertFalse(fm.isDirectory());
+    Assert.assertEquals(fm.getLength(), 1024);
+    Assert.assertEquals(fm.getLastModifiedTime(), mtime.toInstant().toEpochMilli());
+
+    verify(_mockFileSystemClient).listPaths(any(), any());
+    verify(_mockPagedIterable).stream();
+    verify(_mockPathItem).getName();
+    verify(_mockPathItem).isDirectory();
+    verify(_mockPathItem).getContentLength();
+    verify(_mockPathItem).getLastModified();
+  }
+
+  @Test
+  public void testLastModified()
+      throws IOException {
+    when(_mockFileSystemClient.getDirectoryClient(any())).thenReturn(_mockDirectoryClient);
+    when(_mockDirectoryClient.getProperties()).thenReturn(_mockPathProperties);
+    Instant now = Instant.now();
+    OffsetDateTime mtime = Instant.now().atOffset(ZoneOffset.UTC);
+    when(_mockPathProperties.getLastModified()).thenReturn(mtime);
+
+    long actual = _adlsGen2PinotFsUnderTest.lastModified(_mockURI);
+    Assert.assertEquals(actual, now.toEpochMilli());
+
+    verify(_mockFileSystemClient).getDirectoryClient(any());
+    verify(_mockDirectoryClient).getProperties();
+    verify(_mockPathProperties).getLastModified();
+  }
+
   @Test
   public void testListFilesException() {
     when(_mockFileSystemClient.listPaths(any(), any())).thenThrow(_mockDataLakeStorageException);
@@ -220,7 +262,7 @@ public class ADLSGen2PinotFSTest {
     when(_mockDirectoryClient.getProperties()).thenReturn(_mockPathProperties);
     when(_mockPathProperties.getMetadata()).thenReturn(metadata);
     when(_mockFileSystemClient.listPaths(any(), any())).thenReturn(_mockPagedIterable);
-    when(_mockPagedIterable.stream()).thenReturn(Collections.singletonList(_mockPathItem).stream());
+    when(_mockPagedIterable.stream()).thenReturn(Stream.of(_mockPathItem));
     when(_mockPathItem.getName()).thenReturn("foo");
     when(_mockFileSystemClient.deleteDirectoryWithResponse(eq(""), eq(true), eq(null), eq(null), eq(Context.NONE)))
         .thenReturn(_mockSimpleResponse);
@@ -310,8 +352,7 @@ public class ADLSGen2PinotFSTest {
   }
 
   @Test
-  public void testExistsException()
-      throws IOException {
+  public void testExistsException() {
     when(_mockFileSystemClient.getDirectoryClient(any())).thenReturn(_mockDirectoryClient);
     when(_mockDirectoryClient.getProperties()).thenThrow(_mockDataLakeStorageException);
     when(_mockDataLakeStorageException.getStatusCode()).thenReturn(123);
@@ -393,17 +434,15 @@ public class ADLSGen2PinotFSTest {
   @Test
   public void open()
       throws IOException {
-    when(_mockFileSystemClient.getFileSystemName()).thenReturn(MOCK_FILE_SYSTEM_NAME);
-    when(_mockBlobServiceClient.getBlobContainerClient(MOCK_FILE_SYSTEM_NAME)).thenReturn(_mockBlobContainerClient);
-    when(_mockBlobContainerClient.getBlobClient(any())).thenReturn(_mockBlobClient);
-    when(_mockBlobClient.openInputStream()).thenReturn(_mockBlobInputStream);
+    when(_mockFileSystemClient.getFileClient(any())).thenReturn(_mockFileClient);
+    when(_mockFileClient.openInputStream()).thenReturn(_mockFileOpenInputStreamResult);
+    when(_mockFileOpenInputStreamResult.getInputStream()).thenReturn(_mockInputStream);
 
     InputStream actual = _adlsGen2PinotFsUnderTest.open(_mockURI);
-    Assert.assertEquals(actual, _mockBlobInputStream);
+    Assert.assertEquals(actual, _mockInputStream);
 
-    verify(_mockFileSystemClient).getFileSystemName();
-    verify(_mockBlobServiceClient).getBlobContainerClient(MOCK_FILE_SYSTEM_NAME);
-    verify(_mockBlobContainerClient).getBlobClient(any());
-    verify(_mockBlobClient).openInputStream();
+    verify(_mockFileSystemClient).getFileClient(AzurePinotFSUtil.convertUriToUrlEncodedAzureStylePath(_mockURI));
+    verify(_mockFileClient).openInputStream();
+    verify(_mockFileOpenInputStreamResult).getInputStream();
   }
 }
