@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.plugin.minion.tasks;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import java.io.File;
 import java.util.ArrayList;
@@ -38,6 +39,8 @@ import org.apache.pinot.common.utils.TarGzCompressionUtils;
 import org.apache.pinot.common.utils.fetcher.SegmentFetcherFactory;
 import org.apache.pinot.core.common.MinionConstants;
 import org.apache.pinot.core.minion.PinotTaskConfig;
+import org.apache.pinot.minion.event.MinionEventObserver;
+import org.apache.pinot.minion.event.MinionEventObservers;
 import org.apache.pinot.minion.exception.TaskCancelledException;
 import org.apache.pinot.spi.auth.AuthProvider;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
@@ -54,6 +57,10 @@ import org.slf4j.LoggerFactory;
 public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecutor {
   private static final Logger LOGGER = LoggerFactory.getLogger(BaseSingleSegmentConversionExecutor.class);
 
+  // Tracking finer grained progress status.
+  protected PinotTaskConfig _pinotTaskConfig;
+  protected MinionEventObserver _eventObserver;
+
   /**
    * Converts the segment based on the given task config and returns the conversion result.
    */
@@ -63,6 +70,8 @@ public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecut
   @Override
   public SegmentConversionResult executeTask(PinotTaskConfig pinotTaskConfig)
       throws Exception {
+    _pinotTaskConfig = pinotTaskConfig;
+    _eventObserver = MinionEventObservers.getInstance().getMinionEventObserver(pinotTaskConfig.getTaskId());
     String taskType = pinotTaskConfig.getTaskType();
     Map<String, String> configs = pinotTaskConfig.getConfigs();
     String tableNameWithType = configs.get(MinionConstants.TABLE_NAME_KEY);
@@ -89,11 +98,13 @@ public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecut
 
     try {
       // Download the tarred segment file
+      _eventObserver.notifyProgress(_pinotTaskConfig, "Downloading segment from: " + downloadURL);
       File tarredSegmentFile = new File(tempDataDir, "tarredSegment");
       LOGGER.info("Downloading segment from {} to {}", downloadURL, tarredSegmentFile.getAbsolutePath());
       SegmentFetcherFactory.fetchAndDecryptSegmentToLocal(downloadURL, tarredSegmentFile, crypterName);
 
       // Un-tar the segment file
+      _eventObserver.notifyProgress(_pinotTaskConfig, "Decompressing segment from: " + downloadURL);
       File segmentDir = new File(tempDataDir, "segmentDir");
       File indexDir = TarGzCompressionUtils.untar(tarredSegmentFile, segmentDir).get(0);
       if (!FileUtils.deleteQuietly(tarredSegmentFile)) {
@@ -109,6 +120,7 @@ public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecut
           segmentConversionResult.getSegmentName(), segmentName);
 
       // Tar the converted segment
+      _eventObserver.notifyProgress(_pinotTaskConfig, "Compressing segment: " + segmentName);
       File convertedSegmentDir = segmentConversionResult.getFile();
       File convertedTarredSegmentFile =
           new File(tempDataDir, segmentName + TarGzCompressionUtils.TAR_GZ_FILE_EXTENSION);
@@ -164,6 +176,7 @@ public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecut
           tableTypeParameter);
 
       // Upload the tarred segment
+      _eventObserver.notifyProgress(_pinotTaskConfig, "Uploading segment: " + segmentName);
       SegmentConversionUtils.uploadSegment(configs, httpHeaders, parameters, tableNameWithType, segmentName, uploadURL,
           convertedTarredSegmentFile);
       if (!FileUtils.deleteQuietly(convertedTarredSegmentFile)) {
@@ -176,4 +189,10 @@ public abstract class BaseSingleSegmentConversionExecutor extends BaseTaskExecut
       FileUtils.deleteQuietly(tempDataDir);
     }
   }
+
+  // For tests only.
+  @VisibleForTesting
+  public void setMinionEventObserver(MinionEventObserver observer) {
+    _eventObserver = observer;
+  }
 }
