@@ -20,6 +20,7 @@ package org.apache.pinot.segment.local.upsert;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import org.apache.pinot.common.metrics.ServerMetrics;
@@ -30,15 +31,20 @@ import org.apache.pinot.segment.local.upsert.ConcurrentMapPartitionUpsertMetadat
 import org.apache.pinot.segment.local.utils.HashUtils;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.MutableSegment;
+import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.segment.spi.index.mutable.ThreadSafeMutableRoaringBitmap;
+import org.apache.pinot.segment.spi.index.reader.ForwardIndexReader;
 import org.apache.pinot.spi.config.table.HashFunction;
+import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.data.readers.PrimaryKey;
 import org.apache.pinot.spi.utils.ByteArray;
 import org.apache.pinot.spi.utils.BytesUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
+import org.roaringbitmap.buffer.MutableRoaringBitmap;
 import org.testng.annotations.Test;
 
+import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyString;
 import static org.mockito.Mockito.mock;
@@ -55,15 +61,18 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
 
   @Test
   public void testAddReplaceRemoveSegment() {
-    verifyAddReplaceRemoveSegment(HashFunction.NONE);
-    verifyAddReplaceRemoveSegment(HashFunction.MD5);
-    verifyAddReplaceRemoveSegment(HashFunction.MURMUR3);
+    verifyAddReplaceRemoveSegment(HashFunction.NONE, false);
+    verifyAddReplaceRemoveSegment(HashFunction.MD5, false);
+    verifyAddReplaceRemoveSegment(HashFunction.MURMUR3, false);
+    verifyAddReplaceRemoveSegment(HashFunction.NONE, true);
+    verifyAddReplaceRemoveSegment(HashFunction.MD5, true);
+    verifyAddReplaceRemoveSegment(HashFunction.MURMUR3, true);
   }
 
-  private void verifyAddReplaceRemoveSegment(HashFunction hashFunction) {
+  private void verifyAddReplaceRemoveSegment(HashFunction hashFunction, boolean enableSnapshot) {
     ConcurrentMapPartitionUpsertMetadataManager upsertMetadataManager =
         new ConcurrentMapPartitionUpsertMetadataManager(REALTIME_TABLE_NAME, 0, Collections.singletonList("pk"),
-            "timeCol", hashFunction, null, mock(ServerMetrics.class));
+            "timeCol", hashFunction, null, false, mock(ServerMetrics.class));
     Map<Object, RecordLocation> recordLocationMap = upsertMetadataManager._primaryKeyToRecordLocationMap;
 
     // Add the first segment
@@ -73,7 +82,18 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
     ThreadSafeMutableRoaringBitmap validDocIds1 = new ThreadSafeMutableRoaringBitmap();
     List<PrimaryKey> primaryKeys1 = getPrimaryKeyList(numRecords, primaryKeys);
     ImmutableSegmentImpl segment1 = mockImmutableSegment(1, validDocIds1, primaryKeys1);
-    List<RecordInfo> recordInfoList1 = getRecordInfoList(numRecords, primaryKeys, timestamps);
+    List<RecordInfo> recordInfoList1;
+    if (enableSnapshot) {
+      // get recordInfo from validDocIdSnapshot.
+      // segment1 snapshot: 0 -> {5, 100}, 1 -> {4, 120}, 2 -> {2, 100}
+      int[] docIds1 = new int[]{2, 4, 5};
+      MutableRoaringBitmap validDocIdsSnapshot1 = new MutableRoaringBitmap();
+      validDocIdsSnapshot1.add(docIds1);
+      recordInfoList1 = getRecordInfoList(validDocIdsSnapshot1, primaryKeys, timestamps);
+    } else {
+      // get recordInfo by iterating all records.
+      recordInfoList1 = getRecordInfoList(numRecords, primaryKeys, timestamps);
+    }
     upsertMetadataManager.addSegment(segment1, validDocIds1, recordInfoList1.iterator());
     // segment1: 0 -> {5, 100}, 1 -> {4, 120}, 2 -> {2, 100}
     assertEquals(recordLocationMap.size(), 3);
@@ -88,8 +108,20 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
     timestamps = new int[]{100, 100, 120, 80, 80};
     ThreadSafeMutableRoaringBitmap validDocIds2 = new ThreadSafeMutableRoaringBitmap();
     ImmutableSegmentImpl segment2 = mockImmutableSegment(2, validDocIds2, getPrimaryKeyList(numRecords, primaryKeys));
-    upsertMetadataManager.addSegment(segment2, validDocIds2,
-        getRecordInfoList(numRecords, primaryKeys, timestamps).iterator());
+    List<RecordInfo> recordInfoList2;
+    if (enableSnapshot) {
+      // get recordInfo from validDocIdSnapshot.
+      // segment2 snapshot: 0 -> {0, 100}, 2 -> {2, 120}, 3 -> {3, 80}
+      // segment1 snapshot: 1 -> {4, 120}
+      MutableRoaringBitmap validDocIdsSnapshot2 = new MutableRoaringBitmap();
+      validDocIdsSnapshot2.add(0, 2, 3);
+      recordInfoList2 = getRecordInfoList(validDocIdsSnapshot2, primaryKeys, timestamps);
+    } else {
+      // get recordInfo by iterating all records.
+      recordInfoList2 = getRecordInfoList(numRecords, primaryKeys, timestamps);
+    }
+    upsertMetadataManager.addSegment(segment2, validDocIds2, recordInfoList2.iterator());
+
     // segment1: 1 -> {4, 120}
     // segment2: 0 -> {0, 100}, 2 -> {2, 120}, 3 -> {3, 80}
     assertEquals(recordLocationMap.size(), 4);
@@ -174,6 +206,18 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
     return recordInfoList;
   }
 
+  /**
+   * Get recordInfo from validDocIdsSnapshot (enabledSnapshot = True).
+   */
+  private List<RecordInfo> getRecordInfoList(MutableRoaringBitmap validDocIdsSnapshot, int[] primaryKeys,
+      int[] timestamps) {
+    List<RecordInfo> recordInfoList = new ArrayList<>();
+    Iterator<Integer> validDocIdsIterator = validDocIdsSnapshot.iterator();
+    validDocIdsIterator.forEachRemaining((docId) -> recordInfoList.add(
+        new RecordInfo(makePrimaryKey(primaryKeys[docId]), docId, new IntWrapper(timestamps[docId]))));
+    return recordInfoList;
+  }
+
   private List<PrimaryKey> getPrimaryKeyList(int numRecords, int[] primaryKeys) {
     List<PrimaryKey> primaryKeyList = new ArrayList<>();
     for (int i = 0; i < numRecords; i++) {
@@ -187,8 +231,14 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
     ImmutableSegmentImpl segment = mock(ImmutableSegmentImpl.class);
     when(segment.getSegmentName()).thenReturn(getSegmentName(sequenceNumber));
     when(segment.getValidDocIds()).thenReturn(validDocIds);
-    when(segment.getValue(anyInt(), anyString())).thenAnswer(
+    DataSource dataSource = mock(DataSource.class);
+    when(segment.getDataSource(anyString())).thenReturn(dataSource);
+    ForwardIndexReader forwardIndex = mock(ForwardIndexReader.class);
+    when(forwardIndex.isSingleValue()).thenReturn(true);
+    when(forwardIndex.getStoredType()).thenReturn(DataType.INT);
+    when(forwardIndex.getInt(anyInt(), any())).thenAnswer(
         invocation -> primaryKeys.get(invocation.getArgument(0)).getValues()[0]);
+    when(dataSource.getForwardIndex()).thenReturn(forwardIndex);
     return segment;
   }
 
@@ -233,7 +283,7 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
   private void verifyAddRecord(HashFunction hashFunction) {
     ConcurrentMapPartitionUpsertMetadataManager upsertMetadataManager =
         new ConcurrentMapPartitionUpsertMetadataManager(REALTIME_TABLE_NAME, 0, Collections.singletonList("pk"),
-            "timeCol", hashFunction, null, mock(ServerMetrics.class));
+            "timeCol", hashFunction, null, false, mock(ServerMetrics.class));
     Map<Object, RecordLocation> recordLocationMap = upsertMetadataManager._primaryKeyToRecordLocationMap;
 
     // Add the first segment
@@ -295,15 +345,15 @@ public class ConcurrentMapPartitionUpsertMetadataManagerTest {
   public void testHashPrimaryKey() {
     PrimaryKey pk = new PrimaryKey(new Object[]{"uuid-1", "uuid-2", "uuid-3"});
     assertEquals(BytesUtils.toHexString(((ByteArray) HashUtils.hashPrimaryKey(pk, HashFunction.MD5)).getBytes()),
-        "58de44997505014e02982846a4d1cbbd");
+        "6ca926be8c2d1d980acf48ba48418e24");
     assertEquals(BytesUtils.toHexString(((ByteArray) HashUtils.hashPrimaryKey(pk, HashFunction.MURMUR3)).getBytes()),
-        "7e6b4a98296292a4012225fff037fa8c");
+        "e4540494e43b27e312d01f33208c6a4e");
     // reorder
     pk = new PrimaryKey(new Object[]{"uuid-3", "uuid-2", "uuid-1"});
     assertEquals(BytesUtils.toHexString(((ByteArray) HashUtils.hashPrimaryKey(pk, HashFunction.MD5)).getBytes()),
-        "d2df12c6dea7b83f965613614eee58e2");
+        "fc2159b78d07f803fdfb0b727315a445");
     assertEquals(BytesUtils.toHexString(((ByteArray) HashUtils.hashPrimaryKey(pk, HashFunction.MURMUR3)).getBytes()),
-        "8d68b314cc0c8de4dbd55f4dad3c3e66");
+        "37fab5ef0ea39711feabcdc623cb8a4e");
   }
 
   /**
