@@ -488,6 +488,39 @@ public interface MutableForwardIndex extends ForwardIndexReader<ForwardIndexRead
     return getStringMV(docId);
   }
 
+  /**
+   * Reads the BYTES type multi-value at the given document id into the passed in value buffer (the buffer size must be
+   * enough to hold all the values for the multi-value entry) and returns the number of values within the multi-value
+   * entry.
+   *
+   * @param docId Document id
+   * @param valueBuffer Value buffer
+   * @return Number of values within the multi-value entry
+   */
+  default int getBytesMV(int docId, byte[][] valueBuffer) {
+    throw new UnsupportedOperationException();
+  }
+
+  /**
+   * Reads the BYTES type multi-value at the given document id into a buffer and returns the buffer.
+   *
+   * @param docId Document id
+   * @return A buffer containing the multi-value entries
+   */
+  default byte[][] getBytesMV(int docId) {
+    throw new UnsupportedOperationException();
+  }
+
+  @Override
+  default int getBytesMV(int docId, byte[][] valueBuffer, ForwardIndexReaderContext context) {
+    return getBytesMV(docId, valueBuffer);
+  }
+
+  @Override
+  default byte[][] getBytesMV(int docId, ForwardIndexReaderContext context) {
+    return getBytesMV(docId);
+  }
+
   /**
    * Gets the number of multi-values at a given document id and returns it.
    *
@@ -552,4 +585,14 @@ public interface MutableForwardIndex extends ForwardIndexReader<ForwardIndexRead
   default void setStringMV(int docId, String[] values) {
     throw new UnsupportedOperationException();
   }
+
+  /**
+   * Writes the BYTES type multi-value into the given document id.
+   *
+   * @param docId Document id
+   * @param values Values to write
+   */
+  default void setBytesMV(int docId, byte[][] values) {
+    throw new UnsupportedOperationException();
+  }
 }
