@@ -45,6 +45,12 @@ public class SegmentGenerationJobSpec implements Serializable {
    */
   private String _inputDirURI;
 
+  /**
+   * If true, search input files recursively from root directly specified in _inputDirURI.
+   */
+  // TODO: set the default value to false after all clients are aware of this.
+  private boolean _searchRecursively = true;
+
   /**
    * include file name pattern, supported glob pattern.
    * Sample usage:
@@ -161,6 +167,14 @@ public class SegmentGenerationJobSpec implements Serializable {
     _inputDirURI = inputDirURI;
   }
 
+  public boolean isSearchRecursively() {
+    return _searchRecursively;
+  }
+
+  public void setSearchRecursively(boolean searchRecursively) {
+    _searchRecursively = searchRecursively;
+  }
+
   public String getIncludeFileNamePattern() {
     return _includeFileNamePattern;
   }
